/**
 * Requirements
 */
var gulp = require('gulp'),
	angularFilesort = require('gulp-angular-filesort'),
	templateCache = require('gulp-angular-templatecache'),
	shell = require('gulp-shell')
minifyHtml = require('gulp-minify-html'),
concat = require('gulp-concat'),
runSequence = require('run-sequence'),
del = require('del'),
watch = require('gulp-watch');
	
/**
 * Configuration variables 
 */
var forceRebuildCordova = true,//if true, runs cordova build, otherwise copies the files, (iOS not working with false)
	copyMainHtml = true,//not yet implemented
	cordovaBuildPath = '../Connect.Local/',
	cordovaBuildSrcPath = '../Connect.Local/www',
	cordovaIOSPath = '../Connect.Local/www/',
	cordovaBrowserPath = '../Connect.Local/platforms/browser/www/',
	codeSrc = ['./*', './**', './**/*', './**/**/*', '!./node_modules/**', '!./node_modules', '!./node_modules/**/*', '!./bower_components/**', '!./bower_components', '!./bower_components/**/*',],
	devSrc = ['./dist/*', './dist/**/*'];

/*********************************************
Gulp Tasks
**********************************************/

/* The default task for dev builds */
gulp.task('default', ['clean'], function () {
	runSequence([
	//'libraryScripts',
		'appScripts',
		'templates'
	]);
});

gulp.task('dev', function () {
    watch(['js/app/**/*.js', 'js/app/*.js'], function () {
        runSequence('appScripts');
    }, 1500);
    watch(['js/app/**/*.html', 'js/app/*.html'], function () {
        runSequence('templates');
    }, 1500);
});

gulp.task('dev_build_ios', function () {
	runSequence(['appScripts', 'templates'], 'iOSCordova');
    watch(['js/app/**/*.js', 'js/app/*.js'], function () {
        runSequence(['appScripts'], 'iOSCordova');
    }, 500);
    watch(['js/app/**/*.html', 'js/app/*.html'], function () {
        runSequence(['templates'], 'iOSCordova');
    }, 500);
});

gulp.task('build_ios', function () {
	runSequence(['appScripts', 'templates'], 'copy_cordova_src', 'build_ios_cordova');
});

gulp.task('build_browser', function () {
	runSequence(['appScripts', 'templates'], 'copy_cordova_src', 'build_browser_cordova');
});

gulp.task('dev_build_browser', function () {
	runSequence(['appScripts', 'templates'], 'copy_cordova_browser_dist', 'copy_cordova_browser_main_index');
    watch(['js/app/**/*.js', 'js/app/*.js'], function () {
        runSequence(['appScripts'], 'copy_cordova_browser_dist');
    }, 500);
    watch(['main.html', 'index.html', 'js/app/**/*.html', 'js/app/*.html'], function () {
        runSequence(['templates'], 'copy_cordova_browser_dist', 'copy_cordova_browser_main_index');
    }, 500);
});

/*********************************************
Gulp Sub-Tasks
**********************************************/

//gulp application script task, get scripts we want and filesort them via angular file sort
//TODO include all scripts
gulp.task('appScripts', function () {
	return gulp.src([
		'!gulpfile.js',
		'!**/gulpfile.js',
		'js/app/*.js',
		'js/app/**/*.js',
		//'js/*.js',
	])
		.pipe(angularFilesort())
		.pipe(concat('app.js'))
		.pipe(gulp.dest('dist/js'));
});

//runs the cordova build
gulp.task('iOSCordova', function () {
	var cordovaCommands = [
	'echo "Working from:"',
		'pwd',
		'echo \n',
		'cordova build ios'
    ];
	var options = {
		cwd: cordovaBuildPath
	};
	if (forceRebuildCordova) {
		return gulp.src(codeSrc)
			.pipe(gulp.dest(cordovaBuildSrcPath))
			.pipe(shell(cordovaCommands, options));
	} else {
		return gulp.src(codeSrc)
			.pipe(gulp.dest(cordovaIOSPath));
	}

});

//copy the cordova files over
gulp.task('copy_cordova_src', function () {
		return gulp.src(codeSrc)
			.pipe(gulp.dest(cordovaBuildSrcPath));
});

gulp.task('copy_cordova_browser_dist', function() {
	return gulp.src(devSrc)
			.pipe(gulp.dest(cordovaBrowserPath + '/dist'));
});

gulp.task('copy_cordova_browser_main_index', function() {
	return gulp.src(['index.html', 'main.html'])
			.pipe(gulp.dest(cordovaBrowserPath));
});

gulp.task('build_ios_cordova', function () {
	var cordovaCommands = [
		'pwd',
		'cordova build ios'
    ];
	var options = {
		cwd: cordovaBuildPath
	};
	return gulp.src(['']).pipe(shell(cordovaCommands, options));
});

//runs the cordova build
gulp.task('browserCordova', function () {
	var cordovaCommands = [
		'pwd',
		'cordova build browser'
    ];
	var options = {
		cwd: cordovaBuildPath
	};
	if (forceRebuildCordova) {
		return gulp.src(codeSrc)
			.pipe(gulp.dest(cordovaBuildSrcPath))
			.pipe(shell(cordovaCommands, options));
	} else {
		return gulp.src(codeSrc)
			.pipe(gulp.dest(cordovaBrowserPath));
	}

});

//runs the cordova build
gulp.task('build_browser_cordova', function () {
	var cordovaCommands = [
		'cordova build browser'
    ];
	var options = {
		cwd: cordovaBuildPath
	};
		return gulp.src([]).pipe(shell(cordovaCommands, options));

});

//gulp library script task, get the library scripts and concatenate them together, TODO: move to bower / min versions for production
gulp.task('libraryScripts', function () {
	return gulp.src([
		'js/lib/jquery.js',
		'js/lib/moment.js',
		'js/lib/sha256.js',
		'js/lib/cordova.ios.js',
		'!js/lib/cordova.android.js',
		'!js/lib/cordova.wp8.js',
		'js/lib/*.js'
	])
		.pipe(concat('library.js'))
		.pipe(gulp.dest('dist/js'));
});

gulp.task('templates', function () {
	return gulp.src('js/app/**/*.html')
		.pipe(minifyHtml({
			quotes: true,
			empty: true,
			spare: true,
			conditionals: true,
			cdata: true
		}))
		.pipe(templateCache('templates.js', {
			standalone: true
		}))
		.pipe(gulp.dest('dist/js'));
});

/* gulp task to clean out the dist folder to remove any ghosts */
gulp.task('clean', function (cb) {
	return del([
		'dist/js'
	], cb);
});