
//--------------//
//  APP Start  //
//-------------//
(function(){

var ConnectApp = angular.module('ConnectApp', ['connect.app']); //ConnectApp is the name of the application according to Angular

//----------------------------------//
//          EVENT LISTENERS         //
//----------------------------------//
//It is necessary to wait for the device ready event before accessing any device API's (through phonegap plugins) be sure 
//to bootstrap angular after the device is ready (do not simply define the app in main.html), the goAngular callback is below
document.addEventListener("deviceready", goAngular, false); 

//UA Methods that send device tokens/msgs to server are launched below after the sync is finished 
//save the time (in Unix) when the device is paused (i.e. the user closes the app but it is still running in the background OS)
document.addEventListener('pause', function(){
	window.localStorage.setItem('pause', moment().toDate().getTime()); 
});

//listen for when the application is resumed from the OS background
document.addEventListener("resume", function($rootScope) { 
	var checkInTime = parseInt(window.localStorage.getItem('checkInTime'));
	if(!checkInTime){
		$rootScope.canCheckIn = true;
	}

	if (checkInTime && ((moment().toDate().getTime()) - checkInTime) >= 500000){ // Allows CHECK IN after 8 minutes
		$rootScope.canCheckIn = true;
	}
	else{
		$rootScope.canCheckIn == false;
	}

	var pause = parseInt(window.localStorage.getItem('pause'));
	if (pause && ((moment().toDate().getTime()) - pause) >= 21600000){ //checks if the application was paused more than 6 hours ago, in which case restart the web app to sync
		window.location.href = window.location.href.substring(0, window.location.href.indexOf('#')); //this restarts the web app by reloading main.html, it removed jQuery Mobile page ending such as main.html#promos
		
		if($rootScope.canCheckIn == false){
			$rootScope.canCheckIn = true;
		}
	}

	if (window.angular.bootstrap) {
    	//AngularJS is already loaded, so we can return here...

    	if(globals.debugApp){
    		console.log('WARNING: Tried to load angular more than once.');
    	}
  	}
}, false);



// Go Angular - Start up //
//----------------------------------//
var iosemail = ''; //global variable to hold email obtained from a legacy Connect 1.x app on an iOS device

function goAngular(){

	if(globals.debugApp){
   		// Full Name must ALSO be stored in Local Storage in order for the Intro function below to work (see 'local data factory'
   		console.log('Fullname for Greeting is :' + window.localStorage.getItem('FullName') );
   	}

	if( window.localStorage.getItem('FullName') && window.localStorage.getItem('FullName') != ''){
		var name;
		//set options for loader
		if (window.localStorage.getItem('FullName').indexOf(' ') != -1){ //if the user has a space in the name (i.e. has supplied a first and last name)
			name = window.localStorage.getItem('FullName').substring(0, window.localStorage.getItem('FullName').indexOf(' ')) ; //show greeting plus first name plus !
		}
		else{
			name = window.localStorage.getItem('FullName'); //or just show greeting plus full name (user only supplied first name) plus !
		}

		if (window.device.platform == 'Android'){
			var options = {
				label : "Welcome " + name + "!",
				position : "high",
				timeout: 60
			};
		}
		else{
			//var loaderMessage=((messageText==='resume')?"Please Wait.":'Updating Promos');
			//alert("loaderMessage::"+loaderMessage);
			var options = {
				label : "Welcome " + name + "!",
				color : '#000',
				spinnerColor : '#000',
				position : "high",
				timeout: 60
			}; //if this is an iOS device, change the loader color since it defaults to white and can blend in to the loadingpage
		}
	}
	else{
		//set options for loader
		if (window.device.platform == 'Android'){
			var options = {
				label : "Welcome New " + globals.appName + " user.",
				position : "high",
				timeout: 60
			};
		}
		else{
			//var loaderMessage=((messageText==='resume')?"Please Wait.":'Updating Promos');
			//alert("loaderMessage::"+loaderMessage);
			var options = {
				label : "Welcome New " + globals.appName + " user.",
				color : '#000',
				spinnerColor : '#000',
				position : "high",
				timeout: 60
			}; //if this is an iOS device, change the loader color since it defaults to white and can blend in to the loadingpage
		}
	}

	if(window.wizSpinner) {
		window.wizSpinner.show(options); //Phonegap plugin code, shows the device's native loader
	}

	//If LocalStorage is FALSE, load filesystem first.
	if( globals.localStorage == false ){
		window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function(dir){ CreateFilesystem(dir); });

		if(globals.debugApp){
			console.log('DEBUG: Filesystem plugin launched.');
		}
	}
	else{
		startApp(); //Start App here
	}

    //Assigns and init the Brightness plugin, which is used to turn up brightness temp 
    //during coupon redemption
    window.brightness = cordova.require("cordova.plugin.Brightness.Brightness");
    getBrightness();

    if(globals.debugApp){
		console.log("Application started, device is ready");
	}
	//Angular is loaded in FileSystem.js
}




//       Angular initialize         //
//----------------------------------//
function angularInit(element, bootstrap) {
  // Declare some variables to work with later.
  var elements = [element],
      appElement,
      module,
      // Notice, these are all acceptable attribute values for initializing an app
      names = ['ng:app', 'ng-app', 'x-ng-app', 'data-ng-app'],
      NG_APP_CLASS_REGEXP = /sng[:-]app(:s*([wd_]+);?)?s/;

  // A helper method for building up an array of elements to search through.
  function append(element) {
    element && elements.push(element);
  }

  // Loop through the names array and query anything that could be an Angular app,
  // then append the queried elements to our elements array.
  forEach(names, function(name) {
    names[name] = true;
    append(document.getElementById(name));
    name = name.replace(':', ':');
    if (element.querySelectorAll) {
      forEach(element.querySelectorAll('.' + name), append);
      forEach(element.querySelectorAll('.' + name + ':'), append);
      forEach(element.querySelectorAll('[' + name + ']'), append);
    }
  });

  // Loop through our elements array and see if there's anything that should be
  // bootstrapped as an application and assign it to appElement.
  forEach(elements, function(element) {
    if (!appElement) {
      // See if a class name exists that would be trying to register an app
      var className = ' ' + element.className + ' ';
      // See if there is a module name passed in with the class
      var match = NG_APP_CLASS_REGEXP.exec(className);
      // If there is a class on the element we passed in that matches the app class regex...
      if (match) {
        appElement = element;
        module = (match[2] || '').replace(/s+/g, ',');
      } else {
        // Loop through the attributes of the element and see if any of the
        // attribute names match keys from our names array
        forEach(element.attributes, function(attr) {
          if (!appElement && names[attr.name]) {
            appElement = element;
            module = attr.value;
          }
        });
      }
    }
  });

  // Bootstrap our element that has the application declaration
  if (appElement) {
    // There won't always be a module defined, so pass an empty array if none is given
    bootstrap(appElement, module ? [module] : []);
  }
}



//Brightness Plug-in GET//
//----------------------//
function getBrightness() {
    // win([-1,0-1]) float 0-1 is a brightness level, -1 represents a system default
    window.brightness.getBrightness( getSuccess, getFail);

    if(globals.debugApp){
		console.log('DEBUG: getBrightness() was called');
	}
}
function getSuccess(status) {
	if(globals.debugApp){
  	  //alert('GET BRIGHT - Success Message: ' + status);
  	  console.log('GET BRIGHT - Success Message: ' + status);
  	}
    globals.userBrightness = status; //brightness value was gotten and stored in a variable (oldBrightVal)
}
function getFail(status) {
    //alert('GET BRIGHT - Error: ' + status);
    console.log('GET BRIGHT - Error: ' + status);
}




//New Cordova Push Plugin //
//------------------------//
function pushInit(){
	var push = PushNotification.init({
        "android": {"senderID": globals.Change.AndroidPushID, "sound": true},
        "ios": {"alert": true, "badge": true, "sound": true, "clearBadge": true}, 
        "windows": {} 
	});

	if(globals.debugApp){
		console.log("[CORDOVA PUSH] Push init event");
		console.log("[CORDOVA PUSH] Android pushID: " + globals.Change.AndroidPushID);
		//alert('Push init ios/android event.. AndroidPushID: ' + globals.Change.AndroidPushID , function(){});
	}

	return push;
}


// Data object returns the following: data.registrationId
function onPushRegistration(push){
	push.on('registration', function(data) {

		if(globals.debugApp){
       		console.log("[CORDOVA PUSH] Registration event");
        	console.log("[CORDOVA PUSH] Push Registration ID - " + data.registrationId);
       		console.log("[CORDOVA PUSH] Data received : " + JSON.stringify(data));
       		//alert('push registration: ' + JSON.stringify(data) , function(){});
      	    //navigator.notification.alert('Data: ' + JSON.stringify(data) , function(){}, 'Push Registration', 'Ok');
       	}	

       	//Saving of Push Token
       	if( globals.localStorage == false){
       		globals.userAccount.set('PushToken', data.registrationId );
      	    saveLocalData();
      	}
      	else{
			window.localStorage.setItem('PushToken', data.registrationId ); 
		}
		
		sendPushTokenToServer();
    });
}

// Data object returns the following: data.message, data.title, data.count, data.sound, 
// data.image, data.additionalData
function onPushNotification(push){
	push.on('notification', function(data) {

		if(globals.debugApp){
    		console.log("[CORDOVA PUSH] Notification event");
    		console.log("[CORDOVA PUSH] Push message: " + data.title + " - " + data.message);
      	  	console.log("[CORDOVA PUSH] Data received" + JSON.stringify(data));
       		//alert('push notification: ' + JSON.stringify(data) , function(){});
         	//alert('handleIncomingPush::: '+JSON.stringify(event),function(){});
        }

 		var PushPromo = {};
 		var stores = [];
 		var p = getAllPromos(0);
		if(data.message && data.extras.promoid && p.length){ //check if this is a promo push and we have promotions saved on the device
			var promoid = data.extras.promoid; //promoid that came with the push
			p = p.filter(function(item){
				return (item.PromoID == promoid);
			});
			if (p.length){ //if the p array still has promos (i.e. we found a promo matching the promoid of the incoming push promo)
				p = updatePromoLinkVideoPhoto(p);
				PushPromo.PromoID = p[0].PromoID;
				PushPromo.Descript1 = p[0].Descript1;
				PushPromo.Descript2 = p[0].Descript2;
				PushPromo.PromoLink = p[0].PromoLink;
				PushPromo.PromoVideo = p[0].PromoVideo;
				PushPromo.PromoVideoImage = p[0].PromoVideoImage;
				PushPromo.PhotoURL = p[0].PhotoURL;
				PushPromo.DisplaySec = p[0].DisplaySec;
				PushPromo.PromoUse = p[0].PromoUse;
				PushPromo.DescriptPush = event.message;
				PushPromo.DateStart = p[0].DateStart;
				PushPromo.DateEnd = p[0].DateEnd;
			    PushPromo.AccountUses = p[0].AccountUses;
				PushPromo.Uses = p[0].Uses;
				PushPromo.Stores = p[0].Stores;

				//Saving of Push Promo
				if( globals.localStorage == false){
					globals.userAccount.set('PushPromo', JSON.stringify(p) ); // Save Push Promo
				}
				else{
					window.localStorage.setItem('PushPromo', JSON.stringify(p));
				}
			}
		}
        push.finish(function () { // ON Finish
            console.log('[CORDOVA PUSH] Finish successfully called');
        });
    });
}
// e (error) returns: e.massage
function onPushError(push){
	push.on('error', function(e) {
		if(globals.debugApp){
      	    console.log("[CORDOVA PUSH] push error: " + e);
        	//alert('push error: ' + e.message , function(){});
        }
    });
}



// Config - App Function //
//----------------------------------//
ConnectApp.config(function ($httpProvider, $compileProvider) {
    $httpProvider.defaults.headers.post['Content-Type'] = 'text/xml; charset=utf-8'; //default header for Angular http request
    $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|file|blob):|data:image|local:\//);
});


//tells PhoneGap to allow cross origin domain requests via HTTP
$( document ).bind( "mobileinit", function() {
	$.support.cors = true;
    $.mobile.allowCrossDomainPages = true;
}); 


// FUNCTIONS - START AND STOP LOADER //  
//These two helper functions show and hide the loader, note that when the loader is up, the
//user is unable to interact with the app since the loader is 100% width and height and should have the highest z-index
function startLoader(){
	$('#ajaxWaiting').show();
}
function stopLoader(){
	$('#ajaxWaiting').hide();
}



//this function is used throughout the app to reset the scrollbar when page content changes, it is important to
//reset the scroll bar everytime something shows or hides
function resetScrollbar(){
	try{
		$('.ui-content').iscrollview('refresh');
	}
	catch (err){}
}


// FUNCTION: ALERT //
//redefined the regular javascript alert method to call the native notification plugin and call a 
//custom callback function
function alert(message, callback){
	navigator.notification.alert(
	    message,  // message
	    function(){
	    	callback();
	    },         // callback
	    'Error',            // title
	    'Ok'                  // buttonName
	);
} 


//Page Show applicable for all pages in app
$(document).on('pageshow', function(){
	$('body').css('opacity', 1);
	try{
		$('.ui-content').iscrollview('refresh');
		$('#appFaqCollapsible').iscrollview('refresh');
		$('#gameHelpBody').iscrollview('refresh');
	}
	catch (err){}
});

// MAIN - Controller //
//----------------------------------//
var nowMap = null; //global variable for holding now map (google map)
var map = null; //global variable for holding map on store screen (google map)
var mymarker = null; //global variable for holding marker which shows user's current position on the now screen
var watchId = null; 
var carouselShown = false;	

//Controller for main or now screen, a lot of global things happen here
ConnectApp.controller('MainController', function($scope, $rootScope, $timeout, $http, connectAppFooterService, kickbackRewardsService, Loading, Sync, LocalData, ClientSet, GeoLocation, $compile){ 

	function CheckGpsLocation() {
		var checkInTime = parseInt(window.localStorage.getItem('checkInTime'));

		if(!checkInTime){
			$rootScope.canCheckIn = true;
		}
		if (checkInTime && ((moment().toDate().getTime()) - checkInTime) >= 500000){ // Allows CHECK IN after 8 minutes
			$rootScope.canCheckIn = true;
		}
		else{
			$rootScope.canCheckIn == false;
		}

	  	GeoLocation.setCurrentPosition().then(function(){
			if (!$rootScope.Beacon && $rootScope.canCheckIn ){

				if(globals.debugApp){
					console.log('Debug: Set current position for GPS was called..  initNowMap');
				}
				initNowMap(); //if already have beacon, do not update map/check in
			}//initialize now map after setting the location from GeoLocation factory (similar to app resume)

			if($rootScope.RegisteredUser && $rootScope.showGame){
				globals.tid = setTimeout(CheckGpsLocation, 6000); // repeat 
				console.log('LOCATION: Checking GeoLocation.setCurrentPosition()');
			}
		});
	 	
	}

	function abortCheckGpsLocation() { // to be called when you want to stop the timer
		if($rootScope.RegisteredUser && $rootScope.showGame){
		    clearTimeout(globals.tid);

		    if(globals.debugApp){
		   		console.log('LOCATION: GPS Location Check routine has been stoped.');
		   	}
		}
	}


	document.addEventListener('backbutton', function(){
		if ($.mobile.activePage.attr('id') == 'main'){
			navigator.app.exitApp();
		} //if the user is on the now screen then close the app
		else if ($.mobile.activePage.attr('id') == 'promodog'){
			$rootScope.leavePromo();
		} //if the user is on the promo screen then call leave promo
		else{
			$.mobile.back();
		} //otherwise go back one page in jQuery Mobile
		//need to add handling for redeemRegisterAlert screen .. if the user is on the screen from app start then exit app (because the previous page is the loading page) but otherwise (if the user gets to the screen from redeemRegisterAlerted from trying to redeem a promotion then do nothing i.e. let it default to going back to the previous page or the promotion the user was viewing)
	}, true); //add event handler for the android physical back button
	
	$('#main').one('pageshow', function(){ //the "one" binder indicates that this code block only runs the first time main is shown

		if(globals.debugApp){
			//alert('PromoImages count: ' +  ,function(){}); 
			console.log('Splash hidden');
		}

		navigator.splashscreen.hide(); //hide the splash screen
		window.wizSpinner.hide(); 

		$("#triviaQuestion").animate({opacity:1}, 400);
		setTimeout(function(){
			$("#triviaAnswer").animate({opacity:1}, 400);
		}, 400); //we animate triviaAnswer and triviaQuestion since they are not reay on pageshow (the heights on font sizes are still being changed)
			    	//we delay triviaAnswer until triviaQuestion comes in so that the question covers the answer at all times (i.e. the user does not see the
				    //answer animating in)
		carouselShown = true;	

		setTimeout(function(){
			//Start SYNC CHECKER FUNCTION HERE
			setTimeout(function(){
				Sync.start($scope, Sync);
			});
		});
	});

	$("#main").on('pageinit', function(){

		if(globals.debugApp){
			console.log('Main page init fired');
		}

		$rootScope.BrandImage = globals.Change.brandImage; //see CHAINLOADER.JS

		//MAIN Screen Resize
		var height = $(window).height(); // android and ios
		var width = $(window).width();
		var header = $('ui-header').height();
		var footer = $('ui-footer').height();

		//BRAND IMAGE RESIZE
		$("#brandImage").css('height', height * .26); // 26% of screen

		$("#main .ui-content").css('height', height - $('#main .ui-header').height() - $('#main .ui-footer').height() - $('#brandImage').height() - 11);
		var contentheight = $('#main .ui-content').height();

		$(".nowmoduleheader").css('height', contentheight * .05); // all Now Module Headers (3)
		$(".nowmoduleheader").css('padding', contentheight * .004); // all Now Module Headers (3)

		// ** Now Promo Carousel ** //
		$("#mainCarouselWrapper .ui-grid-b").css('height', contentheight * .333);
		var promoModHeight = $('#mainCarouselWrapper .ui-grid-b').height();
		$("#mainCarouselWrapper .tcimage").css('height', promoModHeight );

		var promowidth = Math.ceil(promoModHeight * 1.25);
		$("#mainCarouselWrapper .tcimage").css('width', promowidth + 'px' );
		$("#mainCarousel .touchcarousel-wrapper").css('height', promoModHeight );
		$("#mainCarouselWrapper .touchcarousel-item").css('height', promoModHeight );
		$("#mainCarouselWrapper .touchcarousel-item").css('width', $('.nowPromoModule .tcimage').width() );
		$("#mainCarouselWrapper .touchcarousel").css('height', promoModHeight);

		$("#pCarouselLoading").css('height', $("#mainCarouselWrapper .ui-grid-b").height() );


		//Now Electrum Content Resize
		$("#nowElectrum .ui-grid-d").css('height', contentheight * .21);
		$("#electrumText").css('padding', $('#nowElectrum .ui-grid-d').height() * .23);
		$("#electrumImage").css('height', $('#nowElectrum .ui-grid-d').height() * .9);
		$("#electrumImage").css('width', $('#electrumImage').height());
		$("#electrumImage").css('padding-left', $('#electrumImage').width() * .05 );
		$("#electrumImage").css('margin-top', $('#electrumImage').height() * .09 );


		//Now KickBack Content Resize
		$("#nowKickback .ui-grid-d").css('height', contentheight * .21);
		$("#kickbackText").css('padding', $('#nowKickback .ui-grid-d').height() * .08);
		$("#kickbackText").css('margin-top', $('#nowKickback .ui-grid-d').height() * .18);
		$("#kickbackImage").css('height', $('#nowKickback .ui-grid-d').height() * .9);
		$("#kickbackImage").css('width', $('#kickbackImage').height() * .88);
		$("#kickbackImage").css('padding-left', $('#kickbackImage').width() * .05 );
		$("#kickbackImage").css('margin-top', $('#kickbackImage').height() * .07 );


		// *** Now Feedback Content Resize *** //
		$("#nowFeedback .ui-grid-d").css('height', contentheight * .21);
		$("#nowFeedback #feedbackImage").css('height', $('#nowFeedback .ui-grid-d').height() * .6); // IMAGE
		$("#nowFeedback #feedbackImage").css('width', $('#nowFeedback .ui-grid-d').height() * .6); // IMAGE
		$("#nowFeedback #feedbackImage").css('margin-left', $('#feedbackImage').width() * .12 ); // IMAGE
		$("#nowFeedback #feedbackImage").css('margin-top', $('#feedbackImage').height() * .34 ); // IMAGE
		$("#feedbackText").css('margin-top', $('#nowShareOurApp .ui-grid-d').height() * .19); //  Margin Top


		//Now Trivia Content Resize
		$("#nowTrivia .ui-grid-d").css('height', contentheight * .21);
		$("#nowTrivia #triviaImage").css('height', $('#nowTrivia .ui-grid-d').height() * .84);
		$("#nowTrivia #triviaImage").css('width', $('#nowTrivia .ui-grid-d').height() * .84);
		$("#nowTrivia #triviaImage").css('padding-left', $('#electrumImage').width() * .07 );
		$("#nowTrivia #triviaImage").css('padding-top', $('#electrumImage').height() * .1 );

		$("#triviaQuestion").css('height', $('#nowTrivia .ui-grid-d').height());
		$("#triviaAnswer").css('height', $('#nowTrivia .ui-grid-d').height());

		// *** Now Share Content Resize *** //
		$("#nowShareOurApp .ui-grid-d").css('height', contentheight * .21);
		$("#nowShareOurApp #shareImage").css('height', $('#nowShareOurApp .ui-grid-d').height() * .84); // SHARE IMAGE
		$("#nowShareOurApp #shareImage").css('width', $('#nowShareOurApp .ui-grid-d').height() * .84); // SHARE IMAGE
		$("#nowShareOurApp #shareImage").css('margin-left', $('#shareImage').width() * .08 ); // SHARE IMAGE
		$("#nowShareOurApp #shareImage").css('margin-top', $('#shareImage').height() * .1 ); // SHARE IMAGE
		$("#shareText").css('margin-top', $('#nowShareOurApp .ui-grid-d').height() * .16); // SHARE TEXT - Margin Top

		//Now Game Content Resize
		$("#nowGamePlays .ui-grid-d").css('height', contentheight * .21);
		$("#nowGamePlays .ui-block-b #playNowText").css('margin-top', $('#nowGamePlays .ui-grid-d').height() * .255); // Game TEXT (B) - Margin Top
		$("#nowGamePlays .ui-block-d #playsNumber").css('margin-top', $('#nowGamePlays .ui-grid-d').height() * .12); // Plays Number (D) - Margin Top
		$("#nowGamePlays .ui-block-d #playsText").css('margin-top', $('#nowGamePlays .ui-grid-d').height() * -.08); // Plays Text (D) - Margin Top

		//Now Focus Promo Content Resize
		$("#nowFocusPromo .ui-grid-d").css('height', contentheight * .21);
		$("#focusPromoImage").css('height', $('#nowFocusPromo .ui-grid-d').height() ); // FOCUS PROMO IMAGE
		$("#focusPromoImage").css('width', $('#nowFocusPromo .ui-grid-d').height() * 1.5 ); // FOCUS PROMO IMAGE
		$("#focusPromoImage").css('background-size', $('#nowFocusPromo .ui-grid-d').height() * 1.35 ); // FOCUS PROMO IMAGE

		$("#focusPromoText").css('padding', $('#nowFocusPromo .ui-grid-d').height() * .24); // SHARE TEXT - Margin Top



		if (!$('#mainCarouselWrapper').hasClass('ng-hide') && $scope.Promos.length > 0 ){ //checking to see if the carousel will be shown

			$('#mainCarousel').touchCarousel({ 
				pagingNav: false,
				directionNav: false,
				snapToItems: true,
				scrollbar: false,
				loopItems: true,
				autoplay: true,
				Delay: 1500,
				transitionSpeed: 400,
				scrollToLast: true,
				useWebkit3d: true
			});

			setTimeout(function(){
				$('#mainCarousel').css('margin-left', ( $(window).width() * .5 - $("#mainCarouselWrapper .tcimage").width() / 1.82 ) ); //centering the touch carousel so that the focused image is in the horizontal center of the page
				$("#mainCarousel").animate({opacity:1}, 1800);

				stopPromoCarouselLoader();//method located in sync module
			}, 30);
		}

		//This is for Older Android devices (Android V4.3 and older) that
		//do not support scaling font.
		if(device.platform == "Android" && parseFloat(device.version) < 4.4){
			console.log("VW Font is not supported. Using Standard font (1A)");
			$("#shareText").css('font-size', '37px');
			$("#nowRating .ui-block-b").css('font-size', '11px');
			$("#nowRating .ui-block-d").css('font-size', '11px');
			$("#answerText").css('font-size', '13px');
			$("#questionText").css('font-size', '11px');
			$("#nowFocusPromo .ui-block-b").css('font-size', '10px');
			$("#electrumText").css('font-size', '11px');
			$("#playsNumber").css('font-size', '17px');
			$("#playsText").css('font-size', '13px');
			$("#settingsBut").css('font-size', '11px');
			$(".nowmoduleheader").css('font-size', '11px');
			$(".footerButton").css('font-size', '13px');
		}

		$timeout(function(){
			if(window.wizSpinner){
				window.wizSpinner.hide(); //hide the native loader
			}
			resetScrollbar();
		}, 1);
	});

	$('#main').on('pagehide', function(){
		//$("#mainCarousel").css('opacity', 0);	//set carousel opacity back to 0
		abortCheckGpsLocation();
		$timeout(function(){
			resetScrollbar(); //resetScrollBar, always call this global function whenever the height of the page changes due to something appearing or hiding, ...etc
			if ($rootScope.radiusActive == 1 && watchId != null && typeof radiusnetworks.plugins.proximitykit.clearWatch == 'function'){
				ClearWatch(); //STOP beacon from running in the background
			} 
		});
	});


	//Helper function for proximity kit plug in
	function ClearWatch (){
		$timeout(function(){
			radiusnetworks.plugins.proximitykit.clearWatch(watchId);  // clearWatch( watchId )

			if(globals.debugApp){
				console.log("** Canceling watch ID. It is - " + watchId + " **");
			}
			watchId = null;
		}, 1);
	}

	//Helper function for proximity kit plug in
	var proximityKitSuccessHandler = function(message) {
		var pkEventType = message[radiusnetworks.plugins.proximitykit.constants.keys.eventType];
	    var beacon = null;

	    // Log beacon information if receiving a ranging callback
	    if (pkEventType == radiusnetworks.plugins.proximitykit.constants.eventTypes.rangedBeacon) {
	       beacon = message[radiusnetworks.plugins.proximitykit.constants.keys.beacon];
	       if (beacon != null) {
		      	if ($rootScope.Beacon){
					$rootScope.Beacon = null;
				}
				var s = $scope.Stores.filter(function(item){
					return (item.StoreID == message.beacon.attributes.StoreID)
				});

				if(globals.debugApp){
		       		console.log("Proximity Kit Event: " + pkEventType + " " + JSON.stringify(beacon));
		       	}
		      
		        if (s.length && $rootScope.canCheckIn && !$rootScope.allowCheckIn && $rootScope.RegisteredUser && ClientSet.getShowCheckIn() && !ClientSet.getGameHide()){	//make sure this beacon matches a store and we can check in
					$rootScope.Beacon = message.beacon;
					$rootScope.Beacon.Store = s[0];
					$timeout(function(){

						if(globals.debugApp){
							console.log("** CHECK IN MODULE has been called for store check in by BEACON. **");
						}

						$rootScope.allowCheckIn = true;
						$rootScope.CheckInMessage = "@ " + $rootScope.Beacon.Store.Address1;
						$('#nowCheckIn').removeClass('lightSpeedOut');
						$rootScope.showNowMap = false;

						if($rootScope.showTrivia)
							$rootScope.ReShowTrivia = true;

						$rootScope.showTrivia = false;
						$rootScope.showShare = false;
						$rootScope.showFocusPromo = false;

						ClearWatch(); // Clears the watch for the ibeacon checking
					});
				}
			}
	    }
	    else if (pkEventType == radiusnetworks.plugins.proximitykit.constants.eventTypes.rangedBeacon.didRangeBeacon){
			if ($rootScope.Beacon && ($rootScope.Beacon.attributes.StoreID == message.region.attributes.StoreID) && ($rootScope.Beacon.attributes.Location == message.region.attributes.Location)){
				$rootScope.Beacon = null;
			}
		}
	    // Log event type if receiving another type of callback
	    else {
	    	if(globals.debugApp){
	    		console.log("Proximity Kit Event: " + pkEventType);
	    	}
	    }
	};

    // NOT BEING USED ANYMORE (since v2.5) 
	$rootScope.Error = function(message, callback, cantryagain, tryagain) {

		$timeout(function() {
			$rootScope.ErrorMessage = message;
			$rootScope.CanTryAgain = cantryagain;
		}); //update the error screen error message and indicate if the user can try again (show try again option or not)
		$('#error').show();
		
		$('.errorImage').css('margin-top', ($(window).height() - $('.errorImage').height()) / 2 - $('.errorTextWrapper').height());
		
		$(".errorImage").animate({
			opacity: 1
		}, 2000);

		$('#error').addClass('slideInDown');

		$rootScope.Exit = function() {
			$('#error').addClass('slideOutUp');
			$('#error').hide();
			$('#error').removeClass('slideInDown');
			$(".errorImage").css('opacity', 0);
			$timeout(function() {
				callback();
			});
		}; //function for closing the error screen and calling the callback passed into the function

		$rootScope.TryAgain = function() {
			$('#error').addClass('slideOutUp');
			$('#error').hide();
			$('#error').removeClass('slideInDown');
			$timeout(function() {
				tryagain();
			});
		}; //function for closing screen and trying again (used in gameend)
	};


	$rootScope.redeemRegisterAlerted = false; //global variable to indicate whether or not the user saw the redeemRegisterAlert (Please Register) page after attempting to redeem a promotion without an account
	$('#redeemRegisterAlert').on('pagehide', function(){ //when the redeem register alert page hides
		if ($.mobile.activePage.attr('id') == 'promodog'){ //if we are currently on the promos page (i.e. the user saw redeemRegisterAlert after attempting to redeem a promotion without an account)
			$rootScope.redeemRegisterAlerted = false; //set redeemRegisterAlerted back to false
		}
	});

	// WELCOME MESSAGE / RANDOM GREETING LOGIC //
	var g = getGreeting(); //get a random greeting

	function getGreeting(){
		var now = moment().hour();
		var message = ""; //holds the message to be shown in the now module based on the time of day
		if (5 <= now && now <= 10){
			message = "Good Morning";
		}
		else if (11 <= now && now <= 16){
			message = "Good Afternoon";
		}
		else if (17 <= now && now <= 22){
			message = "Good Evening";
		}
		else{
			message = "Have a great late night";
		}

		return message;
	}

	$scope.getWelcomeMessage = function(){
		if ($rootScope.RegisteredUser){
			if (getFullName().indexOf(' ') != -1){ //if the user has a space in the name (i.e. has supplied a first and last name)
				return g + " " + getFullName().substring(0, getFullName().indexOf(' ')) + "!"; //show greeting plus first name plus !
			}
			else{
				return g + " " + getFullName() + "!"; //or just show greeting plus full name (user only supplied first name) plus !
			}
		}
		else{
			return g + "!"; //if this is an unregistered user we do not have the name so just show greeting anf !
		}
	}; //function got getting welcome message that will show on the main header

	$scope.setPromosForCarousel = function(promos){
		$scope.Promos = promos;
	}


	document.addEventListener("resume", function() {

		var checkInTime = parseInt(window.localStorage.getItem('checkInTime'));

		if(!checkInTime){
			$rootScope.canCheckIn = true;
		}
		if (checkInTime && ((moment().toDate().getTime()) - checkInTime) >= 500000){ // Allows CHECK IN after 8 minutes
			$rootScope.canCheckIn = true;
		}
		else{
			$rootScope.canCheckIn == false;
		}

		if($rootScope.RegisteredUser && $rootScope.showGame){
			globals.tid = setTimeout(CheckGpsLocation, 2000);
		}
	}, false); //add another listener to the app resume event (first one is defined above) to update GPS (location, now map, checkin, now focus promo)
	
	document.addEventListener('pause', function(){
		window.localStorage.setItem('pause', moment().toDate().getTime());
		if($rootScope.showGame){
			abortCheckGpsLocation();
		}
	}); // Pause listener to abort GPS listening


	// LOADING FUNCTION (GETS SET OFF FIRST) //
	Loading.then(function(){

		$timeout(function(){
			if(globals.debugApp){
				console.log("--loading main scene--");
				console.log('Registered User = ' + $rootScope.RegisteredUser);
			}
			
			$rootScope.LogoImage = globals.Change.logoImage; //set the logo image for the menu based on the chain's js file
			$rootScope.AboutImage = globals.Change.aboutImage;

			//handle showing/hiding for various nowmodule (see main.html to see all now modules on main page)
			$rootScope.haveSocialShare = ClientSet.getSocialShareMessage() != null && ClientSet.getShowShare() && getShowShare(); //to show social share make sure the client set has a social share message, allows social share, and that localdata allows social share (i.e. 1 week has passed since last time client saw social share)
			$rootScope.showRating = getShowRating() && ClientSet.getShowRating();
			$rootScope.showGame = ClientSet.getShowGame() && !ClientSet.getGameHide();
			$rootScope.hideGame = ClientSet.getGameHide();
			$rootScope.radiusActive = ClientSet.getRadiusActive();

			if(globals.debugApp){
				console.log('ClientSet.getShowGame()  equals ' + ClientSet.getShowGame() );
				console.log('ClientSet.getGameHide()  equals ' + ClientSet.getGameHide() );
			}

			// *Electrum Client Sets Init* //
			$rootScope.enableElectrum = ClientSet.getElectrumURL() && ClientSet.getElectrumAppID();

			if($rootScope.enableElectrum ){
				$rootScope.electrumMenuMain = ClientSet.getElectrumMenuMain();
				$rootScope.electrumMenu47 = ClientSet.getElectrumMenu47();
				$rootScope.electrumURL = ClientSet.getElectrumURL();
				$rootScope.electrumAppID = ClientSet.getElectrumAppID();
				$rootScope.electrumServerErr = ClientSet.getElectrumServerErr();
				$rootScope.electrumBarcode = getElectrumBarcode();
				$rootScope.electrumPoints = false; // will replace with ClientSet when this feature is fully implemented
				$rootScope.electrumNow47 = ClientSet.getElectrumNow47();
				$rootScope.ElectrumImage = globals.Change.electrumImage;

				if(globals.debugApp){
					console.log("** Electrum URL received: " + $rootScope.electrumURL );
					console.log("** Electrum APP ID received: " + $rootScope.electrumAppID );
					console.log("** Electrum Enabled: " + $rootScope.enableElectrum );
					console.log("**** Electrum Barcode: " + $rootScope.electrumBarcode );
				}
			}

			// *Lottery Client Sets init* //
			$rootScope.lotteryState01 = ClientSet.getLotteryState01();

			if($rootScope.lotteryState01 && $rootScope.lotteryState01 != 0){
				$rootScope.lotteryState02 = ClientSet.getLotteryState02();
				$rootScope.lotteryState03 = ClientSet.getLotteryState03();
				$rootScope.lotteryUrl01 = ClientSet.getLotteryUrl01();
				$rootScope.lotteryUrl02 = ClientSet.getLotteryUrl02();
				$rootScope.lotteryUrl03 = ClientSet.getLotteryUrl03();
			}

			if(globals.debugApp){
				console.log('DEBUG: Checking for Kickback API bool');
			}


			//boolean for kickback loyalty program API
			$rootScope.kickbackEnabled = ClientSet.getKickbackURL() && ClientSet.getKickbackUser() && ClientSet.getKickbackPass();

			//Kickback settings are set here
			if($rootScope.kickbackEnabled){
				$rootScope.kickbackURL = ClientSet.getKickbackURL();
				$rootScope.kickbackUser = ClientSet.getKickbackUser();
				$rootScope.kickbackPass = ClientSet.getKickbackPass();
				$rootScope.kickbackImage = ClientSet.getKickbackImage();

				kickbackRewardsService.updateService($rootScope.kickbackUser, $rootScope.kickbackPass, $rootScope.kickbackURL);

				if(globals.debugApp){
					console.log('** KICKBACK user/pass: '+ ClientSet.getKickbackUser() + ' ' + ClientSet.getKickbackPass() + ' .. URL: ' + ClientSet.getKickbackURL() );
					alert('Kickback user/pass: '+ ClientSet.getKickbackUser() + ' ' + ClientSet.getKickbackPass() + ' .. URL: ' + ClientSet.getKickbackURL() ,function(){}); 
				}
			} 

			$rootScope.debugMode = globals.debugApp;

			var checkInTime = parseInt(window.localStorage.getItem('checkInTime'));
			if(!checkInTime){
				$rootScope.canCheckIn = true;
			}
			if (checkInTime && ((moment().toDate().getTime()) - checkInTime) >= 500000){ // Allows CHECK IN after 8 minutes
				$rootScope.canCheckIn = true;
			}
			else{
				$rootScope.canCheckIn == false;
			}


			//----------------------------------//
			// NOW MODULE STACK Functionality //
			//----------------------------------//
			if(!$rootScope.allowCheckIn){

				$scope.Trivia = getTrivia();

				if (!$scope.Trivia){
					$rootScope.showTrivia = false;
					$scope.TriviaMessage = "";
					$scope.TriviaAnswer = "";
				}
				else{
					$rootScope.showTrivia = ClientSet.getShowTrivia() && true;
					$scope.TriviaMessage = $scope.Trivia.Question;
					$scope.TriviaAnswer = $scope.Trivia.Answer;
				}

				$rootScope.showFasCash = ClientSet.getStates() && ClientSet.getFasCash() && ClientSet.getACHMerchantID() && ClientSet.getACHSubmitURL() && getShowFasCash(); //only used in Fas APP, all other clients have FasCash client set to 0

				// Show Share Flag set to false
				if( !$rootScope.showTrivia){
					if(!ClientSet.getShowFocusPromo()){
						$rootScope.showShare = true;
					}
					else{
						$rootScope.showShare = false;

						if ($scope.Promos){ //if we have promotions

							$rootScope.showFocusPromo = ClientSet.getShowFocusPromo(); //check if client settings allow focus promo
							$scope.FocusPromo = getFocusPromo($scope.Promos);	 //get focus promo (if any)
							
							if($scope.FocusPromo == null){
								$rootScope.showShare = true;
							}
						}
						else{
							$rootScope.showFocusPromo = false;
							$scope.FocusPromo = null;
							//share mod
							$rootScope.showShare = true;
						}

						if ($scope.FocusPromo){ //if there is focus promo then edit styling
							$("#nowFocusPromo .ui-grid-d").css('opacity', 0);
							var u = 'url(' + $scope.FocusPromo.PhotoURL + ')';
							$('#focusPromoImage').css('background', u);
							$('#focusPromoImage').css('background-repeat', 'no-repeat');
							$('#focusPromoImage').css('background-size', 'contain');

							setTimeout(function(){
								$("#nowFocusPromo .ui-grid-d").animate({opacity:1}, 1000);
							}, 50);
						}
					}
				}
			}
			//End of NOW MODULE STACK Functionality
			if(globals.debugApp){
				console.log('ONTO resolve registration redirect');
			}

			$timeout(function(){
				$rootScope.resolveRegistrationRedirect(); //after the loading factory is complete and now modules showing/hiding has been determined, take the user away from the sync page
				$rootScope.$broadcast('connect:ready');
			});
			
			$scope.Promos = getAllPromos(0,'Loading.then function in app.js');

			if(globals.debugApp){
				console.log( "Promos array length in main screen: " + $scope.Promos.length );
			}
			
			//MAKE MYSTORES A SCOPE VARIABLE
			$scope.Stores = globals.Stores;
			
			//Push notification activation
			$timeout(function(){
			    //Initialized Push Notifications and startup Listener methods, one listens for registration, the other for receiving
				//push notifications, and the last for any errors caught
				push = pushInit();
				onPushRegistration(push);
				onPushNotification(push);
				onPushError(push);
			}, 300);
		},1);
	}); //End of Loading Function


	$rootScope.resolveRegistrationRedirect = function(){ //function to take the user away from sync page and to main page or redeemRegisterAert page
		
		if(globals.debugApp){
			console.log('resolve registration redirect');
		}

		$.mobile.changePage('#main');
		
		setTimeout(function(){
			var ClientLog = window.localStorage.getItem('ClientLog');
			if (ClientLog != null) //if there is unsent client logs then send them to the server
				$http.post(GetItemFromLocalData('ClientURL'), soapMessageSaveClientLog(ClientLog), {timeout: 21000} ).then(function(response){
					window.localStorage.setItem('ClientLog', null);
				});
			var UnsentFeedback = []; //array to hold any feedback we are unable to send now
			var FeedbackSoapMessages = localStorage.getItem('Feedback') ? JSON.parse(window.localStorage.getItem('Feedback')) : [];
			if (FeedbackSoapMessages.length){ //if we have unsent feedback soap messages then send them
				for (var i = 0; i < FeedbackSoapMessages.length; i++){
					sendFeedback(FeedbackSoapMessages[i], FeedbackSoapMessages.length, i);
				}
			}

			function sendFeedback(FeedbackSoapMessage, total, curr){ //helper function to send feedback
				setTimeout(function(){
					$http.post(GetItemFromLocalData('ClientURL'), FeedbackSoapMessage(), {timeout: 20000} ).then(function(response){
						if(globals.debugApp){
							console.log("Success: Sent Feedback");
						}

						if (curr == (total - 1)){
							saveUnsentFeedback(); //if the current feedback is the final feedback to be saved then save any unsent feedback (potential bug: this function is an ajax call which runs async, thus the last feedback might be done sending but other feedback is still trying to be sent)
						}
					}, function(error){
						if(globals.debugApp){
							console.log("ERROR: Could not send Feedback");
						}
						UnsentFeedback.push(FeedbackSoapMessage); //in case of error, push unsent feedback so it can be sent later
						if (curr == (total - 1)){
							saveUnsentFeedback();
						}
					});
				}, 10);
			}

			function saveUnsentFeedback(){ //helper function to save unsent feedback
				window.localStorage.removeItem('Feedback');
				if (UnsentFeedback.length > 0){
					window.localStorage.setItem('Feedback', JSON.stringify(UnsentFeedback));
				}
			}
		}, 1); 
	} // End of Resolve Registration


	$('#main').on('pageshow', function(){
		if(globals.debugApp){
			console.log('Splash hidden');
		}
		navigator.splashscreen.hide(); //hide the splash screen 
		window.wizSpinner.hide(); 

		var checkInTime = parseInt(window.localStorage.getItem('checkInTime'));
		if(!checkInTime){
			$rootScope.canCheckIn = true;
		}

		if (checkInTime && ((moment().toDate().getTime()) - checkInTime) >= 500000){ // Allows CHECK IN after 8 minutes
			$rootScope.canCheckIn = true;
		}
		else{
			$rootScope.canCheckIn == false;
		}

		if($rootScope.RegisteredUser && $rootScope.showGame){
			//Set timeout and start CheckGPSLocation intervals (For Check IN).  Stops on 'Pause' state
			if(globals.debugApp){
				console.log('CHECKIN - DEBUG: ***** Locations Check has been started. *****');
			}
			globals.tid = setTimeout(CheckGpsLocation, 2000);
		}

		//------------------------------------------------------//
		// Stop Module Animations after first load Functionality //
		//-------------------------------------------------------//
		// Important: This logic block implements the last module to not animate on
		// the first time showing, so that its already there when page shows
		if($rootScope.showTrivia){
			$("#nowTrivia").removeClass('fadeInUpBig');
		}
		if($rootScope.showFocusPromo){
			$("#nowFocusPromo").removeClass('fadeInUpBig');
		}
		if($rootScope.showShare){
			$("#nowShareOurApp").removeClass('fadeInUpBig');
		}
		if($rootScope.allowCheckIn){
			$("#nowCheckIn").removeClass('fadeInUpBig');
		}
		if($rootScope.enableElectrum && $rootScope.electrum47){
			$("#nowElectrum").removeClass('fadeInUpBig');
		}
		//End of Stop Module Animations

		SaveAppDataJson(); //any save data to file


		//  IN APP EMAIL REGISTRATION VERIFICATION LOGIC //
		//console.log($scope.fullname + " " + $scope.email + " " + $scope.password + " " + $scope.phone + " " + $scope.birthday + " " + $scope.geopromotion);
		//Create a checking function for this and call it on resume and start up
		if( getVerifyPrompt() == 1 ){

			if (window.device.platform == 'Android') {
					var options = {
						label : "Processing..",
						timeout: 120
					};
				} else {
					var options = {
						label : "Processing..",
						color: '#000',
						spinnerColor: '#000',
						timeout: 120
					};
				}
				if(window.wizSpinner) {
					window.wizSpinner.show(options); 
				}

			//startLoader(); 
			//start the AJAX loader (used whenever we are about to make calls to the server or when something will take time) the 
			//loader prevents the user from doing anything else until it hides
			//Debug LOG to Make sure local storage is sending the right info

			$http.post(GetItemFromLocalData('ClientURL'), soapMessageGetAccountID(), {timeout: 16000} ).then(function(response){ //call appropriate soapMessage function (see js/soapMessages.js) with the appropriate parameters	
				//stopLoader(); //stop loader since AJAX call is complete
				if(window.wizSpinner) {
					window.wizSpinner.hide(); 
				}
				var verified = $(response.data).find('GetAccountIDResult').children("Validated").text();

				if(globals.debugApp){
					console.log("verify message is : " + verified );
				}

				//Check if account is validated and if dailog hasnt been shown yet
				if( verified == "true" ){

					$rootScope.RegisteredUser = true; //set the user as registered
					window.localStorage.setItem('verifyPrompt', 2); //set verify prompt to 2 (shown)
					navigator.notification.alert(
						'You are now logged in, Enjoy!',  // message
					    function(){
				  	  	$.mobile.changePage('#interestList');
				  	  },         // callback
					    'Your Account and Email has been Verified',            // title
					    'Ok'                  // buttonName
					);
				}

			}, function(error){ //indicates that the ajax function failed due to poor internet or something else
					if(window.wizSpinner) {
						window.wizSpinner.hide();  //stop the loader
					}
					$rootScope.Error(globals.ajaxTimeoutMessage, //the message to be shown on the error screen
					function(){ //callback
						if(globals.debugApp){
							console.log(" .. ");
						}
				},
				false, //whether or not to show the try again button (used for game ending error)
				function(){
					//try again call back
					if(globals.debugApp){
						console.log(" .. ");
					}
				}); //call the error method, defined in ErrorController below, shows the blue screen
			});
		}


		//----------------------------------//
		//        Beacon functionality      //
		//----------------------------------//

		if (window.device.platform == 'Android'){

			if(globals.debugApp){
				console.log(" Beacon = " + $rootScope.Beacon);
				console.log(" Check in = " + $rootScope.canCheckIn);
				console.log("Debug: Not using proximity kit for android");
			}
			if (!$rootScope.Beacon && $rootScope.canCheckIn ){
				initNowMap(); //if already have beacon, do not update map/check in
			}
		}
		else{  // iOS proximity kit initiation

			$timeout(function(){
				if ($rootScope.radiusActive == 1 && watchId == null && $rootScope.canCheckIn && globals.proximityIsGood){
					if(globals.debugApp){
						console.log('radiusnetworks initializing.. ');
					}

					watchId = radiusnetworks.plugins.proximitykit.watchProximity(proximityKitSuccessHandler, function(message){
						//Failure responce if success hndler doesnt get called
						console.log("Failure: Response from plugin is " + message);
						saveClientLog(97, 0, 0, '', 0, 'beacon watch proximity error: ' + message);
					});
				}   
				if (!$rootScope.Beacon && $rootScope.canCheckIn ){
					initNowMap(); //if already have beacon, do not update map/check in
				}
			}, 3000);

		}

		$timeout(function(){
			resetScrollbar(); //resetScrollBar, always call this global function whenever the height of the page changes due to something appearing or hiding, ...etc
		}, 200);

		if(globals.debugApp){
			console.log("** MAIN PAGE IS SHOWING **");
		}
	});


	//function to show brand image if it is available, ready and we have less than 2 now 
	//modules on the screen (does not include map and promos carousel) OR the promos carousel 
	//is not visible
	$scope.getShowBrand = function(){ // Need this for the Brand image to display in Now Screen
		if (globals.Change.brandImage ){
			setTimeout(function(){
				resetScrollbar();
			}, 500);
			return !$('#mainpanel').is(':visible') || $('.nowmodule:visible').length < 2;
		}
		else{
			return false;
		}
	}; 

	$scope.viewPromo = function(p){ //function for viewing promo, used for focus promo module and push module (similar functions in other areas of the app)
		$rootScope.Promo = p; //the promo that will be shown is always saved in the rootscope since multiple controllers support viewing promos, the promodog page is part of the PromoDialogController (this is the promo details screen)
		$rootScope.promoDialogPic = p.PhotoURL;
		$rootScope.PromoRedemption = true;
		$rootScope.PrizeRedemption = false;
		$rootScope.notRedeemable = p.PromoUse == 0; //determines if we should show the redeem button

		//use colorthief to calculate the coldest ratio based on the color palette and set that color as the promo header color
		var coldestIndex = null;
		var coldestRatio = 0;
		try{
			myimage = new Image();
			myimage.src = $rootScope.Promo.PhotoURL;
			var colorThief = new ColorThief();
			var colors = colorThief.getPalette(myimage, 9);
			for (var i = 0; i < 9; i++){
				if (((colors[i][2]/colors[i][0]) > coldestRatio) && (((colors[i][0] + colors[i][1] + colors[i][2])/3) < 200)){
					coldestRatio = colors[i][2]/colors[i][0];
					coldestIndex = i;
				}
			}
		}
		catch(err){ //color thief is a bit buggy so if an error occurs set the header text color to black or the best coldestRatio you have so far
			if (coldestIndex == null)
				$('.promoDialogDescription').css('color', 'black');
			else{
				var r = colors[coldestIndex][0];
				var g = colors[coldestIndex][1];
				var b = colors[coldestIndex][2];
				$('.promoDialogDescription').css('color', 'rgb(' + r + ',' + g + ',' + b + ')');
			}
		}
		$timeout(function(){
			$.mobile.changePage('#promodog');
			connectAppFooterService.navIcons[2].selected = true;
			connectAppFooterService.navIcons[0].selected = false;
		
		});
		if ($scope.PushPromo && p.PromoID == $scope.PushPromo.PromoID){
			window.localStorage.setItem('PushPromo', null); //if this is the push promo then remove push promo and the push promo now module (potential bug: if the focus promo is the same as the push promo then clicking the focus promo will result in hiding the now push module, i guess that makes sense)
			
			$timeout(function(){
				$scope.PushPromo = null;
			});
		}
		saveClientLog(32, $rootScope.Promo.PromoID, 0, 'viewed promotion');
	};


	$scope.fasCashBack = function(){ //handler for clicking the fas cash now module
		if ($rootScope.RegisteredUser){ //user must be registered for the app to register for fas cash
			$.mobile.changePage('#fascash');
		}
		else{
			navigator.notification.confirm(
			   'You must be a registered FasApp user to register for CashBack. Would you like to register or sign in now?',
			    callBackFunctionFasRegister, // Specify a function to be called
			    'Registration Required',
			    ["Yes", "No"]
			);

			function callBackFunctionFasRegister(b){
			  if(b == 1){ //if the user chooses yes
			    $.mobile.changePage('#redeemRegisterAlert');
			    	 $('#redeemRegisterAlert_back').show();
			  }
			  else {
			  }
			}
		}
	};

	$scope.nowElectrum = function(){
		$.mobile.changePage('#electrum');
		connectAppFooterService.navIcons[1].selected = true;
		connectAppFooterService.navIcons[0].selected = false;
	}
	$scope.nowKickback = function(){
		$.mobile.changePage('#kickbackRewards');
		connectAppFooterService.navIcons[1].selected = true;
		connectAppFooterService.navIcons[0].selected = false;
	}
	$scope.nowFeedback = function(){
		$.mobile.changePage('#feedbackprompt');
		$rootScope.AppFeedback = false;
		connectAppFooterService.navIcons[1].selected = true;
		connectAppFooterService.navIcons[0].selected = false;
	}

	$scope.nowPromos = function(){
		$.mobile.changePage('#promos');
		connectAppFooterService.navIcons[2].selected = true;
		connectAppFooterService.navIcons[0].selected = false;
	}

	$scope.getElectrumMsg = function(){ 
		var msg = '';
		if($rootScope.electrumBarcode){
			msg = 'You are a Registered User. \n Tap to enter the Loyalty page!';
		}
		else{
			msg = 'You are not a Registered User for our Loyalty program. \n Tap Here to Sign up! ';
		}
		return msg;
	};

	$scope.getKickbackMsg = function(){ 
		var kickbackMsg = '';
		if(getKickbackSignedIn() == 1){
			kickbackMsg = 'Access your ' + globals.Change.chainFriendly + ' Rewards card and check your progress on rewards.';
		}
		else{
			kickbackMsg = 'Sign-up for ' + globals.Change.chainFriendly + ' Rewards to start saving 3 cents per gallon and gain access to exclusive rewards.';
		}
		return kickbackMsg;
	};


	$scope.getPlays = function(){ //function to get plays to show in the gameplays now module
		return getGamePlays();
	};


	$scope.getPlayMsg = function(){

		var playMsg = "test";

		if( getGamePlays() == 0){
			playMsg = "Please check back tomorrow for more plays";
			$("#playNowText").css('font-size', '4vw');
			$("#playNowText").css('padding-left', '3px');
			$("#playNowText").css('padding-right', '5px');
		}
		else{
			playMsg = "Play Now";
			$("#playNowText").css('font-size', '8vw');
		}

		if (!$rootScope.RegisteredUser){
			playMsg = "You must register in order to play and earn prizes";
			$("#playNowText").css('font-size', '3.2vw');
			$("#playNowText").css('padding-left', '3px');
			$("#playNowText").css('padding-right', '5px');
		}

		return playMsg;
	};

	$scope.Share = function(){ //function for clicking the share module
		window.plugins.socialsharing.share(ClientSet.getSocialShareMessage() + '\n' + ClientSet.getSocialShareURL(), ClientSet.getSocialShareTitle(), null, null); //call the social sharing phonegap plugin
		$timeout(function(){
			//$rootScope.haveSocialShare = false;
			resetScrollbar();
		}, 10000);
		saveClientLog(0, 0, 0, '', 'user shared app');
	};

	$rootScope.canCheckIn = true; //variable that lets us know if a user can check in, defaults to true and is set to false when the user checks in (so that clicking the check in now module again does nothing), resets to true when the app pauses/resumes since the check in now module might reappear
	
	$scope.CheckIn = function(){
		if ($rootScope.canCheckIn){

			$timeout(function(){
				$rootScope.canCheckIn = false;
				$rootScope.CheckInMessage = "Checking in ..."; //change the message on the check in now module
				$('#nowCheckIn .ui-block-b').css('padding-top', 0);
			});

			if (window.device.platform == 'Android') {
				var options = {
					label : "Processing Check-In..",
					timeout: 120
				};
			} else {
				var options = {
					label : "Processing Check-In..",
					color: '#000',
					spinnerColor: '#000',
					timeout: 120
				};
			}

			if(window.wizSpinner) {
				window.wizSpinner.show(options); 
			}

			var clientURL;
			//Load Client URL
			if( globals.localStorage == false){
				clientURL = GetItemFromLocalData('ClientURL');
			}
			else{
				clientURL = window.localStorage.getItem('ClientURL') 
			}


			$http.post(clientURL, soapMessageCheckInGPSStart(), {timeout: 20000} ).then(function(response){

				if (parseInt($(response.data).find('CheckInGPSStartResult').children("Processed").text()) == 1)
				{
					var token = $(response.data).find('CheckInGPSStartResult').children("TokenID").text(); 

					if ($rootScope.Beacon){
						var s = soapMessageCheckInGPSEnd(token, $rootScope.Beacon.Store.Latitude, $rootScope.Beacon.Store.Longitude, 0, 2);

						if(globals.debugApp){
							console.log("Debug: Checking in with BEACON coordinates (not with GPS)");
						}
						window.localStorage.setItem('checkInTime', moment().toDate().getTime()); //moment in time that check in module was used
					}
					else{
						var s = soapMessageCheckInGPSEnd(token, $rootScope.Latitude, $rootScope.Longitude, $rootScope.Accuracy, 1);

						if(globals.debugApp){
							console.log("Debug: Checking in with GPS coordinates (not with beacon)");
						}
						window.localStorage.setItem('checkInTime', moment().toDate().getTime()); //moment in time that check in module was used
					}

					$http.post(clientURL, s, {timeout: 20000} ).then(function(response){

						if (parseInt($(response.data).find('CheckInGPSEndResult').children("Processed").text()) == 1){
							$timeout(function(){
								$rootScope.CheckInMessage = "Done";
								if(window.wizSpinner) {
									window.wizSpinner.hide();
								}
								var gameplays = $(response.data).find('CheckInGPSEndResult').children("Plays").text();

								//Saving of Gameplays
								if( globals.localStorage == false){
									setGamePlays(gameplays);
									saveLocalData();
								}
								else{
									window.localStorage.setItem('GamePlays', gameplays); //update the user's game plays since they may have been awared plays from this check in
								}
							});
							navigator.notification.alert(
							    $(response.data).find('CheckInGPSEndResult').children("SuccessMsg").text(),  // message
							    function(){
							    	removeCheckIn();
							    },         // callback
							    'Success!',            // title
							    'Ok'                  // buttonName
							);
						}
						else{
							$rootScope.CheckInMessage = "Done";

							if(window.wizSpinner){
							    window.wizSpinner.hide();
							   }
							alert($(response.data).find('Standard').children("MethodErrorMsg").text(), function(){ //if not processed show the error message from the server
								removeCheckIn();
							});
						}
					}, function(error){

						if(window.wizSpinner) {
							window.wizSpinner.hide();
						}

						$rootScope.canCheckIn = true;

						if(globals.debugApp){
							console.log("DEBUG: Logging out");
						}

						navigator.notification.confirm(
							'Please insure you have good reception with your Wifi turned ON.',
							callBackConfirm, // Specify a function to be called
							'Could not reach server... Retry?', ["Yes", "No"]
						);

						function callBackConfirm(b) {
							if (b == 1) { // for YES
								$scope.CheckIn();
							}
							else{ // for NO 
								$rootScope.CheckInMessage = "Done";
								removeCheckIn();
							}

						}
					});
				}

				else{
					if(window.wizSpinner) {
						window.wizSpinner.hide();
					}
					window.localStorage.setItem('checkInTime', moment().toDate().getTime()); //moment in time that check in module was used

					$rootScope.CheckInMessage = "Done";
					alert($(response.data).find('Standard').children("MethodErrorMsg").text(), function(){
						removeCheckIn();
					});
				}

			}, function(error){
				if(window.wizSpinner) {
					window.wizSpinner.hide();
				}

				$rootScope.canCheckIn = true;

				if(globals.debugApp){
					console.log("DEBUG: Logging out");
				}

				navigator.notification.confirm(
					'Please insure you have good reception with your Wifi turned ON.',
					callBackConfirm, // Specify a function to be called
					'Could not reach server... Retry?', ["Yes", "No"]
				);

				function callBackConfirm(b) {
					if (b == 1) { // for YES
						$scope.CheckIn();
					}
					else{ // for NO 
						$rootScope.CheckInMessage = "Done";
						removeCheckIn();
					}
				}
			});
		}
	};



	function removeCheckIn(){    //helper function for removing the check in now module

		setTimeout(function(){

			$('#nowCheckIn').addClass('lightSpeedOut'); //removes the check in module with animation
			$rootScope.allowCheckIn = false; //hides the check in module according to angular ng-show (see main.html)
			$scope.Trivia = getTrivia();

			if ($rootScope.ReShowTrivia){
				$rootScope.ReShowTrivia = false;
				$rootScope.showTrivia = true;
			}
			else{
				
				if(!ClientSet.getShowFocusPromo()){
					$rootScope.showShare = true;
				}
				else{
					$rootScope.showShare = false;

					if ($scope.Promos){ //if we have promotions

						$rootScope.showFocusPromo = ClientSet.getShowFocusPromo(); //check if client settings allow focus promo
						$scope.FocusPromo = getFocusPromo($scope.Promos);	 //get focus promo (if any)
						
						if($scope.FocusPromo == null){
							$rootScope.showShare = true;
						}
					}
					else{
						$rootScope.showFocusPromo = false;
						$scope.FocusPromo = null;
						//share mod
						$rootScope.showShare = true;
					}

					if ($scope.FocusPromo){ //if there is focus promo then edit styling
						$("#nowFocusPromo .ui-grid-d").css('opacity', 0);
						var u = 'url(' + $scope.FocusPromo.PhotoURL + ')';
						$('#focusPromoImage').css('background', u);
						$('#focusPromoImage').css('background-repeat', 'no-repeat');
						$('#focusPromoImage').css('background-size', 'contain');

						setTimeout(function(){
							$("#nowFocusPromo .ui-grid-d").animate({opacity:1}, 1000);
						}, 50);
					}
				}
			}

			$timeout(function(){
				if(window.wizSpinner){
					window.wizSpinner.hide(); 
				}
				resetScrollbar();
			}, 100);

		}, 800);
	}
	
	var initNowMapInProgress = false; //this prevents 2 initNowMap calls from running at the same time
	function initNowMap(){
	    if (typeof google != 'undefined' && $rootScope.haveGPS && $scope.Stores != null && $scope.Stores.length > 0 && !initNowMapInProgress){ //typeof google != 'undefined' check is necessary for instances where the app does not have good internet and the google api has not loaded, also checks if the user has GPS and stores
	    	//initNowMapInProgress = true;
			var nearestStore = findNearestStore({Latitude:$rootScope.Latitude, Longitude:$rootScope.Longitude}, $scope.Stores); //find nearest store based on user location (stored in $rootScope.Latitude and $rootScope.Longitude) and the Stores list, see findNearestStore function below
			resetScrollbar();
	    }
	}


	function getTrivia(){
		if (window.device.version.substring(0,1) != "2"){ //make sure this is not an Android 2.x device since these devices do not format trivia properly, you should fix the formatting issue and remove the restriction
			
			var triviaTimer = true; // dev flag for trivia timer
			var last = window.localStorage.getItem('Trivia.Last');
			var seen = window.localStorage.getItem('Trivia.Seen');

			if (seen){
				seen = parseInt(seen);
			}
			else{
				seen = 0;
			}

			var now = moment().toDate().getTime();
			var TriviaOrder = window.localStorage.getItem('Trivia.Order');

			if (!last || (now - last > 86400000 || !triviaTimer )){ //make sure the user has not seen trivia within 24 hours
				var triviaRandom = JSON.parse(TriviaOrder); //get the array of indeces generated in js/sync.js
				var triviaIndex = parseInt(window.localStorage.getItem('Trivia.Index'));
				if (triviaIndex == trivia.length){
					triviaIndex = 0; //set to 0 (i.e. start over if the user has already seen all trivia)
					window.localStorage.setItem('Trivia.Index', '0');
				}
				else{
					var t = triviaIndex + 1;
					window.localStorage.setItem('Trivia.Index', t);
				} //get the trivia index and update the locally stored trivia index to 0 or the next index
				window.localStorage.setItem('Trivia.Last', now); //set the last trivia to now
				window.localStorage.setItem('Trivia.Seen', 0); //the user has not yet seen the answer
				return trivia[triviaRandom[triviaIndex]];
			}
			else if (!seen){ //if the user has not seen the trivia answer then show the same one again (unless 24 hours have passed in which case the user will see a new one due to the above if statement)
				var triviaRandom = JSON.parse(TriviaOrder);
				var triviaIndex = parseInt(window.localStorage.getItem('Trivia.Index'));
				return trivia[triviaRandom[triviaIndex]];
			}
			else{
				return null;
			}
		}
		else{
			return null;
		}
	}


	function getFocusPromo(promos){ //return the promotion to be shown in the focus promotion time of day based now module
		var now = moment().hour();
		var message = ""; //holds the message to be shown in the now module based on the time of day
		if (5 <= now && now <= 10){
			now = 1;
			message = "Start Your Morning Right!";
		}
		else if (11 <= now && now <= 16){
			now = 2;
			message = "How About This for Lunch?";
		}
		else if (17 <= now && now <= 22){
			now = 3;
			message = "Join Us for Dinner!";
		}
		else{
			now = 4;
			message = "Late Night Snack?";
		}
		var focusPromos = [];
		for (var i = 0; i < promos.length; i++){
			if (promos[i].FocusTime == now){
				focusPromos.push(promos[i]); //go through all the promos we have (passed into this function) and get the ones that match this focus time
			}
		}
		if (focusPromos.length == 0) //no promos match the current focus time
			return null;
		else{
			var index = Math.floor(Math.random() * ((focusPromos.length - 1) - 0 + 1)) + 0;
			focusPromos[index].FocusMessage = message;
			return focusPromos[index]; //randomly select one focus promo and return it
		}
	}




	function findNearestStore(Position, Stores) //helper function that finds nearest store given the list of stores and the user position
	{
		if(globals.debugApp){
			console.log("Find nearest store was called ( GPS )");
		}

		var shortestDistance = null; //variable to keep track of the shortest distance
		var nearestStore = null; //variable to keep track of the nearest store
		var StoreLatLong = null;
		var PositionLatLng = new google.maps.LatLng(Position.Latitude, Position.Longitude); //create LatLng google maps object for the user current position
		
		for (var i = 0; i < Stores.length; i++){

			if(!globals.debugApp){
				StoreLatLong = new google.maps.LatLng(Stores[i].Latitude, Stores[i].Longitude); //foreach store, create LatLng of the store's position
			}
			else{
				StoreLatLong = new google.maps.LatLng(Position.Latitude, Position.Longitude); // Debug only (it takes the devices coordinates, which will always show the check in module)
			}
			// MR WILLIAMS LATLONG : 36.318860, -78.409849
			// used to test gps check-in -->    
			//var StoreLatLong = new google.maps.LatLng(36.318860, -78.409849); //foreach store, create LatLng of the store's position

			// used to test gps check-in -->    
			//var StoreLatLong = new google.maps.LatLng(Position.Latitude, Position.Longitude); //foreach store, create LatLng of the store's position

			var currentDistance = google.maps.geometry.spherical.computeDistanceBetween (PositionLatLng, StoreLatLong); //get the distance between user and the store
			if (shortestDistance == null || currentDistance < shortestDistance){ //if no shortestdistance yet or if the currentdistance is lower than shortestdistance (i.e. this store is closer than the previous store we saved)
				nearestStore = Stores[i]; //set this store as neareststore
				shortestDistance = currentDistance; //update shortestdistance
			}
		}

		// GPS CHECK IN FUNCTIONALITY 
		$timeout(function(){
			//check if the shortestDistance matches criteria for inStore (based on client settings)
			var inStore = $rootScope.haveGPS && ClientSet.getCheckinGPSEnable() && (ClientSet.getCheckinGPSMaxAccuracy() == 0 || ClientSet.getCheckinGPSMaxAccuracy() > $rootScope.Accuracy) && shortestDistance < ClientSet.getCheckinGPSMaxDistance(); 
			shortestDistance *= 0.000621371; //convert shortestdistance from meters to miles
			shortestDistance = Math.round(shortestDistance);
			$scope.mapHeader = "The Nearest Store Is " + shortestDistance + " Mile"+(shortestDistance>1?'s':'')+" Away.";

			if(globals.debugApp){
				console.log('CHECK IN - DEBUG: ' + $scope.mapHeader);
			}

			//make sure user is registered (so we can update game plays when they check in), instore, and client settings allow check in 
			//(game hide also disables check in module)
			$rootScope.allowCheckIn = $rootScope.RegisteredUser && inStore && ClientSet.getShowCheckIn() && !ClientSet.getGameHide() && ClientSet.getKickbackUser() == null; 

			if(globals.debugApp){
				console.log("CHECK IN - DEBUG:  HaveGPS: " + $rootScope.haveGPS +  " - CheckinGPSenabled" + ClientSet.getCheckinGPSEnable() );
				console.log("CHECK IN - DEBUG: In Store?: " + inStore + " - Get show check in (clientset)?: " + ClientSet.getShowCheckIn());
				console.log("CHECK IN - DEBUG: Allow check in is:" + $rootScope.allowCheckIn );
			}

			$rootScope.CheckInMessage = "@ " + nearestStore.Address1;
	
			if ($rootScope.allowCheckIn){

				if(globals.debugApp){
					console.log("CHECK IN - DEBUG: CHECK IN MODULE has been called **VIA GPS** for store check in.");
				}

				//remove in case this is an app pause/resume and the check in module still has lightspeedout class
				$('#nowCheckIn').removeClass('lightSpeedOut'); 

				if($rootScope.showTrivia)
					$rootScope.ReShowTrivia = true;

				$rootScope.showTrivia = false;
				$rootScope.showShare = false;
				$rootScope.showFocusPromo = false;
			}

			if (inStore){
				//save the store the user is in inside $rootScope.inStore (used for feedback and redemption, see below)
				$rootScope.inStore = nearestStore; 
			}
		}, 1);

		return nearestStore;
	}


    // RATE APP MODULE LOGIC //
	var step = 0; //determines which step the user is in for the rating module

	$scope.Yes = function(){
		if (step == 0){
			step = 1;
			$scope.RatingMessage = "Would You Like to Rate This App?"; //if the user selects yes to are you enjoying this app then change the message
		}
		else if (step == 1){ //user chose yes twice
			removeRating();
			setTimeout(function(){
				if (window.device.platform == 'iOS'){
					window.location.href = 'itms-apps://itunes.apple.com/app/id' + globals.Change.AppleID; //goes to the app store for rating (Change is defined in the chain's javascript file)
				}
				else{
					window.location.href = 'market://details?id=' + globals.Change.AndroidID; //goes to the play store for rating
				}
			}, 100);
		}
		else if (step == -1){ //user chose no (i.e. they do not enjoy the app) and then yes (i.e. they want to leave feedback)
			removeRating();
			setTimeout(function(){
				$.mobile.changePage('#feedback');
			}, 100);
		}
	};

	$scope.No = function(){
		if (step == 0){
			step = -1;
			$scope.RatingMessage = "Would You Like to Leave Feedback?"; //user chose no to first question (are oyu enjoying the app)
		}
		else if (step == 1 || step == -1){
			removeRating(); //if user chooses no for second question then just remove rating regardless of what the user said before (perhaps the user's responses should be saved to the client log since we will not receive this)
		}
	};

	function removeRating(){ //helper function to remove the rating module
		$('#mainPageContent').iscrollview('disable'); //rating modlue is not very friendly with scroller so we disable it
		$('#nowRating').addClass('lightSpeedOut');
		$timeout(function(){
			$rootScope.showRating = false;
			$('#mainPageContent').iscrollview('enable'); //reneable scroller
			resetScrollbar();
		}, 3000);
		window.localStorage.setItem('Rating', 0);
	}



	// CUST SURVEY MODULE LOGIC //

	var canChooseSurvey = true; //variable to determine if the user can choose a survey answer (prevents the user from choosing multiple times)
	
	$scope.One = function(){
		if (canChooseSurvey){

			canChooseSurvey = false;
			setTimeout(function(){
				$('#nowSurvey').addClass('lightSpeedOut');
			}, 3000); //animate out the survey module

			$timeout(function(){
				$('#nowSurvey .ui-block-b').animate({opacity:0.25}, 1000); //fade out the other survey answers that the user did not choose
				$('#nowSurvey .ui-block-c').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-d').animate({opacity:0.25}, 1000);
			}, 1);

			$timeout(function(){
				$rootScope.showSurvey = false;
			}, 5000); //hide the survey module according to angular ng-show

			window.localStorage.setItem('survey.last.question', $scope.Survey.Question);
			saveClientLog(90, 0, 0, '', 'user chose survey answer one'); //save the user selection to client log so that the results can be queried on the database
		}
	};
	$scope.Two = function(){
		if (canChooseSurvey){
			canChooseSurvey = false;

			setTimeout(function(){
				$('#nowSurvey').addClass('lightSpeedOut');
			}, 3000);

			$timeout(function(){
				$('#nowSurvey .ui-block-a').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-c').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-d').animate({opacity:0.25}, 1000);
			}, 1);

			$timeout(function(){
				$rootScope.showSurvey = false;
			}, 5000);

			window.localStorage.setItem('survey.last.question', $scope.Survey.Question);
			saveClientLog(91, 0, 0, '', 0, 'user chose survey answer two');
		}
	};
	$scope.Three = function(){
		if (canChooseSurvey){
			canChooseSurvey = false;

			setTimeout(function(){
				$('#nowSurvey').addClass('lightSpeedOut');
			}, 3000);

			$timeout(function(){
				$('#nowSurvey .ui-block-b').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-a').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-d').animate({opacity:0.25}, 1000);
			}, 1);

			$timeout(function(){
				$rootScope.showSurvey = false;
			}, 5000);

			window.localStorage.setItem('survey.last.question', $scope.Survey.Question);
			saveClientLog(92, 0, 0, '', 0, 'user chose survey answer three');
		}
	};
	$scope.Four = function(){
		if (canChooseSurvey){
			canChooseSurvey = false;

			setTimeout(function(){
				$('#nowSurvey').addClass('lightSpeedOut');
			}, 3000);

			$timeout(function(){
				$('#nowSurvey .ui-block-b').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-c').animate({opacity:0.25}, 1000);
				$('#nowSurvey .ui-block-a').animate({opacity:0.25}, 1000);
			}, 1);

			$timeout(function(){
				$rootScope.showSurvey = false;
			}, 5000);

			window.localStorage.setItem('survey.last.question', $scope.Survey.Question);
			saveClientLog(93, 0, 0, '', 0, 'user chose survey answer four');
		}
	};


	var snapper = new Snap({
	  	element: document.getElementById('triviaQuestion')
	}); //create snapper element using trivia question (library used to slide div out to reveal the underlying answer)
	
	if( $(window).width() > 630 ){
			opts = { maxPosition: 500, minPosition: -500};
			snapper.settings(opts);
	}		

	var Trivia = true; //variable to prevent code bock below from running more than once

	snapper.on('animated', function(){
	//after the user animates the snapper, if the state is not closed (i.e. the user fully 
	//animated the snapper until the answer was fully revealed)
		if (snapper.state().state != 'closed' && Trivia){ //after the user animates the snapper, if the state is not closed (i.e. the user fully animated the snapper until the answer was fully revealed)
			Trivia = false;
			$timeout(function(){
				$rootScope.showTrivia = false; // GET RID OF TRIVIA MODULE

				if(!ClientSet.getShowFocusPromo()){
					$rootScope.showShare = true;
				}
				else{
					if ($scope.Promos)
					{ //if we have promotions
						$rootScope.showFocusPromo = ClientSet.getShowFocusPromo(); //check if client settings allow focus promo
						$scope.FocusPromo = getFocusPromo($scope.Promos);	 //get focus promo (if any)
						
						if($scope.FocusPromo == null)
						{
							$rootScope.showShare = true;
						}
					}
					else{
						$rootScope.showFocusPromo = false;
						$scope.FocusPromo = null;

						//share mod
						$rootScope.showShare = true;
					}
		
					if ($scope.FocusPromo){ //if there is focus promo then edit styling
						$("#nowFocusPromo .ui-grid-d").css('opacity', 0);
						var u = 'url(' + $scope.FocusPromo.PhotoURL + ')';
						$('#focusPromoImage').css('background', u);
						$('#focusPromoImage').css('background-repeat', 'no-repeat');
						$('#focusPromoImage').css('background-size', 'contain');
		
						setTimeout(function(){
							$("#nowFocusPromo .ui-grid-d").animate({opacity:1}, 1000);
						}, 50);
					}
				}
			}, 5000);

			window.localStorage.setItem('Trivia.Seen', 1); //set the localStorage for trivia.seen to 1
		}
	});

	$scope.PlayNow = function(){

		if($rootScope.RegisteredUser){
			$.mobile.changePage('#fun');
			connectAppFooterService.navIcons[1].selected = true;
			connectAppFooterService.navIcons[0].selected = false;
		}

		if(!$rootScope.RegisteredUser){
			navigator.notification.confirm(
				'You must be a registered user to access the Game and Prizes. Would you like to register or sign in now?',
				callBackFunctionRedeemRegister, // Specify a function to be called
				'Registration Required', ["Yes", "No"]
			);

			function callBackFunctionRedeemRegister(b) {
				if (b == 1) {
					$.mobile.changePage('#redeemRegisterAlert');
					$('#redeemRegisterAlert_back').show();
					$rootScope.redeemRegisterAlerted = true;
				}
			}
		}

	}; //function for gameplays module play now button


	$rootScope.Game = function(){
		if ($rootScope.RegisteredUser){
			if ($rootScope.electrumPoints && !getElectrumUniqueID()){
				if ($rootScope.electrumMenuMain){
					var message = $rootScope.electrumMenuMain + 'account required to enjoy the game!';
				}
				else{
					var message = 'Account required to enjoy the game!';
				}
				
				navigator.notification.alert(
					message,  // message
					function(){
						$('.ui-panel').panel('close');
					},         // callback
					'Enrollment Required',            // title
					'Ok'                  // buttonName
				);
			}
			else{
				$.mobile.changePage('#fun');
				connectAppFooterService.navIcons[1].selected = true;
				connectAppFooterService.navIcons[0].selected = false;
			}
		}
		else{
			navigator.notification.confirm(
				'You must be a registered user to access the Game and Prizes. Would you like to register or sign in now?',
				callBackFunctionRedeemRegister, // Specify a function to be called
				'Registration Required', ["Yes", "No"]
			);

			function callBackFunctionRedeemRegister(b) {
				if (b == 1) {
					$.mobile.changePage('#redeemRegisterAlert');
					$('#redeemRegisterAlert_back').show();
					$rootScope.redeemRegisterAlerted = true;
				}
			}
		}
	}; //global function for the Fun button in the panel, if the user is registered it changes to the fun page (unless the user is already on the fun page then it just closes the panel, see main.html) .. if the user is not registered then it closes the panel after showing the dialog
});

function navigateTo (lat, lon, successFn, errorFn) {
    cordova.require('cordova/plugin/phonenavigator').doNavigate(lat, lon, successFn, errorFn);
}


})();   // END of Script //
