(function () {
	angular.module('connect.electrum-rewards').factory('electrumRewardsService', electrumRewardsService);

	electrumRewardsService.$inject = ['$http', '$q', '$timeout', '$window', '$rootScope', 'ClientSet', 'ElectrumUser', 'ElectrumRegistrationUser', 'ElectrumPunchCard', 'ElectrumReward'];
	function electrumRewardsService($http, $q, $timeout, $window, $rootScope, ClientSet, ElectrumUser, ElectrumRegistrationUser, ElectrumPunchCard, ElectrumReward) {

		var electrumUser;
		var electrumPassword;
		var electrumEndpoint;

		// ios is 1000 / android 1700
		var timer;
		if(device.platform === "iOS"){
			timer = 1200;
		}
		else{
			timer = 5000;
			console.log('Android loading service is being loaded. local data should have been initialize');
		}

		setTimeout(function() {
			//Credentials for the CHAIN 
			console.log('electrum-DEBUG: Getting credentials from client sets');
			//electrumUser = ClientSet.getElectrumUser();//'handymart',//ClientSet.getKickbackUser()   //MAKE THESE !!!  ** NOTE **
			//electrumPassword = ClientSet.getElectrumPass();//'e8c8164c26a20072aba1f274edccdd01'
			//electrumEndpoint = ClientSet.getElectrumURL();//'https://www.rewardsbutler.com/api/';
			console.log('electrum-DEBUG: electrum Credentials were initialized.. User : ' + electrumUser);
		}, timer);
		


			var electrumStateLocalName = 'connect-electrum-state',
				electrumState,
				publicElectrumState = {};

			var service = {
				getElectrumState: getElectrumState,
				register: register,
				signout: signout,
				signin: signin,
				checkRegistered: checkRegistered,
				getBalance: getBalance,
				cardFromPhone: cardFromPhone,
				verifyPin: verifyPin,
				getInfo: getInfo,
				getPunchStatus: getPunchStatus,
				securityquestions: securityquestions,
				updateService: updateService
			};

			console.log("electrum-DEBUG: initiating service for new electrum rewards");

			loadLocalState();//kick off the state load

			return service;


		function updateService(user, pass, endpoint){
			console.log("electrum-DEBUG: Updated service with: " + user + ' ' + pass + ' ' + endpoint);
			electrumUser = user,
			electrumPassword = pass,
			electrumEndpoint = endpoint;
		}



		function callElectrumAPI(method, data) {

			console.log("electrum: API call made for " + method);
			return $q(function (resolve, reject) {
				var dataToSend = {};

				console.log("electrum: Data being sent " + JSON.stringify(data) );
				for (var key in data) {
					if (data.hasOwnProperty(key)) {
						if (data[key] !== null && data[key] !== undefined && typeof data[key] !== 'function') {
							dataToSend[key] = data[key];
						}
					}
				}
				dataToSend.user = electrumUser;
				dataToSend.pass = electrumPassword;
				dataToSend.call = method;

				console.log("electrum-DEBUG: Data being sent has the credentials of : " + dataToSend.user + ' - ' + dataToSend.pass + ' - ' + dataToSend.call );

				
				$http({
					method: 'POST',
					url: electrumEndpoint,
					params: dataToSend,
					data: $.param(dataToSend),
					headers: {
						'Content-Type': 'application/x-www-form-urlencoded',
						'Access-Control-Allow-Origin': '*'
					}
				}).then(function (result) {
					if (result && result.status === 200 && result.data) {
						if (result.data.status === 0)
							return resolve(result.data.data);
						return reject(result.data);
					}
					return reject(result);
				}, function (err, e, x) {
					console.log("electrum-ERROR: error of API service for electrum -- " + err);
					reject(err);
				});
				dataToSend = null;
			});
		}

		/**
		 * @description save the service electrumState into the local storage
		 */
		function saveLocalState() {
			console.log('electrum-DEBUG: Saving local state in Electrum-Rewards.Service');
			$window.localStorage.setItem(electrumStateLocalName, JSON.stringify(electrumState));
			syncPublicElectrumState();
		}

		/**
		 * @description loads the local electrum State into the service
		 */
		function loadLocalState() {
			console.log('electrum-DEBUG: loading local state in Electrum-Rewards.Service');
			electrumState = JSON.parse($window.localStorage.getItem(electrumStateLocalName));
			if (!electrumState) {
				electrumState = defaultElectrumState();
				saveLocalState();
			} else {
				electrumState.electrumUser = new ElectrumUser(electrumState.electrumUser);
			}
			syncPublicElectrumState();
		}
		
		/**
		 * @description returns a copy of the private electrum State Variable 
		 */
		function getElectrumState() {
			return publicElectrumState;
		}

		function syncPublicElectrumState() {
			publicElectrumState.registered = electrumState.registered;
			if (!publicElectrumState.electrumUser) {
				publicElectrumState.electrumUser = new ElectrumUser(electrumState.electrumUser);
			} else {
				publicElectrumState.electrumUser.$merge(electrumState.electrumUser);
			}
		}
		
		/**
		 * @description returns the default electrumState Object
		 */
		function defaultElectrumState() {
			console.log('electrum-DEBUG: setting default state.');
			window.localStorage.setItem('Electrum.SignedIn', 0);
			return {
				registered: false,
				electrumUser: new ElectrumUser({})
			}
		}

		function signin(CardNumber, PIN, LastName) {
			return $q(function (resolve, reject) {
				service.verifyPin(CardNumber, PIN, LastName).then(function () {
					electrumState.registered = true;
					// Save into local state
					window.localStorage.setItem('Electrum.SignedIn', 1);
					electrumState.electrumUser = new ElectrumUser({
						CardNumber: CardNumber,
						PIN: PIN,
						LastName: LastName
					});
					console.log('electrum-DEBUG: User is signed in and user model was constructed.');
					saveLocalState();
					resolve();
				}, reject);
			});
		}

		/**
		 * @description registers a new electrum user
		 * @param electrum user electrum user the user object to register
		 */
		function register(electrumUserToRegister) {
			return $q(function (resolve, reject) {
				electrumUserToRegister = new ElectrumRegistrationUser(electrumUserToRegister);//cast just in case
				if (electrumUserToRegister.BirthDate) {
					electrumUserToRegister.BirthDate = moment(electrumUserToRegister.BirthDate).format('YYYY-MM-DD');
				}
				else{
					electrumUserToRegister.BirthDate = '0000-00-00';
				}

				console.log('electrum-DEBUG: Mobile phone of User that is registering = ' + electrumUserToRegister.mobile)
				
				callElectrumAPI('register', electrumUserToRegister).then(function (response) {
					electrumState.registered = true;
					window.localStorage.setItem('Electrum.SignedIn', 1);
					electrumState.electrumUser = new ElectrumUser({
						CardNumber: electrumUserToRegister.cardnumber,
						PIN: electrumUserToRegister.PIN,
						FirstName: electrumUserToRegister.FirstName,
						LastName: electrumUserToRegister.LastName,
						Email: electrumUserToRegister.Email,
						mobile: electrumUserToRegister.mobile
					});

					console.log('electrum-DEBUG: User registered and user model was constructed.');
					saveLocalState();
					return resolve();
				}, function (err) {
					console.log('electrum-ERROR: User could not registered : ' + err);
					electrumState.registered = false;
					window.localStorage.setItem('Electrum.SignedIn', 0);
					saveLocalState();
					return reject(err);
				});
			});

		}
		
		/**
		 * @description clears local storage variables effectively signing out
		 */
		function signout() {
			electrumState.registered = false;
			window.localStorage.setItem('Electrum.SignedIn', 0);
			electrumState.electrumUser = new ElectrumUser({});
			saveLocalState();
		}
		
		/**
		 * @description checks if the cardnumber is registered
		 * @param cardnumber int the card number to check if registered
		 */
		function getBalance(cardnumber) {
			return callElectrumAPI('getbalance', { cardnumber: cardnumber });
		}
		
		/**
		 * @description checks if the cardnumber and if the pin is valid
		 * @param cardnumber int the card number to check if registered
		 */
		function verifyPin(cardnumber, PIN, LastName) {
			return callElectrumAPI('verifyPin', { cardnumber: cardnumber, PIN: PIN, LastName: LastName });
		}
		
		/**
		 * @description gets the information about the electrum user
		 * @param cardnumber int the card number to check if registered
		 */
		function getInfo(cardnumber, PIN) {
			return callElectrumAPI('getinfo', { cardnumber: cardnumber, PIN: PIN });
		}
		
		/**
		 * @description checks if the cardnumber is registered
		 * @param cardnumber int the card number to check if registered
		 */
		function checkRegistered(cardnumber) {
			return callElectrumAPI('check_registered', { cardnumber: cardnumber });
		}

		/**
		 * @description checks if there is a card number registered to a phone number
		 * @param phone int the phone number to check cards registered against
		 */
		function cardFromPhone(phone) {
			return callElectrumAPI('card_from_phone', { phone: phone });
		}
		
		/**
		 * @description gets a list of availabile secuirty questions to use on register
		 * @param cardnumber
		 */
		function securityquestions(cardnumber) {
			return callElectrumAPI('securityquestions', { cardnumber: cardnumber });
		}
		
		/**
		 * @description gets the punch status of the cards
		 * @param cardnumber
		 * @param PIN
		 * @param LastName
		 * @returns array(ElectrumPunchCard)
		 * TODO locationID? is this needed, the API errors if not given
		 */
		function getPunchStatus(cardnumber, PIN, LastName) {
			return $q(function (resolve, reject) {
				return callElectrumAPI('getpunchstatus', { cardnumber: cardnumber, PIN: PIN, LastName: LastName, locationID: '123456' }).then(function (punchCards) {
					var i,
						len = punchCards.length,
						electrumPunchCards = new Array(len);
					for (i = 0; i < len; i++) {
						electrumPunchCards[i] = new ElectrumPunchCard(punchCards[i]);
					}
					resolve(electrumPunchCards);
				}, reject);
			});
		}
		
		/**
		 * @description checks if there is a card number registered to a phone number
		 * @param cardnumber
		 */
		function listRewards(cardnumber) {
			return $q(function (resolve, reject) {
				return callElectrumAPI('listrewards', { cardnumber: cardnumber }).then(function (rewards) {
					var i,
						len = rewards.length,
						electrumRewards = new Array(len);
					for (i = 0; i < len; i++) {
						electrumRewards[i] = new ElectrumReward(rewards[i]);
					}
					resolve(electrumRewards);
				}, reject);
			});
		}

	}
} ());