(function() {
	angular.module('connect.interestList')
		.directive('connectInterestList', connectInterestListDirective)
		.controller('connectInterestListController', connectInterestListController);

	connectInterestListDirective.$inject = [];
	connectInterestListController.$inject = ['$scope', '$rootScope', '$http', '$timeout', 'Loading', 'ClientSet'];

	function connectInterestListDirective() {
		var directive = {
			restrict: 'E',
			templateUrl: 'interestList/connect.interestlist.html',
			controller: 'connectInterestListController'
		};
		return directive;
	}

	function connectInterestListController ($scope, $rootScope, $http, $timeout, Loading, ClientSet) { //controller for managing user interests

		Loading.then(function() {
			$timeout(function() {
				console.log("Interested Controller loaded");
			
				$scope.InterestTitle = ClientSet.getInterestTitle();
				$scope.InterestHeader = ClientSet.getInterestHeader(); //set the interest page title and header based on client set (see main.html for where they actually show on the page)
			},1);
		});

		$('#interestList').on('pageshow', function(){
			$('#interestsPageContent').css('height', $(window).height() - 53);
			$('#interestList').trigger('create');	//create the interestlist every time to show the latest interest
		});

		$('#interestList').on('pageinit', function() { //on page init fill up the interests on the page
			$timeout(function() {
				//TODO: JK 5/10/2015 This will cause memory leaks when routing is updated. 
				//This needs to be unbinded on $scope.$destroy
				$scope.Interests = getInterests();
			}, 1);
		});

		var previousPage = '';
		$('#interestList').on('pageshow', function(event, data) {
			previousPage = data.prevPage.attr('id');
		});

		$('#interestList').on('pagehide', function() { //refresh the interests on the page on pagehide in case the user has updated them
			$timeout(function() {
				$scope.Interests = getInterests();
			}, 1);
		});

		$scope.Interests = getInterests(); //called on app startup (after Angular is bootstrapped and controllers begin executing)

		$scope.SaveInterests = function() {
			var interests = ''; //variable to hold the interests to be saved, formatted for the SOAP message
			var interestCount = 0; //variable to hold the number of interests to be saved in case they are above the max
			$(".interestInput:checked").each(function(index) {
				interests = interests + '<RAccountInterestServer><InterestID>' + $(this).attr('id') + '</InterestID></RAccountInterestServer>';
				interestCount++;
			});
			if (interestCount <= ClientSet.getInterestMax()) { //make sure interests are less than or equal to the max defined by the client setting
				window.wizSpinner.show();
				$http.post(GetItemFromLocalData('ClientURL'), soapMessageSaveAccountInterest(interests), {timeout: 10000}).then(function(response) {
					window.wizSpinner.hide();
					var processed = $(response.data).find("Processed").text();
					var error = $(response.data).find('Standard').children('MethodErrorMsg').text();
					if (processed == 'true') {
						navigator.notification.alert(
							'Interests saved.', // message
							function() {}, // callback
							'Success!', // title
							'Ok' // buttonName
						);
						
						if (previousPage == 'accountDetails') //if the user came to interests from the accountDetails screen then take them back there
							$.mobile.back();
						else
							$.mobile.changePage('#main');
						saveInterests(response.data); //save interests to local storage, this function is implemented in js/sync.js
					} else
						alert(error, function() {
							if (previousPage == 'accountDetails')
								$.mobile.back();
							else
								$.mobile.changePage('#main');
						});
				}, function(error) {
					window.wizSpinner.hide();
					$rootScope.Error(globals.ajaxTimeoutMessage, function() {
						if ($rootScope.redeemRegisterAlerted)
							$.mobile.changePage('#promodog');
						else if (previousPage == 'accountDetails')
							$.mobile.back();
						else
							$.mobile.changePage('#main');
					}, false, function() {});
				});
			} else { //if the user is trying to save more interests than max then show the interest overmax message (client setting)
				navigator.notification.alert(
					ClientSet.getInterestOverMax(), // message
					function() {
						setTimeout(function() {
							savingInterests = true;
						}, 1000);
					}, // callback
					'Error', // title
					'Ok' // buttonName
				);
			}
		};
	}
}());