(function () {
	angular.module('connect.kickback-rewards').factory('kickbackRewardsService', kickbackRewardsService);

	kickbackRewardsService.$inject = ['$http', '$q', '$timeout', '$window', '$rootScope', 'ClientSet', 'KickbackUser', 'KickbackRegistrationUser', 'KickbackPunchCard', 'KickbackReward'];
	function kickbackRewardsService($http, $q, $timeout, $window, $rootScope, ClientSet, KickbackUser, KickbackRegistrationUser, KickbackPunchCard, KickbackReward) {

		var kickbackUser;
		var kickbackPassword;
		var kickbackEndpoint;

		// ios is 1000 / android 1700
		var timer;
		if(device.platform === "iOS") {
			timer = 1200;  
		}
		else {
			timer = 5000;
			if(globals.debugApp){
				console.log('Android loading service is being loaded. local data should have been initialize');
			}
		}

		setTimeout(function() {
			//Credentials for the CHAIN 
			kickbackUser = ClientSet.getKickbackUser(); // 'handymart',//ClientSet.getKickbackUser()
			kickbackPassword = ClientSet.getKickbackPass(); // 'e8c8164c26a20072aba1f274edccdd01'
			kickbackEndpoint = ClientSet.getKickbackURL(); // 'https://www.rewardsbutler.com/api/';
			if(globals.debugApp){
				console.log('KICKBACK-DEBUG: Kickback Credentials were initialized.. User : ' + kickbackUser);
				console.log('KICKBACK-DEBUG: Getting credentials from client sets');
			}
		}, timer);
		


			var kickbackStateLocalName = 'connect-kickback-state',
				kickbackState,
				publicKickbackState = {};

			var service = {
				getKickbackState: getKickbackState,
				register: register,
				signout: signout,
				signin: signin,
				checkRegistered: checkRegistered,
				getBalance: getBalance,
				cardFromPhone: cardFromPhone,
				verifyPin: verifyPin,
				getInfo: getInfo,
				getPunchStatus: getPunchStatus,
				securityquestions: securityquestions,
				updateService: updateService
			};

			if(globals.debugApp){
				console.log("KICKBACK-DEBUG: initiating service for Kickback");
			}

			loadLocalState();// kick off the state load

			return service;


		function updateService(user, pass, endpoint){
			console.log("KICKBACK-DEBUG: Updated service with: " + user + ' ' + pass + ' ' + endpoint);
			kickbackUser = user,
			kickbackPassword = pass,
			kickbackEndpoint = endpoint;
		}



		function callKickbackAPI(method, data) {

			if(globals.debugApp){
				console.log("Kickback: API call made for " + method);
			}

			return $q(function (resolve, reject) {
				var dataToSend = {};

				console.log("Kickback: Data being sent " + JSON.stringify(data) );
				for (var key in data) {
					if (data.hasOwnProperty(key)) {
						if (data[key] !== null && data[key] !== undefined && typeof data[key] !== 'function') {
							dataToSend[key] = data[key];
						}
					}
				}
				dataToSend.user = kickbackUser;
				dataToSend.pass = kickbackPassword;
				dataToSend.call = method;

				if(globals.debugApp){
					console.log("KICKBACK-DEBUG: Data being sent has the credentials of : " + dataToSend.user + ' - ' + dataToSend.pass + ' - ' + dataToSend.call );
				}
				
				$http({
					method: 'POST',
					url: kickbackEndpoint,
					params: dataToSend,
					data: $.param(dataToSend),
					headers: {
						'Content-Type': 'application/x-www-form-urlencoded',
						'Access-Control-Allow-Origin': '*'
					}
				}).then(function (result) {
					if (result && result.status === 200 && result.data) {
						if (result.data.status === 0)
							return resolve(result.data.data);
						return reject(result.data);
					}
					return reject(result);
				}, function (err, e, x) {
					if(globals.debugApp){
						console.log("KICKBACK-ERROR: error of API service for Kickback -- " + err);
					}
					reject(err);
				});
				dataToSend = null;
			});
		}

		/**
		 * @description save the service kickbackState into the local storage
		 */
		function saveLocalState() {
			//if(globals.debugApp){
				console.log('KICKBACK-DEBUG: Saving local state in Kickback-Rewards.Service');
			//}
			$window.localStorage.setItem(kickbackStateLocalName, JSON.stringify(kickbackState));
			syncPublicKickbackState();

			var tempKickbackSignedIn = globals.userAccount.get('KickbackSignedIn')
			if(getKickbackSignedIn() == 1){
				kickbackGameEvoke(ClientSet);
				console.log('KB user account exist');
			} //update game scope (URL and card #)
		}

		/**
		 * @description loads the local kickbackState into the service
		 */
		function loadLocalState() {
			if(globals.debugApp){
				console.log('KICKBACK-DEBUG: loading local state in Kickback-Rewards.Service');
			}

			kickbackState = JSON.parse($window.localStorage.getItem(kickbackStateLocalName));

			if (!kickbackState) {
				kickbackState = defaultKickbackState();
				saveLocalState();
			} 
			else {
				kickbackState.kickbackUser = new KickbackUser(kickbackState.kickbackUser);
			}

			syncPublicKickbackState();
		}
		
		/**
		 * @description returns a copy of the private kickbackState Variable 
		 */
		function getKickbackState() {
			return publicKickbackState;
		}

		function syncPublicKickbackState() {
			publicKickbackState.registered = kickbackState.registered;
			if (!publicKickbackState.kickbackUser) {
				publicKickbackState.kickbackUser = new KickbackUser(kickbackState.kickbackUser);
			} 
			else {
				publicKickbackState.kickbackUser.$merge(kickbackState.kickbackUser);
			}
		}
		
		/**
		 * @description returns the default kickbackState Object
		 */
		function defaultKickbackState() {
			if(globals.debugApp){
				console.log('KICKBACK-DEBUG: setting default state.');
			}

			window.localStorage.setItem('Kickback.SignedIn', 0);
			return {
				registered: false,
				kickbackUser: new KickbackUser({})
			}
		}

		function signin(CardNumber, PIN, LastName) {
			return $q(function (resolve, reject) {
				service.verifyPin(CardNumber, PIN, LastName).then(function () {
					kickbackState.registered = true;
					// Save into local state
					window.localStorage.setItem('Kickback.SignedIn', 1);
					kickbackState.kickbackUser = new KickbackUser({
						CardNumber: CardNumber,
						PIN: PIN,
						LastName: LastName
					});
					if(globals.debugApp){
						console.log('KICKBACK-DEBUG: User is signed in and user model was constructed.');
					}

					saveLocalState();
					resolve();
				}, reject);
			});
		}

		/**
		 * @description registers a new kickback user
		 * @param kickbackUser KickbackUser the user object to register
		 */
		function register(kickbackUserToRegister) {
			return $q(function (resolve, reject) {
				kickbackUserToRegister = new KickbackRegistrationUser(kickbackUserToRegister);//cast just in case
				if (kickbackUserToRegister.BirthDate) {
					kickbackUserToRegister.BirthDate = moment(kickbackUserToRegister.BirthDate).format('YYYY-MM-DD');
				}
				else{
					kickbackUserToRegister.BirthDate = '0000-00-00';
				}
				if(globals.debugApp){
					console.log('KICKBACK-DEBUG: Mobile phone of User that is registering = ' + kickbackUserToRegister.mobile)
				}
				
				callKickbackAPI('register', kickbackUserToRegister).then(function (response) {
					kickbackState.registered = true;
					window.localStorage.setItem('Kickback.SignedIn', 1);
					kickbackState.kickbackUser = new KickbackUser({
						CardNumber: kickbackUserToRegister.cardnumber,
						PIN: kickbackUserToRegister.PIN,
						FirstName: kickbackUserToRegister.FirstName,
						LastName: kickbackUserToRegister.LastName,
						Email: kickbackUserToRegister.Email,
						mobile: kickbackUserToRegister.mobile
					});

					if(globals.debugApp){
						console.log('KICKBACK-DEBUG: User registered and user model was constructed.');
					}

					saveLocalState();
					return resolve();
				}, function (err) {
					if(globals.debugApp){
						console.log('KICKBACK-ERROR: User could not registered : ' + err);
					}
					
					kickbackState.registered = false;
					window.localStorage.setItem('Kickback.SignedIn', 0);
					saveLocalState();
					return reject(err);
				});
			});

		}
		
		/**
		 * @description clears local storage variables effectively signing out
		 */
		function signout() {
			kickbackState.registered = false;
			window.localStorage.setItem('Kickback.SignedIn', 0);
			kickbackState.kickbackUser = new KickbackUser({});
			saveLocalState();
		}
		
		/**
		 * @description checks if the cardnumber is registered
		 * @param cardnumber int the card number to check if registered
		 */
		function getBalance(cardnumber) {
			return callKickbackAPI('getbalance', { cardnumber: cardnumber });
		}
		
		/**
		 * @description checks if the cardnumber and if the pin is valid
		 * @param cardnumber int the card number to check if registered
		 */
		function verifyPin(cardnumber, PIN, LastName) {
			return callKickbackAPI('verifyPin', { cardnumber: cardnumber, PIN: PIN, LastName: LastName });
		}
		
		/**
		 * @description gets the information about the kickback user
		 * @param cardnumber int the card number to check if registered
		 */
		function getInfo(cardnumber, PIN) {
			return callKickbackAPI('getinfo', { cardnumber: cardnumber, PIN: PIN });
		}
		
		/**
		 * @description checks if the cardnumber is registered
		 * @param cardnumber int the card number to check if registered
		 */
		function checkRegistered(cardnumber) {
			return callKickbackAPI('check_registered', { cardnumber: cardnumber });
		}

		/**
		 * @description checks if there is a card number registered to a phone number
		 * @param phone int the phone number to check cards registered against
		 */
		function cardFromPhone(phone) {
			return callKickbackAPI('card_from_phone', { phone: phone });
		}
		
		/**
		 * @description gets a list of availabile secuirty questions to use on register
		 * @param cardnumber
		 */
		function securityquestions(cardnumber) {
			return callKickbackAPI('securityquestions', { cardnumber: cardnumber });
		}
		
		/**
		 * @description gets the punch status of the cards
		 * @param cardnumber
		 * @param PIN
		 * @param LastName
		 * @returns array(KickbackPunchCard)
		 * TODO locationID? is this needed, the API errors if not given
		 */
		function getPunchStatus(cardnumber, PIN, LastName) {
			return $q(function (resolve, reject) {
				return callKickbackAPI('getpunchstatus', { cardnumber: cardnumber, PIN: PIN, LastName: LastName, locationID: '123456' }).then(function (punchCards) {
					var i,
						len = punchCards.length,
						kickbackPunchCards = new Array(len);
					for (i = 0; i < len; i++) {
						kickbackPunchCards[i] = new KickbackPunchCard(punchCards[i]);
					}
					resolve(kickbackPunchCards);
				}, reject);
			});
		}
		
		/**
		 * @description checks if there is a card number registered to a phone number
		 * @param cardnumber
		 */
		function listRewards(cardnumber) {
			return $q(function (resolve, reject) {
				return callKickbackAPI('listrewards', { cardnumber: cardnumber }).then(function (rewards) {
					var i,
						len = rewards.length,
						kickbackRewards = new Array(len);
					for (i = 0; i < len; i++) {
						kickbackRewards[i] = new KickbackReward(rewards[i]);
					}
					resolve(kickbackRewards);
				}, reject);
			});
		}

	}
} ());