//      SUMMARY:      
//the loading factory is the main factory for the application, the sync process gets called here ...
//many controllers rely on the Loading factory promise to be resolved which indicates that the 
//sync process is complete and that data is available for controllers to access (or the process 
//eited due to an ajax error or initialSyncDoneTimeout caused the factory promise to be resolved
//and the loading page to hide but the sync is still running in the background, if this is case 
//the user might see slightly outdated data)
(function() {
	angular.module('connect.loading')
	.factory('Loading', loadingService);

	loadingService.$inject = ['$http', '$q', '$rootScope', '$timeout', 'Sync'];


	function loadingService($http, $q, $rootScope, $timeout, Sync) {
		
		setTimeout(function() {
			globals.Change = setChain();
		});

		var deferred = $q.defer(); //Angular's promise library
		
		//The following timeout is needed
		setTimeout(function() {
			globals.Stores = getAllStores(); //Storelist
			setupLocalData($rootScope, $timeout); //defined in js/sync.js
		}, 600); 


		function resolve(){
			if(globals.debugApp){
				console.log("Calling deferred.resolved())");
			}

			$timeout(function(){
				deferred.resolve(); //resolve the Loading promise
			},1);
		}


		// FUNCTION: SETUP LOCAL DATA //
		//function called before syncing setup some data
		function setupLocalData($rootScope, $timeout) {
			//Save ClientURL first (high priority)
			SetItemInLocalData('ClientURL', globals.Change.server + 'Market_' + globals.Change.chain + '_001/WebService/WebService1.asmx' );
			if(globals.debugApp){
		   	 	console.log( 'ClientURL used: ' + GetItemFromLocalData('ClientURL') );
		   		console.log("SetUpLocalData initiated..");
		   		console.log("App version " + globals.appVersion );
		   	}

		    saveClientLog(48,0,0, "SetUpLocalData initiated..");
		    setSyncVersion();//Set sync version variable
			
			var oldAppVersion = getAppVersion();
			var first = moment().toDate().getTime(); //gets the current time in Unix milliseconds
			//window.localStorage.setItem('Rating', first); //Sets Rating to the first app launch time in Unix milliseconds so we can determine when to show the Rating now module (one week from initial app launch)

		    if(oldAppVersion < globals.appVersion){
		    	if(globals.debugApp){
		    		console.log("App Updated: updated to " + globals.appVersion + " from " + oldAppVersion + ".. Full cleanup/sync will take place.");
		    	}
		    	saveClientLog(48,0,0, "App Updated: updated to " + globals.appVersion + " from " + oldAppVersion + ".. Full cleanup/sync will take place.");
		    	globals.initialSyncDoneTimeout = 30000;

				SetItemInLocalData('DataVersion',null);
				SetItemInLocalData('GameVersion',null);
				SetItemInLocalData('SyncNow',null);

		    	clearStoreAndPromoData();
		    }
		    else if ( getSyncNow() == true ){
		    	if(globals.debugApp){
		    		console.log("Sync now was activated: .. Full cleanup/sync will take place.");
		    	}
		    	saveClientLog(49,0,0, "Sync now was activated: .. Full cleanup/sync will take place.");
				globals.initialSyncDoneTimeout = 30000;

				SetItemInLocalData('DataVersion',null);
				SetItemInLocalData('GameVersion',null);
				SetItemInLocalData('SyncNow',null);

				clearStoreAndPromoData();
			}
			else {
				globals.initialSyncDoneTimeout = 20000;
			}

			//needed for the loading of clearStoreAndPromoData();
			setTimeout(function(){

				SetItemInLocalData('ReleaseDate',globals.releaseDate);
				SetItemInLocalData('AppVersion',globals.appVersion);
				SetItemInLocalData('PromosConstruct',0);
				SetItemInLocalData('StoreListRetry',0);
				SetItemInLocalData('PromoRetry',0);
				SetItemInLocalData('DeviceModel',window.device.model);
				SetItemInLocalData('DeviceType','mobile');
				SetItemInLocalData('DeviceVersion',window.device.version);
				SetItemInLocalData('DeviceOS',window.device.platform);

				//$rootScope.RegisteredUser is used throughout js/app.js (Angular) to indicate
				//whether or not this is a registered user. This is not Angular code but we have access to $rootScope since it was passed into the setupLocalData function
				$rootScope.RegisteredUser = getRegisteredUser();
				if(globals.debugApp){
					console.log( '() DEBUG: Setting $rootScope.RegisteredUser in setupLoacalData() to ' + $rootScope.RegisteredUser + ' for acctID - ' + getAccountID() );
				}
				//if client settings have not been saved to local storage (for example they have not yet been downloaded from the server and this is the first app launch), then load them into 
				//localStorage from installClientSets all install Arrays are in js/installTables.js
				// ** LOAD ClientSets ** //
				var cs =  LoadClientSetsFromAppData();

				
				//Test to see if cs (clientsets) is empty
				if (!cs || Object.getOwnPropertyNames(cs).length < 2){
					cs = {};
					//console.log('DEBUG-SYNC: globals.installClientSets == ' + JSON.stringify(installClientSets) );
					for (var h = 0; h < globals.installClientSets.length; h++){
						cs[globals.installClientSets[h].ItemName.toLowerCase()] = globals.installClientSets[h].ItemValue;
					}
					SaveClientSetsToAppData( JSON.stringify(cs) );
					if(globals.debugApp){
						console.log('DEBUG-SYNC: Clientset was not saved..  Using Defaults. : ' + JSON.stringify(cs));
					}
				} else{
					if(globals.debugApp){
						console.log('DEBUG-SYNC: Clientset were prev saved.. Skip Defaults.' + JSON.stringify(cs));
					}
				}
			

				//detects if the UDID has been set (i.e. is this the first time using the app and no UDID has been saved from a legacy Connect 1.x app, 
				//see js/app.js for more about saving UDID and AccountID from legacy Connect customers)
				if (GetItemFromLocalData('UDID') === null || GetItemFromLocalData('UDID') === ''){	
					if (window.device.platform == 'Android')
			   			setUDID('D' + uuid.v1());
			   		if (window.device.platform == 'iOS')
			   			setUDID('A' + uuid.v1()); //set the UDID based on the device OS, and a generated GUID
				}

				if (window.localStorage.getItem('Trivia.Order') === null){ //Trivia is still in localstorage
					var order = [];
					for (var i = 0; i < trivia.length; i++){
						order.push(i);
					}
					order.sort(function() {
					  return 0.5 - Math.random();
					});

					window.localStorage.setItem('Trivia.Order', JSON.stringify(order));
					window.localStorage.setItem('Trivia.Index', '0');
					//Randomly generate the order for the Trivia module questions/answers, set the index to 0 so that the user sees the first item in the array
				}

				if( GetItemFromLocalData('AccountID') === null ){ 
					if(globals.debugApp){
						console.log('DEBUD: Setting Account info blank for New User.. ');
					}

					SetItemInLocalData('AccountID', '');
					SetItemInLocalData('FullName', '');
					SetItemInLocalData('Phone', '');
					SetItemInLocalData('UniqueID', '');
					SetItemInLocalData('Points', '');
					SetItemInLocalData('GamePlays', '');
					SetItemInLocalData('UserType', '');
					SetItemInLocalData('UserStatus', '');
					SetItemInLocalData('GeoPromo', '');
					SetItemInLocalData('Birthday', 0);
					SetItemInLocalData('NewPlayer', 1);

					if (GetItemFromLocalData('DataVersion') === null) //The DataVersion determines whether or not the app requires a download sync or just a regular promo sync
						SetItemInLocalData('DataVersion','install');
				}

				//The device values are helpful when negative feedback comes in. Use the device id (UDID) from the Feedback table to lookup the 
				//device details (model, version, OS). These values are used in the Sync Start web method (js/app.js)
				if(GetItemFromLocalData('PushToken') === null){
					//The push token is determined by Urban Airship (see js/app.js)
					SetItemInLocalData('PushToken', '');
				}

				if(globals.debugApp){
					console.log("SYNC-DEBUG: .. moving to resolve()");
				}

				globals.initialSyncDone = true;
				resolve();

			}, 100);
		
		} //END

		return deferred.promise;
	}
})();
//End of Script


