//      SUMMARY:      
//the loading factory is the main factory for the application, the sync process gets called here ...
//many controllers rely on the Loading factory promise to be resolved which indicates that the 
//sync process is complete and that data is available for controllers to access (or the process 
//eited due to an ajax error or initialSyncDoneTimeout caused the factory promise to be resolved
//and the loading page to hide but the sync is still running in the background, if this is case 
//the user might see slightly outdated data)



(function() {
	angular.module('connect.sync')

	.factory('Sync', syncService);

	syncService.$inject = ['$http', '$rootScope', '$timeout', '$compile', 'ClientSet'];


	function syncService($http, $rootScope, $timeout, $compile, ClientSet) {

		if(globals.debugApp){
			console.log("SYNCSERVICE-DEBUG: SYNC SERVICE LOADED");
		}

		//Create array to hold sync completes states
		var downloadStage = { "serverOK":false, "gameProgress":false, "clientSets":false, "promoHeaders":false, "promos":false, "newPromos":false, "promoImages":false, 
		"storeList":false, "storeServices":false, "services":false, "syncEnd":false, 'promoPage':false, 'promoCarousel':false, 'storePage':false, 'finished':false };


		function startSync($scope, Sync){

			setTimeout(function(){
				var op = LoadPromoImagesFromAppData() ? LoadPromoImagesFromAppData() : null; 

				if(globals.debugApp){
					console.log('TO DETERMINE SPINNER: ' + JSON.stringify( op ) + 'op length = ' + op.length );
				}

				if(op != null && JSON.stringify( op ) != '{}' && window.device.platform != 'Android'){
					stopPromoCarouselLoader(); 
				}
				else{
					startPromoCarouselLoader(); 
					if(globals.debugApp){
						console.log('STARTING SPINNER INITIATED..');
					}
				}

				if(globals.debugApp){
					console.log("SYNCSERVICE-DEBUG: Start Sync() Fired");
				}
				serverOK($scope, Sync);
			}, 1);
		}


		function serverOK($scope, Sync){

			if( globals.syncIsDone == false){

				$http.post(GetItemFromLocalData('ClientURL'), soapMessageServerOK(), {timeout: 20000} ).then(function(response) { 
					if(globals.debugApp){
						console.log('SYNCSERVICE-DEBUG: Server connection was successful in serverOK()');
					}

					if (parseServerOK(response.data)) { 
						var GameProgress = GetItemFromLocalData('GameProgress'); 

						if (GameProgress != null) { //check if the user has any saved game progress that was not synced to the server
							GameProgress = JSON.parse(GameProgress);
							if(globals.debugApp){
								console.log( 'calling end game on unsaved game progress with ' + GameProgress.pointsWon + ' points won and ' + GameProgress.playsLeft + ' plays left and ' + GameProgress.wonPromo + ' promo');
							}
							//saveClientLog(71, 0, 0, '', 0, 'calling end game on unsaved game progress with ' + GameProgress.pointsWon + ' points won and ' + GameProgress.playsLeft + ' plays left and ' + GameProgress.wonPromo + ' promo');
						
							gameProgress(GameProgress, $scope, Sync);
							downloadStage.serverOK = true;
						}
						else{
							//mark GameProgress as true and go to sync type method
							syncType($scope, Sync);
							downloadStage.serverOK = true;
						}
					}
					else{
						if(globals.debugApp){
							console.log('SYNCSERVICE-ERROR: Server connection was FAILED in serverOK() !!!');
						}
						serverOK($scope, Sync); //retry
					}

				}, function(error) {
					globals.timeouts = globals.timeouts + 1;

					if(globals.timeouts > 30){
						if(globals.debugApp){
							console.log('SYNCSERVICE-ERROR: TOO MANY TIMEOUTS. GOING TO OFFLINE MODE. !!!');
						}

						CancelSync();
					}
					else{
						serverOK($scope, Sync);	//retry
						if(globals.debugApp){
							console.log('SYNCSERVICE-ERROR: Server connection was FAILED in serverOK() !!!  TIMEOUTS = ' + globals.timeouts);
						}
					}
				});

			}
		}

		//NOTE- his is Used for too many TimeOuts
		function CancelSync(){
			//set all progress states to true
			downloadStage.clientSet = true;
			downloadStage.serverOK = true;
			downloadStage.gameProgress = true;
			downloadStage.promoHeaders = true;
			downloadStage.promos = true;
			downloadStage.promoImages = true;
			downloadStage.storeList = true;
			downloadStage.storeServices = true;
			downloadStage.services = true;
			downloadStage.syncEnd = true;
			downloadStage.promoCarousel = true;
			downloadStage.storePage = true;
			downloadStage.syncEnd = true;
			downloadStage.promoPage = true;
			downloadStage.finished = true;

			alert( "Offline mode: Could not connect to server after 30 attempts. \nPlease check your data and then go to 'More' and tap on 'Sync Now' to restart the sync. \n" +
				" TIP: Make sure Airplane mode is 'Off', and turn Wifi 'On'", function(){} );
		}



		// GAMEPROGRESS METHOD //
		function gameProgress(GameProgress, $scope, Sync){

			if( globals.syncIsDone == false){

				$http.post(GetItemFromLocalData('ClientURL'), soapMessageGameEnd(GameProgress.pointsWon, GameProgress.playsLeft, GameProgress.wonPromo), {timeout: 22000} ).then(function(response) {

					if (parseServerOK(response.data)) { 

						if(globals.debugApp){
							console.log("SUCCESS: Game Progress was updated!");
						}

						var processed = parseInt($(response.data).find('GameEndResult').children("Processed").text()); //returned by the web method, determines if the server has properly processed the gameend request
						
						SetItemInLocalData('Points', parseInt($(response.data).find('GameStartResult').children("Points").text()) );
						SetItemInLocalData('GamePlays', parseInt($(response.data).find('GameStartResult').children("Plays").text()) );
						SetItemInLocalData('PromoPlay', parseInt($(response.data).find('GameStartResult').children("PromoPlay").text()) );


						//save points, plays , and promoplay from the GameEnd web method response
						var error = $(response.data).find('Standard').children('MethodErrorMsg').text();
						if (processed == 1) {
							//remove saved progress since it has been synced with the server and the app points, plays, and promoplay have been updated
							SetItemInLocalData('GameProgress', '');
						}

						syncType($scope, Sync);
						downloadStage.gameProgress = true;
					}
					else{
						if(globals.debugApp){
							console.log('SYNCSERVICE-ERROR: data was bad in serverOK() !!!');
						}
						gameProgress(GameProgress, $scope, Sync);
					}

				}, function(error) {

					if(globals.debugApp){
						console.log('SYNCSERVICE-ERROR: Server connection was FAILED in serverOK() !!!');
					}
					gameProgress(GameProgress, $scope, Sync);
				}); 

			}
		}

		var fullDownload = false;
		function syncType( $scope, Sync){

			if( globals.syncIsDone == false){
				$http.post(GetItemFromLocalData('ClientURL'), soapMessageSyncStart(), {timeout: 22000} ).then(function(response) { 
					if(globals.debugApp){
						console.log("Now Downloading ClientSets...");
						//console.log('DEBUG-SYNC: data - ' + response.data);
					}
					saveClientLog(15, 0, 0, '', 0, 'Now Downloading ClientSets.');
					fullDownload = true;
					
					downloadClientSet($rootScope, $timeout, $scope, Sync);

					if ($rootScope.RegisteredUser){ //if this is a registered user save the account details returned by the server's syncstart method
						saveAccountDetailJson(response.data);
						$rootScope.RegisteredUser = true;
					}
				}, function(error) {
					syncType( $scope, Sync);
					if(globals.debugApp){
						console.log('SYNCSERVICE-ERROR: Server connection was FAILED in syncType() !!!');
					}
				});
			}
		}



		function syncEnd($scope, Sync){

			if( globals.syncIsDone == false){

				$.ajax({ //ajax call for syncend web method
					url: GetItemFromLocalData('ClientURL'),
					type: "POST",
					dataType: "xml",
					timeout : 21000,
					data: soapMessageSyncEnd(),
					success: function(data){
						saveClientLog(12, 0, 0, '', 0, 'syncend ok');
						if(globals.debugApp){
							console.log("SYNCEND-SUCCESS: syncend() launched");
						}

						saveStoreData(); 
						syncProg = setInterval(syncProgress( Sync, $scope, $rootScope, $timeout, $compile, ClientSet), 1);
						downloadStage.syncEnd = true;
							
						SetItemInLocalData('DataVersion', TempDataVersion); //filesystem save

						if(globals.debugApp){
							console.log('Set DataVersion to: ' + TempDataVersion);
						}
						//if syncend is successful then set the client's dataversion to tempdataversion (which was set to the server's data 
						//version earlier for a downloadsync or is just the same dataversion for a non-download sync)
					},
					error: function(error){
						saveClientLog(13, 0, 0, '', 0, 'syncend error');
						if(globals.debugApp){
							console.log("SYNCEND-ERROR: syncend() did not connect");
						}
						syncEnd($scope, Sync)
					},

					contentType: "text/xml; charset=utf-8"
				});
			}
		}





		// FUNCTION: DOWNLOAD CLIENT SET //
		//AJAX function for downloading client settings, used only in a download sync, called in promosdone function below
		function downloadClientSet($rootScope, $timeout, $scope, Sync) {
			//console.log("Client Set download Reuested");
			var error = false;

			if( globals.syncIsDone == false){

				$.ajax({
					url: GetItemFromLocalData('ClientURL'), //the ClientURL we stored earlier
					type: "POST", //Alway use POST, the web service is SOAP based not RESTful
					dataType: "xml", //Always xml, the webservice does not use JSON
					timeout : 24000, //The ajaxTimeout object is defined in js/app.js
					data: soapMessageClientSetAll(), //soap messages are defined in js/soapMessages.js
					success: function(data){ //ajax function successs callback (i.e. the function did not time out for lack of internet or something 
						//else happened that caused an error in the actual web service call like the server is down, ..etc)
						error = false;

						if(globals.debugApp){
							console.log("SUCCESS: downloadClientSet() was downloaded.");
						}

						saveClientLog(26, 0, 0, 'SUCCESS: downloadClientSet() was downloaded.');
						var cs = {};

						$(data).find('RClientSetServer').each(function() //for each RclientSetServer object in the XML response (for more details on response format for various web methods, see the webservice specification: http://74.208.209.112/Market_GreenSquare_001/WebService/WebService1.asmx)
						{
							var ItemName = $(this).children("ItemName").text(); //The name of the setting
							var ItemValue = $(this).children("ItemValue").text(); //The value of the setting
							cs[ItemName] = ItemValue; //Save the setting as a property of the cs object defined above; the ItemName is the key and the ItemValue is the value
						});

						//save to localStorage, localStorage is basically the local database for this application; 
						//JSON.stringify converts arrays or objects into json string literals, localStorage can only save strings
						downloadStage.clientSet = true;
						SaveClientSetsToAppData( JSON.stringify(cs) );
						downloadPromoHead( $rootScope, $timeout, $scope, Sync);
					},
					error: function(data){
						if(globals.debugApp){
							console.log("ERROR: downloadClientSet() was not downloaded.");
						}

						saveClientLog(54, 0, 0, '', 0, 'ERROR: downloadClientSet() - Could not connect.');
						downloadClientSet($rootScope, $timeout, $scope, Sync);
					},
					contentType: "text/xml; charset=utf-8"
				}).done(function(){
					if(error){
						if(globals.debugApp){
							console.log("ClientSet download HAD AN ERROR");
						}
					}
				});
			}
		} //END


		//FUNCTION: DOWNLOAD STORE LIST //
		//AJAX function for downloading stores, used only in a download sync, called in promosdone function below
		function downloadStoreList($scope, Sync){ 

			if( globals.syncIsDone == false){

				$.ajax({
					url: GetItemFromLocalData('ClientURL'),
					type: "POST",
					dataType: "xml",
					timeout : 23000,
					data: soapMessageStoreListAllByDescript(),
					success: function(data){

						var stuff = []; //local array to hold the stores
						if(globals.debugApp){
							console.log("SUCCESS: Store List was downloaded");
						}
						saveClientLog(19, 0, 0, 'SUCCESS: Store List was downloaded');

						//console.log('Stores Data: '+xmlToString(data)+"--"+window.localStorage.getItem('ClientURL')+"--"+soapMessageStoreListAllByDescript());
						$(data).find('RStoreServer').each(function(index)
						{
							var StoreID = parseInt($(this).children("StoreID").text());
							var Descript = $(this).children("Descript").text();
							//parseInt is used when saving the value if the expected value is an int

							var Address1 = $(this).children("Address1").text();
							var Address2 = $(this).children("Address2").text();
							var City = $(this).children("City").text();
							var State = $(this).children("State").text();
							var Zip = $(this).children("Zip").text();
							var Phone = $(this).children("Phone").text();
							var Latitude = parseFloat($(this).children("Latitude").text()); //parseFloat is used when saving the value if the expected value is a real number (i.e. not an int but has decimals)
							var Longitude = parseFloat($(this).children("Longitude").text());
							var RegularPrice = Math.round(parseFloat($(this).children("RegularPrice").text())*100)/100; //Math.round limits the decimal number to two places
							var PlusPrice = Math.round(parseFloat($(this).children("PlusPrice").text())*100)/100;
							var PremiumPrice = Math.round(parseFloat($(this).children("PremiumPrice").text())*100)/100;
							var DieselPrice = Math.round(parseFloat($(this).children("DieselPrice").text())*100)/100;
							var KerosenePrice = Math.round(parseFloat($(this).children("KerosenePrice").text())*100)/100;
							var GasLastUpdate = $(this).children("GasLastUpdate").text();

							stuff.push({
								StoreID : StoreID,
								Descript: Descript,
								Address1 : Address1,
								Address2 : Address2,
								City : City,
								State : State,
								Zip : Zip,
								Phone : Phone,
								Latitude : Latitude,
								Longitude : Longitude,
								RegularPrice : RegularPrice,
								PlusPrice : PlusPrice,
								PremiumPrice : PremiumPrice,
								DieselPrice : DieselPrice,
								KerosenePrice : KerosenePrice,
								GasLastUpdate : GasLastUpdate
							});
						});

						//GENERATE timeStamp and store it in rootscope, this is displayed with the redemption
						var currentdate = new Date(); 
						var storesSyncDate = "StoreList last synced: " + getMonth(currentdate) + " " + addOrd(currentdate.getDate()) + " " + " @ " + getTime(currentdate);
					    window.localStorage.setItem('StoresSyncDate', storesSyncDate);

					    //save the stringified stores in the global TempStores variable, TempStores will replace local Storage stores in the sync end function
						globals.TempStores = JSON.stringify(stuff); 
						downloadStage.storeList = true;
						//we do not save localStorage automatically here so that Stores, Services, and StoreServices are all saved at the 
						//same time and are on the same DataVersion.. call syncEnd to indicate that one of the asynchrous functions has finished
						downloadServices($scope, Sync);
					},
					error: function(data){
						if(globals.debugApp){
							console.log("ERROR: Store List was not downloaded");
						}
						saveClientLog(53, 0, 0, '', 0, 'ERROR: downloadStoreList() - Could not connect.');
						downloadStoreList($scope, Sync); //RETRY
					},
					contentType: "text/xml; charset=utf-8"
				});

			}
		}//END


		//FUNCTION DOWNLOAD SERVICES
		//AJAX function for downloading services, used only in a download sync, called in promosdone function below
		function downloadServices($scope, Sync){ //removed deffered

			if( globals.syncIsDone == false){

				$.ajax({
					url: GetItemFromLocalData('ClientURL'),
					type: "POST",
					dataType: "xml",
					timeout : 23000,
					data: soapMessageServiceListAll(),
					success: function(data){
						if(globals.debugApp){
							console.log('SUCCESS: Services downloaded');
						}

						saveClientLog(19, 0, 0, 'SUCCESS: Services downloaded' );
						var stuff = [];

						$(data).find('RServiceServer').each(function(index)
						{
							var ServiceID = $(this).children("ServiceID").text();
							var Descript1 = $(this).children("Descript1").text();
							var SortOrder = parseInt($(this).children("SortOrder").text());
							stuff.push({
								ServiceID : ServiceID,
								Descript1 : Descript1,
								SortOrder : SortOrder
							});
						});

						globals.TempServices = JSON.stringify(stuff);

						downloadStage.services = true;

						downloadStoreServices($scope, Sync);
					},
					error: function(data){
						if(globals.debugApp){
							console.log('ERROR: Services was not downloaded');
						}
						saveClientLog(52, 0, 0, '', 0, 'ERROR: downloadServices() - Could not connect.');
						downloadServices($scope, Sync); //RETRY
					},
					contentType: "text/xml; charset=utf-8"
				});
			}
		}//END


		//FUNCTION: DOWNLOAD STORE SERVICES //
		//AJAX function for downloading storeservices, used only in a download sync, called in promosdone function below
		function downloadStoreServices($scope, Sync) { //removed deffered

			if( globals.syncIsDone == false){

				$.ajax({
					url: GetItemFromLocalData('ClientURL'),
					type: "POST",
					dataType: "xml",
					timeout : 23000,
					data: soapMessageStoreServiceListAll(),
					success: function(data){
						if(globals.debugApp){
							console.log('SUCCESS: Store services downloaded');
						}

						saveClientLog(19, 0, 0, 'SUCCESS: Store services downloaded' );
						var stuff = [];
						$(data).find('RStoreServiceServer').each(function(index)
						{
							var ServiceID = $(this).children("ServiceID").text();
							var StoreID = parseInt($(this).children("StoreID").text());
							stuff.push({
								ServiceID : ServiceID,
								StoreID : StoreID
							});
						});

						globals.TempStoreServices = JSON.stringify(stuff);
						downloadStage.storeServices = true;
						syncEnd($scope, Sync);
					},
					error: function(data){

						if(globals.debugApp){
							console.log('ERROR: Store services was not downloaded');
						}

						saveClientLog(51, 0, 0, '', 0, 'ERROR: downloadStoreServices() - Could not connect.');
						downloadStoreServices($scope, Sync);
				},
					contentType: "text/xml; charset=utf-8"
				});
			}
		}



		//FUNCTION: DOWNLOAD PROMO HEAD //
		//AJAX function for downloading promo headers, used only in a download sync, called in js/app.js
		function downloadPromoHead( $rootScope, $timeout, $scope, Sync) { //removed deffered

			if(globals.debugApp){
				console.log("downloadPromoHead() initiated");
			}
			var error = false;

			if( globals.syncIsDone == false){
				$.ajax({
					url: GetItemFromLocalData('ClientURL'),
					type: "POST",
					dataType: "xml",
					timeout : 24000,
					data: soapMessagePromoHeadListAll(),
					success: function(data){
						if(globals.debugApp){
							console.log("SUCCESS: Promo Head Downloaded via downloadPromoHead()");
						}

						saveClientLog(18, 0, 0, 'SUCCESS: Promo Head Downloaded via downloadPromoHead()' );
						error = false;
						var stuff = [];

						$(data).find('RPromoHeadServer').each(function(index){
							var HeadID = $(this).children("HeadID").text();
							var Descript1 = $(this).children("Descript1").text();
							var SortOrder = parseInt($(this).children("SortOrder").text());
							//console.log("promo header received: " + HeadID + ":" + Descript1 + ":" + SortOrder);
							
							stuff.push({
								HeadID : HeadID,
								Descript1 : Descript1,
								SortOrder : SortOrder
							});
						});

						SaveHeadersToAppData( JSON.stringify(stuff) );
						downloadStage.promoHeaders = true;
						downloadPromo($rootScope, $timeout, $scope, Sync); //if no ajax error was thrown, go onto downloadpromo
					},
					error: function(data){
						if(globals.debugApp){
							console.log("ERROR: Promo Head was not Downloaded");
						}

						saveClientLog(17, 0, 0, 'ERROR: Promo Head was not Downloaded: ' + data);
						error = true;
						downloadPromoHead( $rootScope, $timeout, $scope, Sync); //RETRY
					},
					contentType: "text/xml; charset=utf-8"
				}).done(function(){
					if (error){
						if(globals.debugApp){
							console.log("downloadPromoHead() Had an error!");
						}
					}
				});
			}
		}//END


		//TODO: Check for unused promo or duplicate promo images and get rid of them

		// FUNCTION: DOWNLOAD PROMO //
		//AJAX function for download promos, called on every sync either from downloadPromoHead (in which case downloadSync is set to true) or from js/app.js in which case it is set to false
		function downloadPromo($rootScope, $timeout, $scope, Sync){ 
			var error;

			if(globals.debugApp){
				console.log("downloadPromo() initiated");
			}

			if( globals.syncIsDone == false){
				$.ajax({
					url: GetItemFromLocalData('ClientURL'),
					type: "POST",
					dataType: "xml",
					timeout : 30000,
					data: soapMessagePromoList2(),
					success: function(data){
						if(globals.debugApp){
							console.log("SUCCESS: Promos downloaded");
						}
						saveClientLog(18, 0, 0, 'SUCCESS: Promo Info downloaded' );
						error = false;

						var savepromos = [];	//the promos that will be saved to localstorage
						var newpromos = [];		//promos that are new
						var savepromostores = []; //promostores that will be saved to localstorage

						//gets the promoimage object from localstorage or creates a new one if not available, JSON.parse converts a 
						//string into a javascript array or object (it is the opposite of JSON.stringify)
						var PromoImages = LoadPromoImagesFromAppData() ? LoadPromoImagesFromAppData() : {}; 
						var promoImagesDone = 0; //indicates how many promo images have been downloaded from the server

						$(data).find('RPromoServer2').each(function(index){
							var PromoID = $(this).children("PromoID").text();
							var preDescText=$(this).children("Descript1").text();
							var Descript1 =preDescText;
							var Descript2 = $(this).children("Descript2").text();
							var PromoType = parseInt($(this).children("PromoType").text());
							var HeadID = $(this).children("HeadID").text();
							var InterestID = $(this).children("InterestID").text();
							var PromoStore = parseInt($(this).children("PromoStore").text());
							var PromoUse = parseInt($(this).children("PromoUse").text());
							var PromoAccount = parseInt($(this).children("PromoAccount").text());
							var DateStart = ticksToEpoch(parseInt($(this).children("DateStartTicks").text()));
							var DateEnd = ticksToEpoch(parseInt($(this).children("DateEndTicks").text()));
							var DisplaySec = parseInt($(this).children("DisplaySec").text());
							var Uses = parseInt($(this).children("Uses").text());
							var AccountUses = parseInt($(this).children("AccountUses").text());
							var Age = parseInt($(this).children("Age").text());
							var Points = parseInt($(this).children("Points").text());
							var DetailHTMLFile = $(this).children("DetailHTMLFile").text();
							var PhotoFile = $(this).children("PhotoFile").text();
							var ThumbFile = $(this).children("ThumbFile").text();
							var DescriptPush = $(this).children("DescriptPush").text();
							var FocusTime = $(this).children("FocusTime").text();

							if(InterestID){
								if(globals.debugApp){
									console.log("SYNC-PROMOS: WARNING - InterestID " + InterestID + "  (Warning! this promo is using an Interest only ID!");
								}
							}

							if (FocusTime){
								FocusTime = parseInt(FocusTime); //focus time indicates if this promotion can be used in the focus now module: 0 indicates no, 1 is breakfast, 2 is lunch, 3 is dinner, and 4 is late night
							}
							else{
								FocusTime = 0;
							}

							var now = moment().tz('America/New_York').toDate().getTime() - globals.timeZoneOffset;

							if ((Uses == 0 || AccountUses < Uses) && now > DateStart && now < DateEnd){ 
								//make sure the promotion is valid (i.e. today's date is between the promo start and end date, also make sure the promo 
								//is not a redeemable promo (Uses == 0) OR that the user has enough AccountUses to redeem the promotion)	
								if(globals.debugApp){
									console.log("promo saved: " + PromoID);
								}

								savepromos.push({
									PromoID : PromoID,
									Descript1 : Descript1,
									Descript2 : Descript2,
									PromoType : PromoType,
									HeadID : HeadID,
									InterestID : InterestID,
									PromoStore : PromoStore,
									PromoUse : PromoUse,
									PromoAccount : PromoAccount,
									DateStart : DateStart,
									DateEnd : DateEnd,
									DisplaySec : DisplaySec,
									Uses : Uses,
									AccountUses : AccountUses,
									Age : Age,
									Points : Points,
									DetailHTMLFile : DetailHTMLFile,
									PhotoFile : PhotoFile,
									ThumbFile : ThumbFile,
									DescriptPush : DescriptPush,
									FocusTime : FocusTime
								});
							}
							else{
								if(globals.debugApp){
									console.log("SYNC-PROMOS: Invalid promo received: " + PromoID);
								}
							}
						});

						//Simple implemtation to clear images when a small excess of Promo images is incumulated 
						//This can be replaced with a sort alg later
						if( ( savepromos.length * 2.5 ) < PromoImages.length ){
							PromoImages = null;
							if(globals.debugApp){
								console.log('SYNC-DEBUG: PromoImages were wiped.');
							}
						}
								

						// **** Grab Promos and put them into a temp variable -- globals.PromoImagesArr **** //
					    var op =  LoadPromosFromAppData() ? LoadPromosFromAppData() : null; 

					    if(!PromoImages){ //Check to see if PromoImages are empty, if so then Old promos array has to be null
					    	op = null;
					    	PromoImages = {};
					    	if(globals.debugApp){
					    		console.log("promo images arr was empty. grabbing all images");
					    	}
					    }


					    if( op != null ){ // Check to see if old promos exist
					    	var oldpromos = op;
					    	globals.emptyCarousel = false;
					    	if(globals.debugApp){
					    		console.log("There were Old promos grab");
					    	}
					    }
					    else{
					    	var oldpromos = null;
					    	globals.emptyCarousel = true;
					    	if(globals.debugApp){
					    		console.log("No Old promos grab");
					    	}
					    }

						//determine which promos are new (i.e. they have not been saved to the app before)
						for (var i = 0; i < savepromos.length; i++){
							if (!inPromos(savepromos[i].PromoID, oldpromos)){
								newpromos.push(savepromos[i]);
							}
						} 

						if (newpromos.length > 0){
							$(data).find('RPromoStoreServer2').each(function(index){
								var PromoID = $(this).children("PromoID").text();
								var StoreID = parseInt($(this).children('StoreID').text());
								savepromostores.push({
									PromoID : PromoID,
									StoreID : StoreID
								});
							});
						}

						if( oldpromos != null ){ // Check to see if old promos exist
					    	//A block of code to detect a change of promo image name, if there is one
					    	for (var i = 0; i < savepromos.length; i++){
					    		//Check if saved and old promos have the same PromoID
								for (var j = 0; j < oldpromos.length; j++){
									if (oldpromos[j].PromoID == savepromos[i].PromoID){
									
										//CHeck if promos do not contain the same image name, if not then
										if(savepromos[i].PhotoFile != oldpromos[j].PhotoFile){
											if(globals.debugApp){
												console.log("Sync-Debug: !! A new photofile was found for an Old Promo. Let's update it..");
											}

											newpromos.push(savepromos[i]);//push this promo into new promos so the image will be updated
										}
									}
								}
							} 
					    }


						if (newpromos.length > 0){
							if(globals.debugApp){
								console.log('SYNC-PROMOS: There were NEW promos made..');
							}

							downloadStage.newPromos = true;

							//download and save all promostores if there are any new promotions, we do not only download promostores for newpromos 
							//because the logic to check whether or not this promostore is for a new or old promo would likely be slower than just 
							//downloading and saving all of them
							//Clear Promos
							SavePromoStoresToAppData(JSON.stringify(savepromostores));
							
							promoImagesDone = 0;
							var i = 0;

							downloadPromoImage(newpromos, i, function(){

								if(promoImagesDone >= newpromos.length){ //success call back for the downloadpromoimage function, if all the new promo images have been downloaded then save them and call promosDone function (defined below)
									
									//refreshAjax($scope);   // 

									if(globals.debugApp){
										console.log("SYNC-PROMOS: SUCCESS in downloading all new promo imgs.");
									}
									saveClientLog(89, 0, 0, 'SUCCESS in downloading all new promo imgs.' );

									//GENERATE timeStamp and store it in rootscope, this is displayed with the redemption
									var currentdate = new Date(); 
									var promosSyncDate = "Promos last synced: " + getMonth(currentdate) + " " + addOrd(currentdate.getDate()) + " " + " @ " + getTime(currentdate);
									window.localStorage.setItem('PromosSyncDate', promosSyncDate);

									if( globals.syncIsDone == false){

										SavePromoImagesToAppData( JSON.stringify(PromoImages) );
										SavePromosToAppData(JSON.stringify(savepromos)); 
										downloadStage.promoImages = true;
										downloadStage.promos = true;

										promosDone($scope, Sync);
									}
								}
							}, function(){  //error call back for the download promo image function
								error = true;
								if(globals.debugApp){
									console.log("ERROR in downloading promo imgs.");
								}
							});
						}
						else{

							//refreshAjax($scope);

							if( globals.syncIsDone == false){ //check to see if sync was cancled because of logging in or out

								//Important: Always save New data over old so new descriptions and meta data are udated each time.
								if(globals.debugApp){
									console.log('SYNC-PROMOS: No new images downloaded, but lets save all new promo data and refresh pages anyways.. SAVING.');
								}

								SavePromosToAppData(JSON.stringify(savepromos)); 
								SavePromoImagesToAppData( JSON.stringify(PromoImages) );
								downloadStage.newPromos = false; // Flag new promos, so the deleted ones get refreshed off the pages

								saveClientLog(32, 0, 0, 'SUCCESS: But no new promos' );
								if(globals.debugApp){
									console.log('SYNC-PROMOS: There were No new promos made..');
								}

								downloadStage.promos = true;
								downloadStage.promoImages = true;

								//GENERATE timeStamp and store it in rootscope, this is displayed with the redemption
								var currentdate = new Date(); 
								var promosSyncDate = "Promos last synced: " + getMonth(currentdate) + " " + addOrd(currentdate.getDate()) + " " + " @ " + getTime(currentdate);
								window.localStorage.setItem('PromosSyncDate', promosSyncDate);

								promosDone($scope, Sync); //if there are no new promos then just call promosdone right away
							}
						}

						function promosDone($scope, Sync){
							if(globals.debugApp){
								console.log("SYNC-PROMOSDONE: initiated");
							}
							saveClientLog(32, 0, 0, 'SUCCESS: in PromosDone. Saving Promos now' );

							//clear data and data
							//make sure no AJAX functions threw an error before saving the data (so we do not for example save a promo when the corresponding promo image was not downloaded) we also don't want to call
							//remaining AJAX download sync functions since the user likely has bad or no internet at this time
							if(!error && globals.syncIsDone == false){

								if(globals.debugApp){
									console.log("SYNC-DEBUG: No errors found, proceeding with Storelist DL");
								}

								if(fullDownload){
									downloadStoreList($scope, Sync); // this will chain and also get services and store services
								}
								else{
									if(globals.debugApp){
										console.log('Sync-Debug: Skipping storelist and related downloads as this is not a FullDownload.');
									}

									downloadStage.storeServices = true;
									downloadStage.services = true;
									downloadStage.storeList = true;

									syncEnd($scope, Sync);
								}
							}
							else{
								if(globals.debugApp){
									console.log("ERROR: error in promosDone(): " + error);
								}
								downloadPromo($rootScope, $timeout, $scope, Sync); //RETRY PROMO METHOD FROM BEGINNING
							}
						}//END OF PROMOSDONE

						function downloadPromoImage(photo, counter, callback, errorcallback){ //helper function for downloading a promo image

							if(counter < newpromos.length){
								if(globals.debugApp){
									console.log('Debug: image download test [' + counter + '] - ' + photo[counter].PhotoFile ); 
								}

								if( globals.syncIsDone == false){

									$.ajax({
										url: GetItemFromLocalData('ClientURL'),
										type: "POST",
										dataType: "xml",
										timeout : 42000,
										data: soapMessageGetFileChunk('Image', photo[counter].PhotoFile), //this as well as the function updateGameAssets (in js/app.js) uses the GetFileChunk web method which returns the image as a base64 encoded string
										success: function(data){
											if(globals.debugApp){
												console.log("SUCCESS: A promo-image was downloaded via downloadpromoimage()");
											}
											saveClientLog(44, 0, 0, "SUCCESS: A promo-image was downloaded via downloadpromoimage()");

											var error = $(data).find('Standard').children('MethodErrorLevel').text();
											if (error == "None"){
												if (photo[counter].PhotoFile.substring(photo[counter].PhotoFile.indexOf('.') + 1) == 'png') //check if the photofile name is png or jpg and save accordingly
													PromoImages[photo[counter].PhotoFile] = "data:image/png;base64," + $(data).find('GetFileChunkResult').children('FilePart').text();
												else{
													PromoImages[photo[counter].PhotoFile] = "data:image/jpeg;base64," + $(data).find('GetFileChunkResult').children('FilePart').text();
												}
												promoImagesDone++; //increments promoImagesDone which is used in the success call back to determine if all promo images have been downloaded
												
												if(globals.debugApp){
													console.log("Image Download successful (line1059/sync.js) : " + photo[counter].PhotoFile +  "  :  "+ promoImagesDone);
												}
											}
											else{
												if(globals.debugApp){
													console.log("ERROR: could not download promo image: "  + photo[counter].PhotoFile + " - Error = " + error);
												}
												saveClientLog(45, 0, 0, "ERROR: promo image downloaded but caused an error: "  + photo[counter].PhotoFile + " - Error = " + error);
											}

											var next = counter + 1;
											callback();

											downloadPromoImage(photo, next, callback, errorcallback);
										},
										contentType: "text/xml; charset=utf-8",
										error: function(data){
											//var next = counter + 1;
											if(globals.debugApp){
												console.log("error! in downloadPromoImage()");
												console.log("ERROR: could not connect to download promo image: "  + photo[counter].Photofile + " - Error = " + data);
											}
											saveClientLog(46, 0, 0, "ERROR: could not connect to download promo image: "  + photo[counter].Photofile + " - Error = " + data);
											errorcallback();

											downloadPromoImage(photo, counter, callback, errorcallback);
										}
									});
								}
							}
						}

						function inPromos(promoid, promos){ //helper function to check if a promoid is in a promos array
							if (!promos){
								return false;
							}

							for (var i = 0; i < promos.length; i++){
								if (promos[i].PromoID == promoid){
									return true;
								}
							}
							return false;
						}
					},
					error: function(data){
						saveClientLog(62, 0, 0, 'ERROR: in downloadPromo() - Promo information was not downloaded.' );
						if(globals.debugApp){
							console.log('ERROR: in downloadPromo() - Promo information was not downloaded.' );
						}

						error = true;
						downloadPromo($rootScope, $timeout);
					},
					contentType: "text/xml; charset=utf-8"
				});
			}
		} //END



		return {
		    start: function($scope, Sync) {
		    	setTimeout(function(){
			    	if(globals.debugApp){
			      	    console.log("Start FUNCTION FIRED!");
			      	}
			      	startSync($scope, Sync); // Start App's Sync Background Process
		      	},1);
		    },

		    //In main page of app, get
		    getGameProgressState: function(){
		    	if(globals.debugApp){
		    		console.log("getGameProgressState FUNCTION FIRED! - Returning bool value of downloadStage.gameProgress : " + downloadStage.gameProgress);
		    	}
		    	return downloadStage.gameProgress;
		    },
		    getPromoHeadersState: function(){
		    	if(globals.debugApp){
		    		console.log("getPromoHeadersState FUNCTION FIRED! - Returning bool value of downloadStage.promoHeaders : " + downloadStage.promoHeaders);
		    	}
		    	return downloadStage.promoHeaders;
		    },
		    getClientSets: function(){
		    	if(globals.debugApp){
		    		console.log("getClientSets FUNCTION FIRED! - Returning bool value of downloadStage.clientSets : " + downloadStage.clientSets);
		    	}
		    	return downloadStage.clientSets;
		    },
		    getPromosState: function(){
		    	if(globals.debugApp){
		    		console.log("getPromosState FUNCTION FIRED! - Returning bool value of downloadStage.promos : " + downloadStage.promos);
		    	}
		    	return downloadStage.promos;
		    },
		    getPromosImagesState: function(){
		    	if(globals.debugApp){
		    		console.log("getPromoImagesState FUNCTION FIRED! - Returning bool value of downloadStage.promoImages : " + downloadStage.promoImages);
		    	}
		    	return downloadStage.promoImages;
		    },
		    getStoreListState: function(){
		    	if(globals.debugApp){
		    		console.log("getStoreListState FUNCTION FIRED! - Returning bool value of downloadStage.storeList : " + downloadStage.storeList);
		    	}
		    	return downloadStage.storeList;
		    },
		    getStoreServicesState: function(){
		    	if(globals.debugApp){
		    		console.log("getStoreServicesState FUNCTION FIRED! - Returning bool value of downloadStage.storeServices : " + downloadStage.storeServices);
		    	}
		    	return downloadStage.storeServices;
		    },
		    getServicesState: function(){
		    	if(globals.debugApp){
		    		console.log("getServicesState FUNCTION FIRED! - Returning bool value of downloadStage.services : " + downloadStage.services);
		    	}
		    	return downloadStage.services;
		    },
		    getSyncEndState: function(){
		    	if(globals.debugApp){
		    		console.log("getSyncEndState FUNCTION FIRED! - Returning bool value of downloadStage.syncEnd : " + downloadStage.syncEnd);
		    	}
		    	return downloadStage.syncEnd;
		    },
		    getPromoPageState: function(){
		    	if(globals.debugApp){
		    		console.log("getPromoPageState FUNCTION FIRED! - Returning bool value of downloadStage.promoPage : " + downloadStage.promoPage);
		    	}
		    	return downloadStage.promoPage;
		    },
		    setPromoPageState: function(value){
		    	if(globals.debugApp){
		    		console.log("setPromoPageState FUNCTION FIRED! - Returning bool value of downloadStage.promoPage : " + downloadStage.promoPage);
		    	}
		    	downloadStage.promoPage = value;
		    },
		    getStorePageState: function(){
		    	if(globals.debugApp){
		    		console.log("getStorePageState FUNCTION FIRED! - Returning bool value of downloadStage.storePage : " + downloadStage.storePage);
		    	}
		    	return downloadStage.storePage;
		    },
		    setStorePageState: function(value){
		    	if(globals.debugApp){
		    		console.log("setStorePageState FUNCTION FIRED! - Returning bool value of downloadStage.storePage : " + downloadStage.storePage);
		    	}
		    	downloadStage.storePage = value;
		    },
		    getPromoCarouselState: function(){
		    	if(globals.debugApp){
		    		console.log("getPromoCarouselState FUNCTION FIRED! - Returning bool value of downloadStage.promoCarousel : " + downloadStage.promoCarousel);
		    	}
		    	return downloadStage.promoCarousel;
		    },
		    setPromoCarouselState: function(value){
		    	if(globals.debugApp){
		    		console.log("getPromoCarouselState FUNCTION FIRED! - Returning bool value of downloadStage.promoCarousel : " + downloadStage.promoCarousel);
		    	}
		    	downloadStage.promoCarousel = value;
		    },
		    getFinishedState: function() {
		    	if(globals.debugApp){
		       		console.log("Get Finished FUNCTION FIRED!");
		       	}
		      	//Create an alert for testing
		      	return downloadStage.finished;
		    },
		    setFinishedState: function(value) {
		    	if(globals.debugApp){
		       		console.log("Set Finished FUNCTION FIRED!");
		       	}
		      	//Create an alert for testing
		      	downloadStage.finished = value;
		    },
		    getNewPromosState: function() {
		    	if(globals.debugApp){
		       		console.log("Get newPromos FUNCTION FIRED!");
		       	}
		      	//Create an alert for testing
		      	return downloadStage.newPromos;
		    },
		    setNewPromosState: function(value) {
		    	if(globals.debugApp){
		       		console.log("Set newPromos FUNCTION FIRED!");
		       	}
		      	//Create an alert for testing
		      	downloadStage.newPromos = value;
		    }
		};
	}
})();
//End of FACTORY

function stopPromoCarouselLoader(){
	if(globals.spinner){
		globals.spinner.stop();
	}
		$('#pCarouselLoading').hide(); 
		globals.spinnerIsActive = false;

		if(globals.debugApp){
			console.log('Sync-Debug: Spinner was stopped');
		}
}

function startPromoCarouselLoader(){

	var opts = {
		  lines: 14 // The number of lines to draw
		, length: 15 // The length of each line
		, width: 8 // The line thickness
		, radius: 24 // The radius of the inner circle
		, scale: .5 // Scales overall size of the spinner
		, corners: 1 // Corner roundness (0..1)
		, color: '#000' // #rgb or #rrggbb or array of colors
		, opacity: 0.25 // Opacity of the lines
		, rotate: 32 // The rotation offset
		, direction: 1 // 1: clockwise, -1: counterclockwise
		, speed: 1 // Rounds per second
		, trail: 60 // Afterglow percentage
		, fps: 20 // Frames per second when using setTimeout() as a fallback for CSS
		, zIndex: 2e9 // The z-index (defaults to 2000000000)
		, className: 'spinner' // The CSS class to assign to the spinner
		, top: '64%' // Top position relative to parent
		, left: '50%' // Left position relative to parent
		, shadow: false // Whether to render a shadow
		, hwaccel: false // Whether to use hardware acceleration
		, position: 'absolute' // Element positioning
	}
	var target = document.getElementById('carouselLoader');
	globals.spinner = new Spinner(opts).spin(target);

	if(globals.debugApp){
		console.log('Sync-Debug: Spinner started via startPromoCarouselLoader();');
	}

	$('#pCarouselLoading').show(); 
	globals.spinnerIsActive = true;
}


function syncProgress(SyncObj, $scope, $rootScope, $timeout, $compile, ClientSet){
	if(globals.debugApp){
		console.log('syncProgress called');
	}
	intervalId = setInterval(syncRun, 600);

	function syncRun() {

		if(globals.debugApp){
			console.log('Sync-Debug: sync check ran');
		}

		//check to see if store related sync is finished
		if( SyncObj.getStoreServicesState() && SyncObj.getStoreListState() && SyncObj.getServicesState() && !SyncObj.getStorePageState() ){

			//Evoke refresh of store page
			storePageEvoke($scope);
			servicesPageEvoke($scope)
			SyncObj.setStorePageState(true);

			//Evoke Loyalty settings if exist (KICKBACK)
			//var tempKickbackSignedIn = globals.userAccount.get('KickbackSignedIn')
			//if(getKickbackSignedIn() == 1){
			kickbackGameEvoke(ClientSet);
			console.log('KB user account exist');
			//}

			//Evoke Loyalty settings if exist (ELECTRUM)
			ElectrumEvoke(ClientSet, $rootScope);

			if(globals.debugApp){
				console.log('setStorePagelState set to TRUE');
				//alert('setStorePagelState set to TRUE');
			}
		}

		if( SyncObj.getPromoHeadersState() && SyncObj.getPromosState() && SyncObj.getPromosImagesState() && !SyncObj.getPromoPageState() ){

			if( SyncObj.getNewPromosState() || window.device.platform == 'Android' ){ //Android needs Loader everytime
				
				promoPageEvoke($scope,0);
				prizePageEvoke($scope);

				SyncObj.setPromoPageState(true);

				if(globals.debugApp){
					console.log('setPromoPageState set to TRUE');
				}

				//Evoke refresh of promo carousel
				promoCarouselEvoke($scope, $rootScope, $timeout, $compile); // PROMO CAROUSEL EVOKE			

				SyncObj.setPromoCarouselState(true);

				if(globals.debugApp){
					console.log('setPromoCarouselState set to TRUE');
					//alert('setPromoPageState and setPromoCarouselState set to TRUE');
				}
			}
			else{
				if(globals.debugApp){
					console.log('no new promos - no need to evoke Promo content');
					//alert('no new promos - no need to evoke Promo content');
				}

				promoPageEvoke($scope,0);
				prizePageEvoke($scope);
				
				SyncObj.setPromoCarouselState(true);
				SyncObj.setPromoPageState(true);
			}
		}

		//Set 'FINISH STATE'
		if( SyncObj.getPromoPageState() && SyncObj.getStorePageState() && SyncObj.getPromoCarouselState() && SyncObj.getSyncEndState() ){
			//Set finish state
			SyncObj.setFinishedState(true);

			if(globals.debugApp){
				console.log('Finished State set to TRUE');
				//alert('Finished State set to TRUE');
			}
		}

		if( SyncObj.getFinishedState() ){
			//cancle this interval method
			if(globals.debugApp){
				console.log('interval cleared. SYNC FINISHED. SAVING DATA.');
			}

			globals.syncIsDone = true;
			//alert('SYNC FINISHED. SAVING DATA.');

			//SAVE DATA
			saveLocalData(); //save user account
			setTimeout(function(){
				SaveAppDataJson(); // Save all the Data that was initialized into the FILE
			}, 100);

			clearInterval(intervalId);
		}
	};
}

function stopSync(){
	if(globals.syncIsDone == false){
		clearInterval(intervalId); //clear interval that checks sync progress
		globals.syncIsDone = true;// helps to cancel ajax calls 

		if(globals.debugApp){
			console.log('SYNC DEBUG: stopSync was called. SYNCING WAS CANCELED. INTERVAL CLEARED.');
		}
	}
}

function isSyncFinished(){
	return globals.syncIsDone;
}



// GLOBAL SYNC FUNCTIONS //
//----------------------/-/

function refreshAjax($scope){
	//TODO Refresh KB game URL after register and log ins.

	if(globals.page == 'Specials'){
		if(window.device.platform == 'Android'){
			var options = {
				label : "Refreshing specials.."
			};
		}
		else{
			var options = {
				label : "Refreshing specials..",
				color : '#000',
				spinnerColor : '#000'
			}; //if this is an iOS device, change the loader color since it defaults to white and can blend in to the loadingpage
		} 
	}
	else{
		if(window.device.platform == 'Android'){
			var options = {
				label : "..."
			};
		}
		else{
			var options = {
				label : "...",
				color : '#000',
				spinnerColor : '#000'
			}; //if this is an iOS device, change the loader color since it defaults to white and can blend in to the loadingpage
		} 
	}

	window.wizSpinner.show(options);
}

function kickbackGameEvoke(ClientSet){ //Use this when user is a KB club member

	var kickBackShow = ClientSet.getKickbackURL() && ClientSet.getKickbackUser() && ClientSet.getKickbackPass();

	if(kickBackShow){
		//TODO : CHECK TO SEE IF KICKBACK IS ENABLED
		//var scope = angular.element($("#kickback-punch-card")).scope();
		//scope.updateKbGame();

		var scope = angular.element($("#kbIframeDirective")).scope();
		scope.updateKbGameURL();

		var scope = angular.element($("#connect-kickback-rewards")).scope();
		scope.$broadcast('kickback-rewards:refresh');//for kb clients	

		console.log('kickbackGameEvoke fired.');
	}
}

function ElectrumEvoke(ClientSet, $rootScope){
	//update URL, the only setting that gets changed
	// *Electrum Client Sets Init* //
	$rootScope.enableElectrum = ClientSet.getElectrumURL() && ClientSet.getElectrumAppID();

	if($rootScope.enableElectrum ){
		$rootScope.electrumMenuMain = ClientSet.getElectrumMenuMain();
		$rootScope.electrumMenu47 = ClientSet.getElectrumMenu47();
		$rootScope.electrumURL = ClientSet.getElectrumURL();
		$rootScope.electrumAppID = ClientSet.getElectrumAppID();
		$rootScope.electrumServerErr = ClientSet.getElectrumServerErr();
		$rootScope.electrumBarcode = getElectrumBarcode();
		$rootScope.electrumPoints = false; // will replace with ClientSet when this feature is fully implemented
		$rootScope.electrumNow47 = ClientSet.getElectrumNow47();
		$rootScope.ElectrumImage = globals.Change.electrumImage;

		console.log('Electrum was refreshed - ElectrumEvoke()');
		console.log("** Electrum URL received: " + $rootScope.electrumURL );

		if(globals.debugApp){
			console.log("** Electrum URL received: " + $rootScope.electrumURL );
			console.log("** Electrum APP ID received: " + $rootScope.electrumAppID );
			console.log("** Electrum Enabled: " + $rootScope.enableElectrum );
			console.log("**** Electrum Barcode: " + $rootScope.electrumBarcode );
		}
	}
	else{
		console.log('Electrum refresh is not needed so it will Not refresh. - ElectrumEvoke()');
	}
}

// PAGE EVOKE FUNCTIONS //
function storePageEvoke($scope){
	var scope = angular.element($("#storesDirective")).scope();
	scope.showLatestStoreData();
}


function prizePageEvoke($scope){
	$("#prizeWrapper").empty();
	//console.log('Prizes in scope (prizes REFRESH) : ' + JSON.stringify( $scope.PrizeHeaders ) );
	var scope = angular.element($("#prizesDirective")).scope();
	scope.showLatestPrizeData();

	setTimeout(function(){ //PRIZE PAGE REFRESH WITH SCOPE.APPLY
		scope.prizePageRefresh();
	},10);
}//END of prizePageEvoke


function promoPageEvoke($scope, type){
	var scopeOnPage = angular.element($("#PromosDirective")).scope(); // Grab the scope on Promo Page that has 'onPage' variable

	if(type == 1 || scopeOnPage.onPage == true){  //if type is 1 or when user is on the promos screen
		var proScope = angular.element($("#PromosDirective")).scope(); // Grab the scope on Promo Page that has 'promoPageRefresh()' method
		proScope.promoPageRefresh();
	}
	else{
		var proScope = angular.element($("#PromosDirective")).scope(); // Grab the scope on Promo Page that has 'promosRefreshed' variable
		proScope.promosRefreshed = true;
	}
}//END of prizePageEvoke



// PAGE EVOKE FUNCTIONS //
function servicesPageEvoke($scope){
	var serScope = angular.element($("#StoreServDirective")).scope();
	serScope.showLatestServicesData();
}



function promoCarouselEvoke($scope, $rootScope, $timeout, $compile){
	if( !globals.spinnerIsActive ){
		startPromoCarouselLoader();
	}

	$scope.Promos = getAllPromos(0, 'promoPageEvoke');

	if(globals.debugApp){
		if($scope.Promos.length > 0){
			console.log('Now-Carousel-Success: Promos exist for the NOW CAROUSEL');
		}
		else{
			console.log('Now-Carousel-Error: Promos DO NOT exist for the NOW CAROUSEL!!');
		}
	}

	// ** Now Promo Carousel ** //
	var contentheight = $('#main .ui-content').height();

	// CAROUSEL START UP //
    if( globals.emptyCarousel ){
		$('#mainCarousel').data('touchCarousel').destroy(); //destroy the static carousel (important as it doesnt support resizing or adding/sub)
    }
	
	$("#mainCarouselWrapper #cWrapper").empty();

	var carouselHTMLcontainer = '<div id="mainCarousel" class="touchcarousel minimal-light">' +
	    	'<ul id = "nowScreenCarousel" class="touchcarousel-container">' + 
	    		'<li class = "touchcarousel-item" ng-repeat = "Promo in Promos | limitTo: 8">' +
		    		'<img class = "tcimage mainCarouselImage" src = "{{Promo.PhotoURL}}"/>' +
	    		'</li>' +
	   		'</ul>' +
		'</div>';

	setTimeout(function(){

		var scope = angular.element($("#main")).scope();
		scope.setPromosForCarousel($scope.Promos);
		$("#mainCarouselWrapper #cWrapper").html( $compile ( carouselHTMLcontainer )( $scope ) ); //Append new elements

		setTimeout(function(){
			
			// ** Now Promo Carousel ** //
			$("#mainCarouselWrapper .ui-grid-b").css('height', contentheight * .333);
			$("#pCarouselLoading").css('height', $("#mainCarouselWrapper .ui-grid-b").height() );
			var promoModHeight = $('#mainCarouselWrapper .ui-grid-b').height();
			$("#mainCarouselWrapper #cWrapper").css('height', promoModHeight);
			$("#mainCarouselWrapper .tcimage").css('height', promoModHeight );

			var promowidth = Math.ceil(promoModHeight * 1.25);
			$("#mainCarouselWrapper .tcimage").css('width', promowidth + 'px' );
			$("#mainCarousel .touchcarousel-wrapper").css('height', promoModHeight );
			$("#mainCarouselWrapper .touchcarousel-item").css('height', promoModHeight );
			$("#mainCarouselWrapper .touchcarousel-item").css('width', $('.nowPromoModule .tcimage').width() );
			$("#mainCarouselWrapper .touchcarousel").css('height', promoModHeight);

			setTimeout(function(){
			
				if (!$('#mainCarouselWrapper').hasClass('ng-hide') && $scope.Promos.length > 0 ){ //checking to see if the carousel will be shown
					if(globals.debugApp){
						console.log('STARTING PROMO CAROUSEL NOW MODULE');
					}

					$('#mainCarousel').touchCarousel({ 
						pagingNav: false,
						directionNav: false,
						snapToItems: true,
						scrollbar: false,
						loopItems: true,
						autoplay: true,
						Delay: 1500,
						transitionSpeed: 400,
						scrollToLast: true,
						useWebkit3d: true
					});

					setTimeout(function(){
						$('#mainCarousel').css('margin-left', ( $(window).width() * .5 - $("#mainCarouselWrapper .tcimage").width() / 1.82 ) );  //centering the touch carousel so that the focused image is in the horizontal center of the page
						
						setTimeout(function(){
							$("#mainCarousel").animate({opacity:1}, 500);

							setTimeout(function(){
								stopPromoCarouselLoader(); //Stop loader in now promo carousel
							}, 100);
						}, 10);
					}, 30);
				}
			}, 100);
		},600);	
	},100);
}//End of promoCarouselEvoke



//NOTE: MAKE SURE TO SAVE / LOAD PRESORTED ARRAY  (NVM , MIGHT NOT NEED TO)


// Get All Headers with Promos - Function //
//****************************************//
//TODO: MOVE THIS TO ANGULAR SERVICE AND DEPENDENCY INJECTION
function getAllHeadersWithPromos(trace){ //returns all headers with their promotions (used on the promo screen) stored on the device
	if(globals.debugApp){
		if(trace){
			console.log('DEBUG-SYNC: calling getAllHeadersWithPromos() from - ' + trace);
		}
		console.log("getAllHeadersWithPromos() initiated");
	}

	var finalHeaders = []; //array to hold the result sent to the user
	var headers = LoadHeadersFromAppData() ? LoadHeadersFromAppData() : null;

	if (headers){
		headers.sort(function(a, b){
			return a.SortOrder - b.SortOrder;
		}); //sort headers by their sort order

		var promos = getAllPromos(0, 'getAllHeadersWithPromos()'); //get all regular promotions (not prizes)

		for (var i = 0; i < headers.length; i++){
			headers[i].Promos = promos.filter(function(item){
				return (item.HeadID == headers[i].HeadID);
			});
		} //for each header, push the promotions that have that header's headerid
		for (var i = 0; i < headers.length; i++){
			if (headers[i].Promos.length > 0){
				finalHeaders.push(headers[i]);
			}
		} //for each header only use the ones that have promotions
	}

	return finalHeaders;
}


// Update Promo Link / Video / Photo - Function //
//----------------------------------------------//
//TODO: MOVE THIS TO ANGULAR SERVICE AND DEPENDENCY INJECTION
function updatePromoLinkVideoPhoto(p){ //helper function for checking and updating promotions that have a link and/or youtube video as part of the Descript2 such as: [http://www.google.com]Check out Google!
 	var Descript2 = p.Descript2;
	if (Descript2.indexOf('[') == -1){ //if there is no bracket then this is a regular Descript2 (with no link)
		var PromoLink = '';
		var PromoVideo = '';
		var PromoVideoImage = '';
	}
	else{
		if (Descript2.indexOf('youtube') == -1){ //this is a regular link (not youtube video)
			var PromoLink = Descript2.substring(1, Descript2.indexOf(']'));
			var PromoVideo = '';
			var PromoVideoImage = '';
		}
		else{ //this is a youtube video link
			var PromoLink = '';
			var PromoVideo = Descript2.substring(1, Descript2.indexOf(']'));
			var youtubeID = Descript2.substring(Descript2.indexOf('=') + 1, Descript2.indexOf(']')); //extract the youtube video id from the url
			var PromoVideoImage = 'http://img.youtube.com/vi/' + youtubeID + '/0.jpg'; //if this is a youtube link, use the youtube API to set the promovideoimage as the video thumbnail
		}
		Descript2 = Descript2.substring(Descript2.indexOf(']') + 1); //set the Descript2 to the text after the [link]
	}

	p.Descript2 = Descript2;
	p.PromoLink = PromoLink;
	p.PromoVideo = PromoVideo;
	p.PromoVideoImage = PromoVideoImage;
	
	return p;
 }


 // Get Age - Function //
//*******************//
//returns the age of the user (if available)
function getAge(){ 
	if(globals.debugApp){ 
		console.log('DEBUG-SYNC: Getting Age in getAge()')
	}
	//var birthdayEpoch = parseInt(window.localStorage.getItem('Birthday')); //get birthday from local storage which is stored as unix epoch time
	var birthdayEpoch = parseInt(getBirthday()); 

	if (!birthdayEpoch){
		return 0;
	}
	else{
		var fromNow = moment(birthdayEpoch).month(0).from(moment().month(0)); //get the difference between the user's birthday and today
		if (fromNow.indexOf('years') == -1){ //no years string in fromNow
			return 0; //the user is 0 years old
		}
		else{
			return parseInt(fromNow.substring(0, fromNow.indexOf('y') - 1)); //return the number of years to show the age
		}
	}
}


//TODO: GLOBAL FUNCTION TO ANGULAR JS SERVICE
//Function for getting all promotions saved on the device by type (0 is a regular promo and 1 is a prize)
function getAllPromos(type, trace){ 
	if(globals.debugApp){
		if(trace){
			console.log('DEBUG-SYNC: calling getAllPromos() from - ' + trace);
		}
		console.log("Getting all Promos in (getAllPromos())..");
	}

	var use = []; //array to hold the promos that will be sent back
	var p = LoadPromosFromAppData() ? LoadPromosFromAppData() : null; 
	var pi = LoadPromoImagesFromAppData() ? LoadPromoImagesFromAppData() : null; 

	//console.log('getAllPromos:  p = ' + JSON.stringify(p) );
	
	if (p && pi){
		//console.log('promoimages test - ' + JSON.stringify( pi ) );
		p = p.filter(function(item){
			return (item.PromoType == type);
		}); //filter promos based on type
		
		if (p.length){

			var myAge = getAge();
			var myInterests = getInterests();
			var now = moment().tz('America/New_York').toDate().getTime() - globals.timeZoneOffset;

			for (var i = 0; i < p.length; i++){
				p[i] = updatePromoLinkVideoPhoto(p[i]); //see function definition above
				var promoInterestID = p[i].InterestID;
				var promoAge = p[i].Age;
				var usePromo = false;
				var endDate = p[i].DateEnd; //promo end date

				if(now > p.DateStart && now < p.DateEnd){
					usePromo = false;
					if(globals.debugApp){
						console.log('GetAllPromos-Debug: A promo was expired or was not valid yet');
					}
				}

				if (promoInterestID){
					for (var j = 0; j < myInterests.length; j++){
						if (myInterests[j].InterestID == promoInterestID){
							usePromo = true;
						}
					}
				} //if the promo has an interest id only use it if it is part of the user's interests
				else{
					usePromo = true;
				}

				p[i].PhotoURL = pi[p[i].PhotoFile]; //set the promo PhotoURL to the matching value in the PromoImages object
				//console.log("SYNC-DEBUG: promo photofile being processed in getAllPromos()" + i + ' -- ' + pi[p[i].PhotoFile]); // do not use

				if (usePromo && ((promoAge && myAge < promoAge) || !p[i].PhotoURL)) {
					//if the promo has the age flag set (promoAge) make sure the user is the right age and 
					//make sure the photo exists from promo images
					usePromo = false;
					if(globals.debugApp){
						console.log("GetAllPromos-Debug: promoImage was missing or the age filter was used..");
					}
				}

				if (usePromo){
					use.push(p[i]); //push the promo to the array we will send back

					if (p[i].PromoStore){ 
						use[use.length - 1].hasStores = true;  //promo has Stores FLAG
					}
					else{
						use[use.length - 1].hasStores = false;  //promo has Stores FLAG
					}

					/* GET RID OF GRABBING STORES FOR PROMO - MOVE TO PROMO SPECIFIC PAGE  - DONE */
					if (!use[use.length - 1].Stores){
						use[use.length - 1].Stores = []; //if no stores have been added, add an empty array as the stores property
					}
				}
				else{
					if(globals.debugApp){
						console.log("GetAllPromos-Debug: promo couldnt be used!! in getAllPromos() sync.js");
					}
				}
			}
		}
		else{
			if(globals.debugApp){
				console.log("GetAllPromos-Debug: p.length was null or zero in getAllPromos()" );
			}
		}
	}
	else{
		if(globals.debugApp){
			console.log("GetAllPromos-Debug: Promos or promo images arrays were empty!!! in getAllPromos()" );
		}
	}
	return use;
}


//TODO: This should be inside a angular service instead of a global function and dependency injected
// Get All Services - Function //
//****************************//
//function to get all Services stored on the device
function getAllServices(){ 
	var services = LoadServicesFromAppData('getAllServices()') ? LoadServicesFromAppData('getAllServices()') : null;

	if (!services || services.length < 1){
		//console.log('DEBUG-SYNC: DEFAULT Services was loaded -- getAllServices()');
		services = globals.installServices;

		services.sort(function(a, b){
			return a.SortOrder - b.SortOrder;
		});
		//console.log('DEBUG-SYNC: sorting finished-- getAllServices()');

		//make sure all Services are not selected by default (the user selects and deselects services to filter stores on the stores screen)
		for (var i = 0; i < services.length; i++){
			services[i].Selected = false;
		} 
		if(globals.debugApp){
			console.log('DEBUG-SYNC: defaulting selected finished -- getAllServices()');
		}

	}
	else{
		if(globals.debugApp){
			console.log('DEBUG-SYNC: Services was loaded from file in getAllServices()');
		}
	}
	return services;
}



//TODO: GLOBAL FUNCTION. SHOULD BE USED IN A SERVICE WHEN EVERYTHING IS WORKING AND INJECTED
function getAllStores($scope, trace){ 
	if(globals.debugApp){
		if(trace){
			console.log('DEBUG-SYNC: calling getAllStores() from - ' + trace);
		}
	}

	var stores = LoadStoresFromAppData() ? LoadStoresFromAppData() : null;

	if (!stores || stores.length < 2){
		if(globals.debugApp){
			console.log('DEBUG-SYNC: Using default Store List!! ');
		}
		stores = globals.installStores; //if no stores are saved on the device (due to perhaps lack of internet that caused the sync process to fail or this is the first time using the app and the stores are still being downloaded in the background) then use the stores in js/installTables.js
	}
	else {
		if(globals.debugApp){
			console.log('DEBUG-SYNC: A saved d/l Store List was found!! ');
		}
	}

	var storeservices = LoadStoreServicesFromAppData() ? LoadStoreServicesFromAppData() : null;

	if(globals.debugApp){
		console.log('Store Services Debug: store services was loaded in getAllStores()');
		//console.log('Store Services Debug: store services  = ' + JSON.stringify( storeservices ) ); //Comment out when not debugging Store Services
	}

	if (!storeservices || storeservices.length < 1){
		storeservices = globals.installStoreServices;
		if(globals.debugApp){
			console.log('Store Services Debug: USING DEFAULT store services file ---- in getAllStores()');
		}
	}

	var servicesArr = getAllServices(); // Get all services that were created for the app's services filter

	for (var i = 0; i < stores.length; i++){ //for each store
		var temp = storeservices.filter(function(item){
			return (item.StoreID == stores[i].StoreID);
		}); //get all StoreServices that match this Store's id

		var s;// Get all services that were created for the app's services filter

		s = servicesArr.filter(function(item){
			return (inThisStore(item.ServiceID));
		}); //filter all services to only include services in this store
		
		function inThisStore(serviceid){ //helper function to check if a service is in the current store (by checking if the serviceid is in the temp array of StoreServices which was filtered to only show StoreServices for this store)
			for (var k = 0; k < temp.length; k++){
				if (temp[k].ServiceID == serviceid){
					//console.log('Store Services Debug: Comparing of this pair was True - ' + temp[k].ServiceID + ' / ' + serviceid);
					return true;
				}
				else{
					if(globals.debugApp){
						console.log('Store Services Debug: Comparing of this pair was FALSE !!! - ' + temp[k].ServiceID + ' / ' + serviceid);
					}
				}
			}
			return false;
		}

		stores[i].Services = []; //add a Services property to the store with an empty array
		for (var j = 0; j < s.length; j++){
			stores[i].Services.push(s[j]);
		} //add all the services we found above

		//console.log( "Store Services Debug: FINAL Services injected for this store : " + JSON.stringify( stores[i].Services ) );
	}
	if(globals.debugApp){
		console.log('Finished getting all stores in - getAllStores()');
	}
	return stores;
}


//TODO: Move this method to a service and inject where it is needed
//function for getting interests
function getInterests(){ 
	var interests = GetItemFromLocalData('Interests');

	if (!interests){
		return [];
	}
	else{
		return JSON.parse(interests);
	}
}



// FUNCTION:  PARSE SERVER OK //
//used in js/app.js to determine if the Server is Ready based on the response from the ServerOK web method
function parseServerOK(data) {
	var output;
	if ($(data).find('ServerOKResult').children("Accessed").text() == 'true'){
		output = true;
	}
	else{
		output = false;
	}
	return output;
}



var TempDataVersion;
function setSyncVersion(){
	TempDataVersion = GetItemFromLocalData('DataVersion');
}



//FUNCTION: PARSE SYNC START //
//used in js/app.js to determine if the data versions match (no download sync needed) or not (download sync is needed)
function parseSyncStart(data) {
		var output = false;
		var AlwaysSync = parseInt($(data).find('SyncStartResult').children("AlwaysSync").text());
		var DataVersion = $(data).find('SyncStartResult').children("DataVersion").text();

		//if (AlwaysSync == 1 || DataVersion != window.localStorage.getItem('DataVersion')){
		if(globals.debugApp){
			console.log('DataVersion compared: ' + DataVersion + '  / ' + GetItemFromLocalData('DataVersion') );
		}

		if (AlwaysSync == 1 || DataVersion != GetItemFromLocalData('DataVersion') ){
			TempDataVersion = DataVersion; //set TempDataVersion to the server data version so that the client data version can be updated later (see syncend function on this page)
			output = true;
		}
		return output; //return whether or not we need a download sync
}//END





// FUNCTION: XML TO STRING
function xmlToString(xmlData) {
    var xmlString;
    //IE
    if (window.ActiveXObject){
        xmlString = xmlData.xml;
    }
    // code for Mozilla, Firefox, Opera, etc.
    else{
        xmlString = (new XMLSerializer()).serializeToString(xmlData);
    }
    return xmlString;
}//END




//FUNCTION: SEND PUSH TOKEN TO SERVER //
//AJAX function for downloading promo headers, used only in a download sync, called in js/app.js
var tokenRetry = 0;
function sendPushTokenToServer() {
	$.ajax({ //ajax call for set push web method
		url: GetItemFromLocalData('ClientURL'),
		type: "POST",
		dataType: "xml",
		timeout : 16000,
		data: soapMessageSetPush(),
		success: function(data){
			saveClientLog(12, 0, 0, 'push token that was received by server : ' + globals.userAccount.get('PushToken') );
			if(globals.debugApp){
				console.log('SUCCESS: push token sent to server: ' + globals.userAccount.get('PushToken') );
			}	
		},
		error: function(error){
			saveClientLog(13, 0, 0, 'push token was not sent to server: ' + error);
			if(globals.debugApp){
				console.log('ERROR: pushing token was not sent to server' + error);
			}

			//push token retry (retry on)
			if(tokenRetry < 4){
				tokenRetry = tokenRetry + 1;
				setTimeout(function(){
					sendPushTokenToServer();
				},10000);
			}
		},
		contentType: "text/xml; charset=utf-8"
	});
} //END



//--------------------//
//  REFRESH METHODS   //
//--------------------//
function refreshClientSet() {
	//console.log("Client Set download Requested");
	var error=false;
	$.ajax({
		url: GetItemFromLocalData('ClientURL'), //the ClientURL we stored earlier
		type: "POST", //Alway use POST, the web service is SOAP based not RESTful
		dataType: "xml", //Always xml, the webservice does not use JSON
		timeout : 30000, //The ajaxTimeout object is defined in js/app.js
		data: soapMessageClientSetAll(), //soap messages are defined in js/soapMessages.js
		success: function(data){ //ajax function successs callback (i.e. the function did not time out for lack of internet or something 
			//else happened that caused an error in the actual web service call like the server is down, ..etc)
			error = false;
			if(globals.debugApp){
				console.log("SUCCESS: REFRESH ClientSet() was downloaded.");
			}

			saveClientLog(90, 0, 0, '', 0, 'SUCCESS: REFRESH ClientSet() was downloaded.');
			var cs = {};

			$(data).find('RClientSetServer').each(function(){ //for each RclientSetServer object in the XML response (for more details on response format for various web methods, see the webservice specification: http://74.208.209.112/Market_GreenSquare_001/WebService/WebService1.asmx)
				var ItemName = $(this).children("ItemName").text(); //The name of the setting
				var ItemValue = $(this).children("ItemValue").text(); //The value of the setting
				cs[ItemName] = ItemValue; //Save the setting as a property of the cs object defined above; the ItemName is the key and the ItemValue is the value
			});

			//save to localStorage, localStorage is basically the local database for this application; 
			//JSON.stringify converts arrays or objects into json string literals, localStorage can only save strings
			SaveClientSetsToAppData( JSON.stringify(cs) );

			refreshStoreList();// Move to storelist now
		},
		error: function(data){
			if(globals.debugApp){
				console.log("ERROR: downloadClientSet() was not downloaded.");
			}
			saveClientLog(91, 0, 0, '', 0, 'ERROR: REFRESH downloadClientSet() was not downloaded.');
		},
		contentType: "text/xml; charset=utf-8"
	}).done(function(){
		if(globals.debugApp){
			console.log("ClientSet download DONE");
		}
	});
} //END


//AJAX function for downloading stores, used only in a download sync
function refreshStoreList(){
	$.ajax({
		url: GetItemFromLocalData('ClientURL'),
		type: "POST",
		dataType: "xml",
		timeout : 20000,
		data: soapMessageStoreListAllByDescript(),
		success: function(data){

			var stuff = []; //local array to hold the stores
			if(globals.debugApp){
				console.log("SUCCESS: Store List was downloaded");
			}

			saveClientLog(94, 0, 0, '', 0, 'SUCCESS: REFRESH Store List was downloaded');
			//console.log('Stores Data: '+xmlToString(data)+"--"+window.localStorage.getItem('ClientURL')+"--"+soapMessageStoreListAllByDescript());

			$(data).find('RStoreServer').each(function(index){
				var StoreID = parseInt($(this).children("StoreID").text());
				var Descript = $(this).children("Descript").text();
				//parseInt is used when saving the value if the expected value is an int

				var Address1 = $(this).children("Address1").text();
				var Address2 = $(this).children("Address2").text();
				var City = $(this).children("City").text();
				var State = $(this).children("State").text();
				var Zip = $(this).children("Zip").text();
				var Phone = $(this).children("Phone").text();
				var Latitude = parseFloat($(this).children("Latitude").text()); //parseFloat is used when saving the value if the expected value is a real number (i.e. not an int but has decimals)
				var Longitude = parseFloat($(this).children("Longitude").text());
				var RegularPrice = Math.round(parseFloat($(this).children("RegularPrice").text())*100)/100; //Math.round limits the decimal number to two places
				var PlusPrice = Math.round(parseFloat($(this).children("PlusPrice").text())*100)/100;
				var PremiumPrice = Math.round(parseFloat($(this).children("PremiumPrice").text())*100)/100;
				var DieselPrice = Math.round(parseFloat($(this).children("DieselPrice").text())*100)/100;
				var KerosenePrice = Math.round(parseFloat($(this).children("KerosenePrice").text())*100)/100;
				var GasLastUpdate = $(this).children("GasLastUpdate").text();
				stuff.push({
					StoreID : StoreID,
					Descript: Descript,
					Address1 : Address1,
					Address2 : Address2,
					City : City,
					State : State,
					Zip : Zip,
					Phone : Phone,
					Latitude : Latitude,
					Longitude : Longitude,
					RegularPrice : RegularPrice,
					PlusPrice : PlusPrice,
					PremiumPrice : PremiumPrice,
					DieselPrice : DieselPrice,
					KerosenePrice : KerosenePrice,
					GasLastUpdate : GasLastUpdate
				});
			});

			//GENERATE timeStamp and store it in rootscope, this is displayed with the redemption
			var currentdate = new Date(); 
			var storesSyncDate = "StoreList last synced: " + getMonth(currentdate) + " " + addOrd(currentdate.getDate()) + " " + " @ " + getTime(currentdate);
		    window.localStorage.setItem('StoresSyncDate', storesSyncDate);

			globals.TempStores = JSON.stringify(stuff); //save the stringified stores in the globals TempStores variable, TempStores will replace local Storage stores in the sync end function
			//we do not save localStorage automatically here so that Stores, Services, and StoreServices are all saved at the same time and are on the same DataVersion

			SaveStoresToAppData( globals.TempStores );
			globals.TempStores = null;

			SetItemInLocalData('StoreListRetry', 0);

			refreshServices();

		},
		error: function(data){
			if(globals.debugApp){
				console.log("ERROR: Store List was not downloaded");
			}
			saveClientLog(93, 0, 0, '', 0, 'ERROR: REFRESH Store List was not downloaded');
			globals.ajaxTimedOut = true;
		},
		contentType: "text/xml; charset=utf-8"
	});
}//END



function refreshStoreServices() {
	$.ajax({
		url: GetItemFromLocalData('ClientURL'),
		type: "POST",
		dataType: "xml",
		timeout : 20000,
		data: soapMessageStoreServiceListAll(),
		success: function(data){
			if(globals.debugApp){
				console.log('SUCCESS: Store services downloaded');
			}

			saveClientLog(95, 0, 0, '', 0, 'SUCCESS: Store services downloaded');
			var stuff = [];

			$(data).find('RStoreServiceServer').each(function(index)
			{
				var ServiceID = $(this).children("ServiceID").text();
				var StoreID = parseInt($(this).children("StoreID").text());
				stuff.push({
					ServiceID : ServiceID,
					StoreID : StoreID
				});
			});

			globals.TempStoreServices = JSON.stringify(stuff);
			SaveStoreServicesToAppData( globals.TempStoreServices );
			globals.TempStoreServices = null;
		},
		error: function(data){
			if(globals.debugApp){
				console.log('ERROR: Store services was not downloaded');
			}
			saveClientLog(96, 0, 0, '', 0, 'ERROR: Store services was not downloaded');
			globals.ajaxTimedOut = true;
		},
		contentType: "text/xml; charset=utf-8"
	});
}


function refreshServices() {
	$.ajax({
		url: GetItemFromLocalData('ClientURL'),
		type: "POST",
		dataType: "xml",
		timeout : 20000,
		data: soapMessageServiceListAll(),
		success: function(data){
			if(globals.debugApp){
				console.log('SUCCESS: REFRESH Services downloaded');
			}
			saveClientLog(97, 0, 0, '', 0, 'SUCCESS: REFRESH Services downloaded');
			var stuff = [];

			$(data).find('RServiceServer').each(function(index){
				var ServiceID = $(this).children("ServiceID").text();
				var Descript1 = $(this).children("Descript1").text();
				var SortOrder = parseInt($(this).children("SortOrder").text());
				stuff.push({
					ServiceID : ServiceID,
					Descript1 : Descript1,
					SortOrder : SortOrder
				});
			});

			globals.TempServices = JSON.stringify(stuff);
			SaveServicesToAppData( globals.TempServices );
			globals.TempServices = null;

			refreshStoreServices();
		},
		error: function(data){
			if(globals.debugApp){
				console.log('ERROR: Services was not downloaded');
			}
			saveClientLog(97, 0, 0, '', 0, 'ERROR: Services was not downloaded');
			globals.ajaxTimedOut = true;
		},
		contentType: "text/xml; charset=utf-8"
	});
}//END




function refreshPromoHead($scope, $rootScope, $timeout, $compile, type){
	if(globals.debugApp){
		console.log("refreshPromoHead()  initiated");
	}

	var error = false;

	$.ajax({
		url: GetItemFromLocalData('ClientURL'),
		type: "POST",
		dataType: "xml",
		timeout : 20000,
		data: soapMessagePromoHeadListAll(),
		success: function(data){
			if(globals.debugApp){
				console.log("SUCCESS: REFRESH Promo Head Downloaded via refreshPromoHead()");
			}

			saveClientLog(97, 0, 0, '', 0, 'ERROR: REFRESH Promo Head Downloaded via refreshPromoHead()');
			error = false;
			var stuff = [];
			$(data).find('RPromoHeadServer').each(function(index){
				var HeadID = $(this).children("HeadID").text();
				var Descript1 = $(this).children("Descript1").text();
				var SortOrder = parseInt($(this).children("SortOrder").text());
				//console.log("promo header received: " + HeadID + ":" + Descript1 + ":" + SortOrder);

				stuff.push({
					HeadID : HeadID,
					Descript1 : Descript1,
					SortOrder : SortOrder
				});
			});

			SaveHeadersToAppData( JSON.stringify(stuff) );
		},
		error: function(data){
			if(globals.debugApp){
				console.log("ERROR: REFRESH Promo Head was not Downloaded");
			}

			saveClientLog(98, 0, 0, '', 0, 'ERROR: REFRESH Promo Head was not Downloaded');

			setTimeout(function(){	
				screenRefresh($scope, $rootScope, $timeout, $compile, type);
			}, 1000);
		},
		contentType: "text/xml; charset=utf-8"
	}).done(function(){
		if(globals.debugApp){
			console.log("refreshPromoHead()  DONE - going into refreshPromo()");
		}

		refreshPromo($scope, $rootScope, $timeout, $compile, type); //if no ajax error was thrown, go onto downloadpromo
	});
}//END


function storeRefresh(){
	refreshClientSet(); //starts with Clientsets and ends with Store Services
}

function storeRefreshFinished(){

	globals.Stores = getAllStores();

	setTimeout(function(){
		var scope = angular.element($("#storesDirective")).scope();
		scope.showLatestStoreData();
	}, 10);

	setTimeout(function(){
		$('#storesLoading').animate({opacity:0}, 400);
		$('#storeList').listview('refresh');

		setTimeout(function(){
			$('#storesLoading').hide();
			window.wizSpinner.hide();
			$('#storeList').listview('refresh');

			if(globals.debugApp){
				console.log("ListView Refreshed! ");
			}
		}, 400); // about 3 secs is needed for showlateststoredata
	}, 400);
}


//TODO - Create logic to delete unused PromoImages from PromoImage array
function refreshPromo($scope, $rootScope, $timeout, $compile, type){ 
	var error;

	$.ajax({
		url: GetItemFromLocalData('ClientURL'),
		type: "POST",
		dataType: "xml",
		timeout : 35000,
		data: soapMessagePromoList2(),
		success: function(data){
			if(globals.debugApp){
				console.log("SUCCESS: REFRESH Promos downloaded");
			}

			saveClientLog(99, 0, 0, '', 0, 'SUCCESS: REFRESH Promos downloaded');
			error = false;
			var savepromos = []; //the promos that will be saved to localstorage
			var newpromos = [];	//promos that are new
			var savepromostores = []; //promostores that will be saved to localstorage
			//gets the promoimage object from localstorage or creates a new one if not available, JSON.parse converts a string into a javascript array or object (it is the opposite of JSON.stringify)
			var PromoImages = LoadPromoImagesFromAppData() ? LoadPromoImagesFromAppData() : {}; 

			var promoImagesDone = 0; //indicates how many promo images have been downloaded from the server

			$(data).find('RPromoServer2').each(function(index){
				var PromoID = $(this).children("PromoID").text();
				var preDescText=$(this).children("Descript1").text();
				var Descript1 =preDescText;
				var Descript2 = $(this).children("Descript2").text();
				var PromoType = parseInt($(this).children("PromoType").text());
				var HeadID = $(this).children("HeadID").text();
				var InterestID = $(this).children("InterestID").text();
				var PromoStore = parseInt($(this).children("PromoStore").text());
				var PromoUse = parseInt($(this).children("PromoUse").text());
				var PromoAccount = parseInt($(this).children("PromoAccount").text());
				var DateStart = ticksToEpoch(parseInt($(this).children("DateStartTicks").text()));
				var DateEnd = ticksToEpoch(parseInt($(this).children("DateEndTicks").text()));
				var DisplaySec = parseInt($(this).children("DisplaySec").text());
				var Uses = parseInt($(this).children("Uses").text());
				var AccountUses = parseInt($(this).children("AccountUses").text());
				var Age = parseInt($(this).children("Age").text());
				var Points = parseInt($(this).children("Points").text());
				var DetailHTMLFile = $(this).children("DetailHTMLFile").text();
				var PhotoFile = $(this).children("PhotoFile").text();
				var ThumbFile = $(this).children("ThumbFile").text();
				var DescriptPush = $(this).children("DescriptPush").text();
				var FocusTime = $(this).children("FocusTime").text();

				if(InterestID){
					if(globals.debugApp){
						console.log("InterestID " + InterestID + " for " + Descript1 + " (Warning! this promo is using an Interest only ID!");
					}
				}

				if (FocusTime){
					FocusTime = parseInt(FocusTime); //focus time indicates if this promotion can be used in the focus now module: 0 indicates no, 1 is breakfast, 2 is lunch, 3 is dinner, and 4 is late night
				}
				else{
					FocusTime = 0;
				}

				var now = moment().tz('America/New_York').toDate().getTime() - globals.timeZoneOffset;

				if ((Uses == 0 || AccountUses < Uses) && now > DateStart && now < DateEnd){ 
				//make sure the promotion is valid (i.e. today's date is between the promo start and end date, also make sure the promo 
				//is not a redeemable promo with zero 'Uses' OR that the user has enough AccountUses to redeem the promotion)
					savepromos.push({
						PromoID : PromoID,
						Descript1 : Descript1,
						Descript2 : Descript2,
						PromoType : PromoType,
						HeadID : HeadID,
						InterestID : InterestID,
						PromoStore : PromoStore,
						PromoUse : PromoUse,
						PromoAccount : PromoAccount,
						DateStart : DateStart,
						DateEnd : DateEnd,
						DisplaySec : DisplaySec,
						Uses : Uses,
						AccountUses : AccountUses,
						Age : Age,
						Points : Points,
						DetailHTMLFile : DetailHTMLFile,
						PhotoFile : PhotoFile,
						ThumbFile : ThumbFile,
						DescriptPush : DescriptPush,
						FocusTime : FocusTime
					});
				}
				else{
					if(globals.debugApp){
						console.log("Promo is either invalid (date/uses) or not redeemable (event): " + PromoID);
					}
				}
			});

			//GENERATE timeStamp and store it in rootscope, this is displayed with the redemption
			var currentdate = new Date(); 
			var promosSyncDate = "Promos last synced: " + getMonth(currentdate) + " " + addOrd(currentdate.getDate()) + " " + " @ " + getTime(currentdate);
		    window.localStorage.setItem('PromosSyncDate', promosSyncDate);

		    var op =  LoadPromosFromAppData() ? LoadPromosFromAppData() : null; 

		    if(!PromoImages){ //Check to see if PromoImages are empty, if so then Old promos array has to be null
		    	op = null;
		    	PromoImages = {};
		    	if(globals.debugApp){
		    		console.log("promo images arr was empty. grabbing all images");
		    	}
		    }

		    if( op != null ){ // Check to see if old promos exist
		    	var oldpromos = op;
		    	if(globals.debugApp){
		    		console.log("There were Old promos grab");
		    	}
		    }else{
		    	var oldpromos = null;
		    }

			//determine which promos are new (i.e. they have not been saved to the app before)
			for (var i = 0; i < savepromos.length; i++){
				if (!inPromos(savepromos[i].PromoID, oldpromos)){
					newpromos.push(savepromos[i]);
				}
			} 

			if (newpromos.length > 0){
				$(data).find('RPromoStoreServer2').each(function(index){
					var PromoID = $(this).children("PromoID").text();
					var StoreID = parseInt($(this).children('StoreID').text());
					savepromostores.push({
						PromoID : PromoID,
						StoreID : StoreID
					});
				});
			}

			// Check for new Image Files, if there are any we need to download them by adding that promo to 'savedpromos' 
			if( oldpromos != null ){ // Check to see if old promos exist
		    	//A block of code to detect a change of promo image name, if there is one
		    	for (var i = 0; i < savepromos.length; i++){
		    		//Check if saved and old promos have the same PromoID
					for (var j = 0; j < oldpromos.length; j++){
						if (oldpromos[j].PromoID == savepromos[i].PromoID){
						
							//CHeck if promos do not contain the same image name, if not then
							if(savepromos[i].PhotoFile != oldpromos[j].PhotoFile){
								if(globals.debugApp){
									console.log("Sync-Debug: !! A new photofile was found for an Old Promo. Let's update it..");
								}
								newpromos.push(savepromos[i]);//push this promo into new promos so the image will be updated
							}
						}
					}
				} 
		    }


			if (newpromos.length > 0){
				//download and save all promostores if there are any new promotions, we do not only download promostores for newpromos 
				//because the logic to check whether or not this promostore is for a new or old promo would likely be slower than just 
				//downloading and saving all of them
				//Clear Promos
				SavePromoStoresToAppData(JSON.stringify(savepromostores));
				promoImagesDone = 0;
				var i = 0;

				downloadPromoImage(newpromos, i, function(){

					if(promoImagesDone >= newpromos.length){ //success call back for the downloadpromoimage function, if all the new promo images have been downloaded then save them and call promosDone function (defined below)
						
						if(globals.debugApp){
							console.log("SUCCESS in downloading all new promo imgs.");
						}

						saveClientLog(89, 0, 0, 'SUCCESS in downloading all new promo imgs.' );

						if (!error){ 
							//if there are any promos to be saved, save them to localStorage
							SavePromoImagesToAppData( JSON.stringify(PromoImages) ); //Save Promo images
							SavePromosToAppData(JSON.stringify(savepromos)); 

							SaveAppDataJson(); //SAVE DATA
						}
						else{
							if(globals.debugApp){
								console.log("error in Refresh Promos: " + error);
							}
						}

						storeRefresh();
						setTimeout(function(){
							$scope.Headers = getAllHeadersWithPromos('refreshPromos()'); 

							setTimeout(function(){
								$scope.Promos = getAllPromos(0,'refreshPromos()');
									
								setTimeout(function(){	
									if(type == 0){ // type is to tell whether its for Prizes or Promos
										promoPageEvoke($scope, 1);
									}
									else{
										prizePageEvoke($scope, $rootScope, $timeout, $compile);
									}
								}, 1200);
							}, 100);
						}, 1000);	

						if(globals.debugApp){
							console.log("SUCCESS: Promos/StoreList/Services/StoreServices were refreshed");
						}
						
					}
				}, function(){  //error call back for the download promo image function

					error = true;
					if(globals.debugApp){
						console.log("ERROR in downloading promo imgs in Promo Refresh");
					}

					saveClientLog(88, 0, 0, 'ERROR in downloading promo imgs in Promo Refresh' );
				});

				
			}
			else{
				if(globals.debugApp){
					console.log("SUCCESS.. promos that were expired or invalid were just removed.");
				}

				saveClientLog(89, 0, 0, 'SUCCESS.. promos that were expired or invalid were just removed.' );

				if (!error){ 
					//if there are any promos to be saved, save them to localStorage
					SavePromoImagesToAppData( JSON.stringify(PromoImages) ); //Save Promo images
					SavePromosToAppData(JSON.stringify(savepromos)); 
					SaveAppDataJson(); //SAVE DATA
				}
				else{
					if(globals.debugApp){
						console.log("error in Refresh Promos: " + error);
					}
					//have alert here..
				}

				
				setTimeout(function(){
					$scope.Headers = getAllHeadersWithPromos('refreshPromos()'); 

					setTimeout(function(){
						$scope.Promos = getAllPromos(0,'refreshPromos()');
							
						setTimeout(function(){	
							if(type == 0){ // type is to tell whether its for Prizes or Promos
								promoPageEvoke($scope, 1); //send 1 to trigger the evoke immediately
							}
							else{
								prizePageEvoke($scope, $rootScope, $timeout, $compile);
							}
						}, 1000);
					}, 100);
				}, 1000);	

				if(globals.debugApp){
					console.log("SUCCESS: Promos/StoreList/Services/StoreServices were refreshed");
				}
			}
			


			function downloadPromoImage(photo, counter, callback, errorcallback){ //helper function for downloading a promo image
				if(counter < newpromos.length){
					if(globals.debugApp){
						console.log('Debug: image download test [' + counter + '] - ' + photo[counter].PhotoFile ); 
					}

					$.ajax({
						url: GetItemFromLocalData('ClientURL'),
						type: "POST",
						dataType: "xml",
						timeout : 40000,
						data: soapMessageGetFileChunk('Image', photo[counter].PhotoFile), //this as well as the function updateGameAssets (in js/app.js) uses the GetFileChunk web method which returns the image as a base64 encoded string
						success: function(data){
							if(globals.debugApp){
								console.log("SUCCESS: A promo-image was downloaded via downloadpromoimage()");
							}
							saveClientLog(44, 0, 0, "SUCCESS: A promo-image was downloaded via downloadpromoimage()");

							var error = $(data).find('Standard').children('MethodErrorLevel').text();
							if (error == "None"){
								if (photo[counter].PhotoFile.substring(photo[counter].PhotoFile.indexOf('.') + 1) == 'png'){ //check if the photofile name is png or jpg and save accordingly
									PromoImages[photo[counter].PhotoFile] = "data:image/png;base64," + $(data).find('GetFileChunkResult').children('FilePart').text();
								}
								else{
									PromoImages[photo[counter].PhotoFile] = "data:image/jpeg;base64," + $(data).find('GetFileChunkResult').children('FilePart').text();
								}
								promoImagesDone++; //increments promoImagesDone which is used in the success call back to determine if all promo images have been downloaded
								
								if(globals.debugApp){
									console.log("Image Download successful (line1059/sync.js) : " + photo[counter].PhotoFile +  "  :  "+ promoImagesDone);
								}
							}
							else{
								if(globals.debugApp){
									console.log("ERROR: could not download promo image: "  + photo[counter].PhotoFile + " - Error = " + error);
								}
								saveClientLog(45, 0, 0, "ERROR: promo image downloaded but caused an error: "  + photo[counter].PhotoFile + " - Error = " + error);
							}

							var next = counter + 1;
							callback();

							downloadPromoImage(photo, next, callback, errorcallback);
						},
						contentType: "text/xml; charset=utf-8",
						error: function(data){
							//var next = counter + 1;
							if(globals.debugApp){
								console.log("error! in downloadPromoImage()");
								console.log("ERROR: could not connect to download promo image: "  + photo[counter].Photofile + " - Error = " + data);
							}
							saveClientLog(46, 0, 0, "ERROR: could not connect to download promo image: "  + photo[counter].Photofile + " - Error = " + data);
							errorcallback();
						}
					});
				}
			}

			function inPromos(promoid, promos){ //helper function to check if a promoid is in a promos array
				if (!promos){
					return false;
				}
				for (var i = 0; i < promos.length; i++){
					if (promos[i].PromoID == promoid){
						return true;
					}
				}
				return false;
			}

		},
		error: function(data){
			if(globals.debugApp){
				console.log("ERROR: in Refresh downloadPromo()");
			}
			saveClientLog(59, 0, 0, '', 0, 'ERROR: in Refresh downloadPromo()');

			setTimeout(function(){	
				screenRefresh($scope, $rootScope, $timeout, $compile);
			}, 1000);
		},
		contentType: "text/xml; charset=utf-8"
	});
}

function saveStoreData(){
	if (globals.TempStores){
		SaveStoresToAppData(globals.TempStores);
	}
	
	if (globals.TempStoreServices){
		SaveStoreServicesToAppData(globals.TempStoreServices);
	}
	
	if (globals.TempServices){
		SaveServicesToAppData( globals.TempServices );
	}
	
	SaveAppDataJson(); 

	if(globals.debugApp){
		console.log('SYNC-END - DEBUG : ## ALL STORE/SERVICES DATA was Saved..');
	}

	globals.TempStores = null;
	globals.TempServices = null;
	globals.TempStoreServices = null;
}


//Used in setupLocalData($rootScope) when a full download sync is flagged
function clearStoreAndPromoData(){
	DeletePromoImagesJson();
	DeletePromosJson();
	DeletePromoStoresJson();
	DeleteHeadersJson();
	DeleteStoresJson();
	DeleteStoreServicesJson();
	DeleteServicesJson();
	DeleteClientSetsJson();

	SaveAppDataJson();
}



function ClearDataAndLogOut(){
	DeleteAppData();
}




// FUNCTION DECRYPT ACCOUNT IF //
//helper function to decrypt accountid, used in js/soapMessages.js
function decryptAccountID() {
	if (globals.userAccount.get('AccountID') != ''){
		if(globals.debugApp){
			console.log('AccountID utilized : ' + sjcl.decrypt("AccountID", globals.userAccount.get('AccountID') ) );
		}

		return sjcl.decrypt("AccountID", globals.userAccount.get('AccountID') );
	}
	else{
		if(globals.debugApp){
			console.log('AccountID utilized : Empty');
		}
		return '';
	}
}



// FUNCTION SAVE ACCOUNT DETAIL //
//helper function to save save account details, used in js/app.js in web methods that return the account object as 
//part of the response (such as SyncStart, LinkAccount, CreateAccount, ChangeAccountDetail, GetAccountID) 
function saveAccountDetailJson(data){
	var AccountID = $(data).find("AccountID").text();
	var Email = $(data).find("Email").text();
	var FullName = $(data).find("FullName").text();
	var Phone = $(data).find("Phone").text();
	var UniqueID = parseInt($(data).find("UniqueID").text());
	var Points = parseInt($(data).find("Points").text());
	var GamePlays = parseInt($(data).find("GamePlays").text());
	var UserType = $(data).find("UserType").text();
	var UserStatus = $(data).find("UserStatus").text();
	var Birthday = $(data).find("Birthday").text();

	if(globals.debugApp){
		console.log("AccountID received from server : " + AccountID);
		console.log("Email received from server : " + Email);
		console.log("FullName received from server : " + FullName);
		console.log("Points received from server : " + Points);
		console.log("Gameplays received from server : " + GamePlays);
	}

	if (Birthday != "0001-01-01T00:00:00"){ //make sure the birthday is set
		var BirthdayTicks = ticksToEpoch(parseInt($(data).find("BirthdayTicks").text())); //see below for details about ticksToEpoch
	}
	else{
		BirthdayTicks = 0; //if the user has not specified a birthday, save birthday as 0
	}

	var GeoPromo = parseInt($(data).find("GeoPromo").text());
	var encryptedAccountID = sjcl.encrypt("AccountID", AccountID); //always encrypt AccountID before saving

	SetItemInLocalData('AccountID', encryptedAccountID);
	SetItemInLocalData('Email', Email);
	SetItemInLocalData('FullName', FullName);
	SetItemInLocalData('Phone', Phone);
	SetItemInLocalData('UniqueID', UniqueID);
	SetItemInLocalData('Points', Points);
	SetItemInLocalData('GamePlays', GamePlays);
	SetItemInLocalData('UserType', UserType);
	SetItemInLocalData('Birthday', Birthday);
	SetItemInLocalData('GeoPromo', GeoPromo);
	window.localStorage.setItem('FullName', FullName);

	saveInterests(data); // Function below
}


// FUNCTION: SAVE INTERESTS //
//helper function for saving the user's interests (returned as part of the Account object in the XML response)
function saveInterests(data){
	var Interests = [];
	$(data).find('RInterestServer').each(function(){
		var InterestID = $(this).children("InterestID").text();
		var Descript1 = $(this).children("Descript1").text();
		var Selected = parseInt($(this).children("Selected").text());
		Interests.push({
			InterestID:InterestID,
			Descript1:Descript1,
			Selected:Selected
		});
	});

	SetItemInLocalData('Interests', JSON.stringify(Interests) );
	saveLocalData();
}



//   FUNTION: SAVE CLIENT LOG   //
//  SAVE CLENTLOG IN LOCAL DATA //
function saveClientLog(logtype, promoid, storeid, beaconudid, beaconloc, comment){ //function used throughout the app to save client log, clientlogs are pushed to the server (see js/app/js) after every sync
	//these are helpful for monitoring errors. To query for errors on the server, search for client logs or type 1
	var soap = soapMessageClientLog(logtype, promoid, storeid, beaconudid, beaconloc, comment);
	//var clientlog = window.localStorage.getItem('ClientLog');
	var clientlog = GetItemFromLocalData('ClientLog');

	if (clientlog == null){
		SetItemInLocalData('ClientLog', soap );
	}
	else{
		clientlog = clientlog + soap;
		//window.localStorage.setItem('ClientLog', clientlog);
		SetItemInLocalData('ClientLog', clientlog );
	}
}


//this function is used throughout the app to reset the scrollbar when page content changes, it is important to
//reset the scroll bar everytime something shows or hides
function resetScrollbar(){
	try{
		$('.ui-content').iscrollview('refresh');
	}
	catch (err){}
}


// FUNCTION: ALERT //
//redefined the regular javascript alert method to call the native notification plugin and call a 
//custom callback function
function alert(message, callback){
	navigator.notification.alert(
	    message,  // message
	    function(){
	    	callback();
	    },         // callback
	    'Error',            // title
	    'Ok'                  // buttonName
	);
} 


function autoTruncate(length, classToFind){
	var len = length;
	var p = document.getElementsByClassName(classToFind).innerHTML;

	if(p){
		for(var i=0; i < p.length; i++){

	  		var trunc = p[i];
		  	if (trunc.length > len){
		    /* Truncate the content of the P, then go back to the end of the
		       previous word to ensure that we don't truncate in the middle of
		       a word */
		    trunc = trunc.substring(0, len);
		    trunc = trunc.replace(/\w+$/, '');
		    p[i] = trunc;
		   }
	    }
	}
}

function removeFromArray(arr, what) {
    var found = arr.indexOf(what);

    while (found !== -1) {
        arr.splice(found, 1);
        found = arr.indexOf(what);
    }
}




// FUNCTIONS EPOCH TO TICKS and TICKS TO EPOCH
//The below two helper functions convert between "Ticks" which are sent back from the server for Dates/Times (e.x. BirthdayTicks) 
//which represent C# ticks and "Epoch" which is Unix epoch time
function epochToTicks(e){
	return (e * 10000) + 621355968000000000;
}
function ticksToEpoch(t){
    return (t - 621355968000000000) / 10000;
}


function addOrd(n) { //helper method for the timestamp of a redemption
		var ords = [,'st','nd','rd'];
		var ord, m = n%100;
	return n + ((m > 10 && m < 14)? 'th' : ords[m%10] || 'th');
}
function getMonth(currentdate){ //helper method for the timestamp of a redemption
	var monthNames = ["January", "February", "March", "April", "May", "June", "July", "August", 
	"September", "October", "November", "December"];

	return monthNames[currentdate.getMonth()];
}
function getMinutes(n){ //helper method for the timestamp of a redemption
	if(n < 10){
		return '0' + n;
	}
	else{
		return n;
	}
}
function getTime(currentdate){ //helper method for the timestamp of a redemption
	if( currentdate.getHours() < 12 ){
		return currentdate.getHours() + ":" + getMinutes( currentdate.getMinutes() ) + "am"; 
	}
	else{
		if(currentdate.getHours() > 12){
			return ( currentdate.getHours() - 12 ) + ":" + getMinutes( currentdate.getMinutes() ) + "pm"; 
		}
		else{
			return currentdate.getHours()  + ":" + getMinutes( currentdate.getMinutes() ) + "pm"; 
		}
	}
}



